/* Menus and command line interface for Clustal W  */
/* DES was here MARCH. 1994 */
/* DES was here SEPT.  1994 */
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <signal.h>
#include <setjmp.h>
#include "clustalw.h"

static jmp_buf jmpbuf;
#ifndef VMS
#ifndef AIX
#define BADSIG (void (*)())-1
#endif
#endif

static void jumper(int);

static void jumper(int i)
{
        longjmp(jmpbuf,1);
}


/*
*	Prototypes
*/


static void pair_menu(QUICKPW_OPTPTR quickpw_opt,PW_OPTPTR pw_opt, Boolean dnaflag);
static void multi_menu(Boolean dnaflag,MULT_OPTPTR mult_opt);
static void gap_penalties_menu(MULTGAP_OPTPTR gap_opt);
static void multiple_align_menu(OPTPTR opt,ALN mult_aln);          /* multiple alignments menu */
static void motif_menu(OPTPTR opt,ALN mult_aln);          /* multiple alignments menu */
static void profile_align_menu(OPTPTR opt,ALN mult_aln);           /* profile       "      "   */
static void phylogenetic_tree_menu(ALN mult_aln,sint explicit_type,TREE_OPTPTR tree_opt,BSTREE_OPTPTR bstree_opt);
static void format_options_menu(ALN mult_aln,ALNOUT_OPTPTR alnout_opt,SS_OPTPTR ss_opt,sint nseqs);     /* format of alignment output */
static void tree_format_options_menu(TREE_OPTPTR tree_opt,BSTREE_OPTPTR bstree_opt); /* format of tree output */
static void ss_options_menu(SS_OPTPTR ss_opt);
static sint secstroutput_options(SS_OPTPTR ss_opt);
static sint treealgo_options(PW_OPTPTR pw_opt);
static sint set_matrix(char *title,char *userfile,MatMenu menu, char *matnam, sint matn);

/*
*	 Global variables
*/
static sint 	debug;

static Boolean 	save_parameters=FALSE;
static Boolean  amenus_active=FALSE;	/* set when main_menu first displayed to indicate
					that we're in interactive mode */

MatMenu dnamatrix_menu = {3,
                "IUB","iub",
                "CLUSTALW(1.6)","clustalw",
                "User defined",""
                };

MatMenu matrix_menu = {6,
                "BLOSUM series","blosum",
                "PAM series","pam",
                "Gonnet series","gonnet",
                "Bin series","bin",
                "Identity matrix","id",
                "User defined",""
                };

MatMenu pw_matrix_menu = {6,
                "BLOSUM 30","blosum",
                "PAM 350","pam",
                "Gonnet 250","gonnet",
                "Bin 30","bin",
                "Identity matrix","id",
                "User defined",""
                };


static char phylip_name[FILENAMELEN]="";
static char clustal_name[FILENAMELEN]="";
static char dist_name[FILENAMELEN]="";
static char bionj_name[FILENAMELEN]="";
static char nexus_name[FILENAMELEN]="";

static char *secstroutput_txt[] = {
				"Secondary Structure",
				"Gap Penalty Mask",
				"Structure and Penalty Mask",
				"None"	};
				                

static char *treealgo_txt[] = {
				"NJ",
				"BIONJ" };


void main_menu(OPTPTR opt,ALN mult_aln,char *revision_level)
{
        int catchint;
	static char lin1[MAXLINE+1];
	static char line[MAXLINE+1];

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }

	amenus_active=TRUE;

	while(TRUE) {
		fprintf(stdout,"\n\n\n");
		fprintf(stdout," **************************************************************\n");
		fprintf(stdout," ******** LeClustal %s Multiple Sequence Alignments   ********\n",revision_level);
		fprintf(stdout," **************************************************************\n");
		fprintf(stdout,"\n\n");
		
		fprintf(stdout,"     1. Sequence Input From Disc\n");
		fprintf(stdout,"     2. Multiple Alignments\n");
		fprintf(stdout,"     3. Profile / Structure Alignments\n");
		fprintf(stdout,"     4. Phylogenetic trees\n");
		fprintf(stdout,"\n");
		fprintf(stdout,"     S. Execute a system command\n");
		fprintf(stdout,"     H. HELP\n");
		fprintf(stdout,"     X. EXIT (leave program)\n\n\n");
		
		getstr("Your choice",lin1);

		switch(toupper(*lin1)) {
			case '1': 
fprintf(stdout,"\n\nSequences should all be in 1 file.\n");
fprintf(stdout,"\n7 formats accepted: \n");
fprintf(stdout,
"NBRF/PIR, EMBL/SwissProt, Pearson (Fasta), GDE, Clustal, GCG/MSF, RSF.\n\n\n");
                		getstr("Enter the name of the sequence file",line);
        			if(*line == EOS) break;
				seq_input(line,opt->explicit_type,FALSE,&mult_aln);
				phylip_name[0]=EOS;
				clustal_name[0]=EOS;
				dist_name[0]=EOS;
				bionj_name[0]=EOS;
				nexus_name[0]=EOS;
				break;
			case '2': multiple_align_menu(opt,mult_aln);
				break;
			case '3': profile_align_menu(opt,mult_aln);
				break;
			case '4': phylogenetic_tree_menu(mult_aln,opt->explicit_type,opt->tree_opt,opt->bstree_opt);
				break;
			case 'S': do_system();
				break;
			case '?':
			case 'H': get_help('1',TRUE);
				break;
			case 'Q':
			case 'X': exit(0);
				break;
			default: fprintf(stdout,"\n\nUnrecognised Command\n\n");
				break;
		}
	}
}









static void motif_menu(OPTPTR opt,ALN mult_aln)
{
        int catchint;
	static char lin1[MAXLINE+1];
	static char lin2[MAXLINE+1];

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


    while(TRUE)
    {
        fprintf(stdout,"\n\n\n");
        fprintf(stdout,"****** MOTIF PARAMETER MENU ******\n");
        fprintf(stdout,"\n\n");


        fprintf(stdout,"    1.  Make Ballast Motif File\n");
        fprintf(stdout,"    2.  Load motif file         ");
        if((*opt->mult_opt).motif_filename[0]!='\0') fprintf(stdout,": %s",(*opt->mult_opt).motif_filename);
	fprintf(stdout,"\n");
        fprintf(stdout,"    3.  Use motif file          : %s\n",
                                        (!(*opt->mult_opt).use_motifs) ? "OFF" : "ON");
        fprintf(stdout,"\n");

        fprintf(stdout,"    S.  Execute a system command\n");
        fprintf(stdout,"    H.  HELP\n");
        fprintf(stdout,"    or press [RETURN] to go back to main menu\n\n\n");

        getstr("Your choice",lin1);
        if(*lin1 == EOS) return;

        switch(toupper(*lin1))
        {
        case '1': align(&mult_aln,*opt,TRUE);
            break;
        case '2':
		fprintf(stdout,"Motif File Currently: %s\n",(*opt->mult_opt).motif_filename);

		getstr("Enter new filename (or [RETURN] to quit)",lin2);
                if (*lin2 != EOS) {
			strcpy((*opt->mult_opt).motif_filename,lin2);
                }
            break;
        case '3': (*opt->mult_opt).use_motifs ^= TRUE;
	    break;
        case 'S': do_system();
            break;
        case '?':
        case 'H': get_help('2',TRUE);
            break;
        case 'Q':
        case 'X': return;

        default: fprintf(stdout,"\n\nUnrecognised Command\n\n");
            break;
        }
    }
}







static void multiple_align_menu(OPTPTR opt,ALN mult_aln)
{
        int catchint;
	static char lin1[MAXLINE+1];

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


    while(TRUE)
    {
        fprintf(stdout,"\n\n\n");
        fprintf(stdout,"****** MULTIPLE ALIGNMENT MENU ******\n");
        fprintf(stdout,"\n\n");


        fprintf(stdout,"    1.  Do complete multiple alignment now (%s)\n",
                        (!(*opt->pw_opt).quick_pairalign) ? "Slow/Accurate" : "Fast/Approximate");
        fprintf(stdout,"    2.  Produce guide tree file only\n");
        fprintf(stdout,"    3.  Do alignment using old guide tree file\n\n");
        fprintf(stdout,"    4.  Toggle Slow/Fast pairwise alignments = %s\n\n",
                                        (!(*opt->pw_opt).quick_pairalign) ? "SLOW" : "FAST");
	fprintf(stdout,"    5.  Motif parameters\n");
        fprintf(stdout,"    6.  Pairwise alignment parameters\n");
        fprintf(stdout,"    7.  Multiple alignment parameters\n\n");
	fprintf(stdout,"    8.  Reset gaps before alignment?");
	if((*opt->mult_opt).reset_alignments_new)
		fprintf(stdout," = ON\n");
	else
		fprintf(stdout," = OFF\n");
        fprintf(stdout,"    9.  Toggle screen display          = %s\n",
                                        (!(*opt->alnout_opt).showaln) ? "OFF" : "ON");
        fprintf(stdout,"    0.  Output format options\n");
        fprintf(stdout,"\n");

        fprintf(stdout,"    S.  Execute a system command\n");
        fprintf(stdout,"    H.  HELP\n");
        fprintf(stdout,"    or press [RETURN] to go back to main menu\n\n\n");

        getstr("Your choice",lin1);
        if(*lin1 == EOS) return;

        switch(toupper(*lin1))
        {
        case '1': align(&mult_aln,*opt,TRUE);
		  create_alignment_output(mult_aln,0,mult_aln.nseqs,*opt->alnout_opt,*opt->ss_opt);
		if (save_parameters) create_parameter_output(*opt,mult_aln);
            break;
        case '2': make_tree(&mult_aln,*opt,TRUE);
		if (save_parameters) create_parameter_output(*opt,mult_aln);
            break;
        case '3': align_from_tree(&mult_aln,*opt,TRUE);
		if (save_parameters) create_parameter_output(*opt,mult_aln);
            break;
        case '4': (*opt->pw_opt).quick_pairalign ^= TRUE;
            break;
	case '5': motif_menu(opt,mult_aln);
	    break;
        case '6': pair_menu(opt->quickpw_opt,opt->pw_opt,mult_aln.dnaflag);
            break;
        case '7': multi_menu(mult_aln.dnaflag,opt->mult_opt);
            break;
	case '8': (*opt->mult_opt).reset_alignments_new ^= TRUE;
	    if((*opt->mult_opt).reset_alignments_new==TRUE)
		(*opt->mult_opt).reset_alignments_all=FALSE;
            break;
        case '9': (*opt->alnout_opt).showaln ^= TRUE;
	    break;
        case '0': format_options_menu(mult_aln,opt->alnout_opt,opt->ss_opt,mult_aln.nseqs);
            break;
        case 'S': do_system();
            break;
        case '?':
        case 'H': get_help('2',TRUE);
            break;
        case 'Q':
        case 'X': return;

        default: fprintf(stdout,"\n\nUnrecognised Command\n\n");
            break;
        }
    }
}









static void profile_align_menu(OPTPTR opt,ALN mult_aln)
{
        int catchint;
	static char lin1[MAXLINE+1];
	static char line[MAXLINE+1];


        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


    while(TRUE)
    {
	fprintf(stdout,"\n\n\n");
        fprintf(stdout,"****** PROFILE AND STRUCTURE ALIGNMENT MENU ******\n");
        fprintf(stdout,"\n\n");

        fprintf(stdout,"    1.  Input 1st. profile             ");
        if (mult_aln.prf1.nseqs>0) fprintf(stdout,"(loaded)");
        fprintf(stdout,"\n");
        fprintf(stdout,"    2.  Input 2nd. profile/sequences   ");
        if (mult_aln.nseqs>mult_aln.prf1.nseqs) fprintf(stdout,"(loaded)");
        fprintf(stdout,"\n\n");
        fprintf(stdout,"    3.  Align 2nd. profile to 1st. profile\n");
        fprintf(stdout,"    4.  Align sequences to 1st. profile (%s)\n\n",
                        (!(*opt->pw_opt).quick_pairalign) ? "Slow/Accurate" : "Fast/Approximate");
        fprintf(stdout,"    5.  Toggle Slow/Fast pairwise alignments = %s\n\n",
                                        (!(*opt->pw_opt).quick_pairalign) ? "SLOW" : "FAST");
        fprintf(stdout,"    6.  Pairwise alignment parameters\n");
        fprintf(stdout,"    7.  Multiple alignment parameters\n\n");
        fprintf(stdout,"    8.  Toggle screen display                = %s\n",
                                        (!(*opt->alnout_opt).showaln) ? "OFF" : "ON");
        fprintf(stdout,"    9.  Output format options\n");
        fprintf(stdout,"    0.  Secondary structure options\n");
        fprintf(stdout,"\n");
        fprintf(stdout,"    S.  Execute a system command\n");
        fprintf(stdout,"    H.  HELP\n");
        fprintf(stdout,"    or press [RETURN] to go back to main menu\n\n\n");

        getstr("Your choice",lin1);
        if(*lin1 == EOS) return;

        switch(toupper(*lin1))
        {
        case '1': 
	  getstr("Enter the name of the sequence file",line);
       	  if(*line == EOS) break;
          profile_input(line,1,opt->explicit_type,&mult_aln);
            break;
        case '2': 
	  getstr("Enter the name of the sequence file",line);
       	  if(*line == EOS) break;
          profile_input(line,2,opt->explicit_type,&mult_aln);
            break;
        case '3': profile_align(&mult_aln,*opt,FALSE,FALSE,TRUE);       /* align the 2 alignments now */
		if (save_parameters) create_parameter_output(*opt,mult_aln);
            break;
        case '4': new_sequence_align(&mult_aln,*opt,FALSE,TRUE);  /* align new sequences to profile 1 */
		if (save_parameters) create_parameter_output(*opt,mult_aln);
            break;
        case '5': (*opt->pw_opt).quick_pairalign ^= TRUE;
	    break;
        case '6': pair_menu(opt->quickpw_opt,opt->pw_opt,mult_aln.dnaflag);
            break;
        case '7': multi_menu(mult_aln.dnaflag,opt->mult_opt);
            break;
        case '8': (*opt->alnout_opt).showaln ^= TRUE;
	    break;
        case '9': format_options_menu(mult_aln,opt->alnout_opt,opt->ss_opt,mult_aln.nseqs);
            break;
        case '0': ss_options_menu(opt->ss_opt);
            break;
        case 'S': do_system();
            break;
        case '?':
        case 'H': get_help('6',TRUE);
            break;
        case 'Q':
        case 'X': return;

        default: fprintf(stdout,"\n\nUnrecognised Command\n\n");
            break;
        }
    }
}


static void ss_options_menu(SS_OPTPTR ss_opt)
{
        int catchint;
	static char lin1[MAXLINE+1];

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


	while(TRUE) {
	
		fprintf(stdout,"\n\n\n");
		fprintf(stdout," ********* SECONDARY STRUCTURE OPTIONS *********\n");
		fprintf(stdout,"\n\n");

		fprintf(stdout,"     1. Use profile 1 secondary structure / penalty mask  ");
		if(ss_opt->use_ss1)
			fprintf(stdout,"= YES\n");
		else
			fprintf(stdout,"= NO\n");
		fprintf(stdout,"     2. Use profile 2 secondary structure / penalty mask  ");
		if(ss_opt->use_ss2)
			fprintf(stdout,"= YES\n");
		else
			fprintf(stdout,"= NO\n");
		fprintf(stdout,"\n");
		fprintf(stdout,"     3. Output in alignment  ");
		fprintf(stdout,"= %s\n",secstroutput_txt[ss_opt->output_struct_penalties]);
		fprintf(stdout,"\n");

		fprintf(stdout,"     4. Helix gap penalty                     :%d\n",(pint)ss_opt->helix_penalty);
		fprintf(stdout,"     5. Strand gap penalty                    :%d\n",(pint)ss_opt->strand_penalty);
		fprintf(stdout,"     6. Loop gap penalty                      :%d\n",(pint)ss_opt->loop_penalty);

		fprintf(stdout,"     7. Secondary structure terminal penalty  :%d\n",(pint)ss_opt->helix_end_penalty);
		fprintf(stdout,"     8. Helix terminal positions       within :%d      outside :%d\n",
		                                 (pint)ss_opt->helix_end_minus,(pint)ss_opt->helix_end_plus);
		fprintf(stdout,"     9. Strand terminal positions      within :%d      outside :%d\n",
		                                 (pint)ss_opt->strand_end_minus,(pint)ss_opt->strand_end_plus);

		fprintf(stdout,"\n\n");
		fprintf(stdout,"     H. HELP\n\n\n");
		
		getstr("Enter number (or [RETURN] to exit)",lin1);
		if( *lin1 == EOS) return;
		
		switch(toupper(*lin1)) {
			case '1': ss_opt->use_ss1 ^= TRUE;
				break;
			case '2': ss_opt->use_ss2 ^= TRUE;
				break;
			case '3': ss_opt->output_struct_penalties = secstroutput_options(ss_opt);
				break;
			case '4':
				fprintf(stdout,"Helix Penalty Currently: %d\n",(pint)ss_opt->helix_penalty);
				ss_opt->helix_penalty=getint("Enter number",1,9,ss_opt->helix_penalty);
				break;
			case '5':
				fprintf(stdout,"Strand Gap Penalty Currently: %d\n",(pint)ss_opt->strand_penalty);
				ss_opt->strand_penalty=getint("Enter number",1,9,ss_opt->strand_penalty);
				break;
			case '6':
				fprintf(stdout,"Loop Gap Penalty Currently: %d\n",(pint)ss_opt->loop_penalty);
				ss_opt->loop_penalty=getint("Enter number",1,9,ss_opt->loop_penalty);
				break;
			case '7':
				fprintf(stdout,"Secondary Structure Terminal Penalty Currently: %d\n",
				          (pint)ss_opt->helix_end_penalty);
				ss_opt->helix_end_penalty=getint("Enter number",1,9,ss_opt->helix_end_penalty);
				ss_opt->strand_end_penalty = ss_opt->helix_end_penalty;
				break;
			case '8':
				fprintf(stdout,"Helix Terminal Positions Currently: \n");
				fprintf(stdout,"        within helix: %d     outside helix: %d\n",
				                            (pint)ss_opt->helix_end_minus,(pint)ss_opt->helix_end_plus);
				ss_opt->helix_end_minus=getint("Enter number of residues within helix",0,3,ss_opt->helix_end_minus);
				ss_opt->helix_end_plus=getint("Enter number of residues outside helix",0,3,ss_opt->helix_end_plus);
				break;
			case '9':
				fprintf(stdout,"Strand Terminal Positions Currently: \n");
				fprintf(stdout,"        within strand: %d     outside strand: %d\n",
				                            (pint)ss_opt->strand_end_minus,(pint)ss_opt->strand_end_plus);
				ss_opt->strand_end_minus=getint("Enter number of residues within strand",0,3,ss_opt->strand_end_minus);
				ss_opt->strand_end_plus=getint("Enter number of residues outside strand",0,3,ss_opt->strand_end_plus);
				break;
			case '?':
			case 'H':
				get_help('B',TRUE);
				break;
			default:
				fprintf(stdout,"\n\nUnrecognised Command\n\n");
				break;
		}
	}
}


static sint treealgo_options(PW_OPTPTR pw_opt)
{

	static char lin1[MAXLINE+1];

        while(TRUE)
        {
                fprintf(stdout,"\n\n\n");
                fprintf(stdout," ********* Guide Tree Algorithm *********\n");
                fprintf(stdout,"\n\n");


                fprintf(stdout,"     1. %s\n",treealgo_txt[0]);
                fprintf(stdout,"     2. %s\n",treealgo_txt[1]);
                fprintf(stdout,"     H. HELP\n\n");
                fprintf(stdout,
"     -- Current algorithm is %s ",treealgo_txt[pw_opt->treealgo]);
                fprintf(stdout,"--\n");


                getstr("\n\nEnter number (or [RETURN] to exit)",lin1);
                if(*lin1 == EOS) return(pw_opt->treealgo);

        	switch(toupper(*lin1))
        	{
       	 		case '1': return(0);
        		case '2': return(1);
			case '?': 
        		case 'H': get_help('C',TRUE);
            		case 'Q':
        		case 'X': return(0);

        		default: fprintf(stdout,"\n\nUnrecognised Command\n\n");
            		break;
        	}
        }
}



static sint secstroutput_options(SS_OPTPTR ss_opt)
{

	static char lin1[MAXLINE+1];

        while(TRUE)
        {
                fprintf(stdout,"\n\n\n");
                fprintf(stdout," ********* Secondary Structure Output Menu *********\n");
                fprintf(stdout,"\n\n");


                fprintf(stdout,"     1. %s\n",secstroutput_txt[0]);
                fprintf(stdout,"     2. %s\n",secstroutput_txt[1]);
                fprintf(stdout,"     3. %s\n",secstroutput_txt[2]);
                fprintf(stdout,"     4. %s\n",secstroutput_txt[3]);
                fprintf(stdout,"     H. HELP\n\n");
                fprintf(stdout,
"     -- Current output is %s ",secstroutput_txt[ss_opt->output_struct_penalties]);
                fprintf(stdout,"--\n");


                getstr("\n\nEnter number (or [RETURN] to exit)",lin1);
                if(*lin1 == EOS) return(ss_opt->output_struct_penalties);

        	switch(toupper(*lin1))
        	{
       	 		case '1': return(0);
        		case '2': return(1);
      			case '3': return(2);
        		case '4': return(3);
			case '?': 
        		case 'H': get_help('C',TRUE);
            		case 'Q':
        		case 'X': return(0);

        		default: fprintf(stdout,"\n\nUnrecognised Command\n\n");
            		break;
        	}
        }
}







static void phylogenetic_tree_menu(ALN mult_aln,sint explicit_type,TREE_OPTPTR tree_opt,BSTREE_OPTPTR bstree_opt)
{
        int catchint;
	static char lin1[MAXLINE+1];
        char line[FILENAMELEN+1];

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


    while(TRUE)
    {
        fprintf(stdout,"\n\n\n");
        fprintf(stdout,"****** PHYLOGENETIC TREE MENU ******\n");
        fprintf(stdout,"\n\n");

        fprintf(stdout,"    1.  Input an alignment\n");
        fprintf(stdout,"    2.  Exclude positions with gaps?        ");
	if(tree_opt->tossgaps)
		fprintf(stdout,"= ON\n");
	else
		fprintf(stdout,"= OFF\n");
        fprintf(stdout,"    3.  Correct for multiple substitutions? ");
	if(tree_opt->kimura)
		fprintf(stdout,"= ON\n");
	else
		fprintf(stdout,"= OFF\n");
        fprintf(stdout,"    4.  Draw tree now\n");
        fprintf(stdout,"    5.  Bootstrap tree\n");
	fprintf(stdout,"    6.  Output format options\n");
        fprintf(stdout,"\n");
        fprintf(stdout,"    S.  Execute a system command\n");
        fprintf(stdout,"    H.  HELP\n");
        fprintf(stdout,"    or press [RETURN] to go back to main menu\n\n\n");

        getstr("Your choice",lin1);
        if(*lin1 == EOS) return;

        switch(toupper(*lin1))
        {
       	 	case '1':
fprintf(stdout,"\n\nSequences should all be in 1 file.\n");
fprintf(stdout,"\n7 formats accepted: \n");
fprintf(stdout,
"NBRF/PIR, EMBL/SwissProt, Pearson (Fasta), GDE, Clustal, GCG/MSF, RSF.\n\n\n");
                		getstr("Enter the name of the sequence file",line);
        			if(*line == EOS) break;
				seq_input(line,explicit_type,FALSE,&mult_aln);
				phylip_name[0]=EOS;
				clustal_name[0]=EOS;
				dist_name[0]=EOS;
				bionj_name[0]=EOS;
				nexus_name[0]=EOS;
         	   	break;
        	case '2': tree_opt->tossgaps ^= TRUE;
          	  	break;
      		case '3': tree_opt->kimura ^= TRUE;;
            		break;
        	case '4': phylogenetic_tree(mult_aln,phylip_name,clustal_name,dist_name,bionj_name,nexus_name,TRUE,*tree_opt);
            		break;
        	case '5': bootstrap_tree(mult_aln,phylip_name,clustal_name,nexus_name,TRUE,*tree_opt,*bstree_opt);
            		break;
		case '6': tree_format_options_menu(tree_opt,bstree_opt);
			break;
        	case 'S': do_system();
            		break;
            	case '?':
        	case 'H': get_help('7',TRUE);
            		break;
            	case 'Q':
        	case 'X': return;

        	default: fprintf(stdout,"\n\nUnrecognised Command\n\n");
            	break;
        }
    }
}






static void tree_format_options_menu(TREE_OPTPTR tree_opt,BSTREE_OPTPTR bstree_opt) /* format of tree output */
{	
        int catchint;
	static char lin1[MAXLINE+1];

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


	while(TRUE) {
	fprintf(stdout,"\n\n\n");
	fprintf(stdout," ****** Format of Phylogenetic Tree Output ******\n");
	fprintf(stdout,"\n\n");
	fprintf(stdout,"     1. Toggle CLUSTAL format tree output    =  %s\n",
					(!tree_opt->output_tree_clustal)  ? "OFF" : "ON");
	fprintf(stdout,"     2. Toggle Phylip format tree output     =  %s\n",
					(!tree_opt->output_tree_phylip)   ? "OFF" : "ON");
	fprintf(stdout,"     3. Toggle Phylip distance matrix output =  %s\n",
					(!tree_opt->output_tree_distances)? "OFF" : "ON");
	fprintf(stdout,"     4. Toggle BioNJ format tree output      =  %s\n",
					(!tree_opt->output_tree_bionj)? "OFF" : "ON");
	fprintf(stdout,"     5. Toggle Nexus format tree output      =  %s\n\n",
					(!tree_opt->output_tree_nexus)? "OFF" : "ON");
	fprintf(stdout,"     6. Toggle Phylip bootstrap positions    =  %s\n\n",
(bstree_opt->bootstrap_format==BS_NODE_LABELS) ? "NODE LABELS" : "BRANCH LABELS");
	fprintf(stdout,"\n");
	fprintf(stdout,"     H. HELP\n\n\n");	
	
		getstr("Enter number (or [RETURN] to exit)",lin1);
		if(*lin1 == EOS) return;
		
		switch(toupper(*lin1)) {
			case '1':
				tree_opt->output_tree_clustal   ^= TRUE;
				break;
			case '2':
              			tree_opt->output_tree_phylip    ^= TRUE;
			  	break;
			case '3':
              			tree_opt->output_tree_distances ^= TRUE;
			  	break;
			case '4':
              			tree_opt->output_tree_bionj ^= TRUE;
			  	break;
			case '5':
              			tree_opt->output_tree_nexus ^= TRUE;
			  	break;
			case '6':
              			if (bstree_opt->bootstrap_format == BS_NODE_LABELS)
					bstree_opt->bootstrap_format = BS_BRANCH_LABELS;
				else
					bstree_opt->bootstrap_format = BS_NODE_LABELS;
			  	break;
			case '?':
			case 'H':
				get_help('0',TRUE);
				break;
			default:
				fprintf(stdout,"\n\nUnrecognised Command\n\n");
				break;
		}
	}
}


static void format_options_menu(ALN mult_aln,ALNOUT_OPTPTR alnout_opt,SS_OPTPTR ss_opt,sint nseqs)      /* format of alignment output */
{	
	char path[FILENAMELEN+1];
	static char lin1[MAXLINE+1];
    int catchint;

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


	while(TRUE) {
	fprintf(stdout,"\n\n\n");
	fprintf(stdout," ********* Format of Alignment Output *********\n");
	fprintf(stdout,"\n\n");
	fprintf(stdout,"     1. Toggle CLUSTAL format output     =  %s\n",
					(!alnout_opt->output_clustal) ? "OFF" : "ON");
	fprintf(stdout,"     2. Toggle NBRF/PIR format output    =  %s\n",
					(!alnout_opt->output_nbrf) ? "OFF" : "ON");
	fprintf(stdout,"     3. Toggle GCG/MSF format output     =  %s\n",
					(!alnout_opt->output_gcg) ? "OFF" : "ON");
	fprintf(stdout,"     4. Toggle PHYLIP format output      =  %s\n",
					(!alnout_opt->output_phylip) ? "OFF" : "ON");
	fprintf(stdout,"     5. Toggle GDE format output         =  %s\n\n",
					(!alnout_opt->output_gde) ? "OFF" : "ON");
	fprintf(stdout,"     6. Toggle GDE output case           =  %s\n",
					(!alnout_opt->lowercase) ? "UPPER" : "LOWER");
	fprintf(stdout,"     7. Toggle CLUSTALW sequence numbers =  %s\n",
					(!alnout_opt->seq_numbers) ? "OFF" : "ON");
	fprintf(stdout,"     8. Toggle output order              =  %s\n\n",
					(alnout_opt->output_order==0) ? "INPUT FILE" : "ALIGNED");
	fprintf(stdout,"     9. Create alignment output file(s) now?\n\n");
	fprintf(stdout,"     0. Toggle parameter output          = %s\n",
					(!save_parameters) ? "OFF" : "ON");
	fprintf(stdout,"\n");
	fprintf(stdout,"     H. HELP\n\n\n");	
	
		getstr("Enter number (or [RETURN] to exit)",lin1);
		if(*lin1 == EOS) return;
		
		switch(toupper(*lin1)) {
			case '1':
				alnout_opt->output_clustal ^= TRUE;
				break;
			case '2':
              			alnout_opt->output_nbrf ^= TRUE;
			  	break;
			case '3':
              			alnout_opt->output_gcg ^= TRUE;
			  	break;
			case '4':
              			alnout_opt->output_phylip ^= TRUE;
			  	break;
			case '5':
              			alnout_opt->output_gde ^= TRUE;
			  	break;
			case '6':
              			alnout_opt->lowercase ^= TRUE;
			  	break;
			case '7':
              			alnout_opt->seq_numbers ^= TRUE;
			  	break;
			case '8':
                                if (alnout_opt->output_order == INPUT) alnout_opt->output_order = ALIGNED;
              			else alnout_opt->output_order = INPUT;
			  	break;
			case '9':		/* DES */
				if(nseqs<=0) {
					error("No sequences loaded");
					break;
				}
				if(!open_alignment_output(mult_aln.filename,alnout_opt)) break;
				create_alignment_output(mult_aln,0,nseqs-1,*alnout_opt,*ss_opt);
				break;
        		case '0': save_parameters ^= TRUE;
	   			 break;
			case '?':
			case 'H':
				get_help('5',TRUE);
				break;
			default:
				fprintf(stdout,"\n\nUnrecognised Command\n\n");
				break;
		}
	}
}












static void pair_menu(QUICKPW_OPTPTR quickpw_opt,PW_OPTPTR pw_opt, Boolean dnaflag)
{
        int catchint;
	float    pw_go_penalty,      pw_ge_penalty;
	static char lin1[MAXLINE+1];
	sint    wind_gap,ktup,window,signif;

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


        if(dnaflag) {
                pw_go_penalty     = pw_opt->dna_go_penalty;
                pw_ge_penalty     = pw_opt->dna_ge_penalty;
                ktup       = quickpw_opt->dna_ktup;
                window     = quickpw_opt->dna_window;
                signif     = quickpw_opt->dna_signif;
                wind_gap   = quickpw_opt->dna_wind_gap;

        }
        else {
                pw_go_penalty     = pw_opt->prot_go_penalty;
                pw_ge_penalty     = pw_opt->prot_ge_penalty;
                ktup       = quickpw_opt->prot_ktup;
                window     = quickpw_opt->prot_window;
                signif     = quickpw_opt->prot_signif;
                wind_gap   = quickpw_opt->prot_wind_gap;

        }

	while(TRUE) {
	
		fprintf(stdout,"\n\n\n");
		fprintf(stdout," ********* PAIRWISE ALIGNMENT PARAMETERS *********\n");
		fprintf(stdout,"\n\n");

		fprintf(stdout,"     Slow/Accurate alignments:\n\n");

		fprintf(stdout,"     1. Gap Open Penalty       :%4.2f\n",pw_go_penalty);
		fprintf(stdout,"     2. Gap Extension Penalty  :%4.2f\n",pw_ge_penalty);
		fprintf(stdout,"     3. Protein weight matrix  :%s\n" ,
                                        pw_matrix_menu.opt[pw_opt->matnum-1].title);
		fprintf(stdout,"     4. DNA weight matrix      :%s\n" ,
                                        dnamatrix_menu.opt[pw_opt->dnamatnum-1].title);
		fprintf(stdout,"\n");

		fprintf(stdout,"     Fast/Approximate alignments:\n\n");

		fprintf(stdout,"     5. Gap penalty            :%d\n",(pint)wind_gap);
		fprintf(stdout,"     6. K-tuple (word) size    :%d\n",(pint)ktup);
		fprintf(stdout,"     7. No. of top diagonals   :%d\n",(pint)signif);
		fprintf(stdout,"     8. Window size            :%d\n\n",(pint)window);

                fprintf(stdout,"     9. Toggle Slow/Fast pairwise alignments ");
                if(pw_opt->quick_pairalign)
                      fprintf(stdout,"= FAST\n\n");
                else
                      fprintf(stdout,"= SLOW\n\n");
		fprintf(stdout,"     0. Guide tree algorithm = %s\n\n",
					treealgo_txt[pw_opt->treealgo]);


		fprintf(stdout,"     H. HELP\n\n\n");
		
		getstr("Enter number (or [RETURN] to exit)",lin1);
		if( *lin1 == EOS) {
                        if(dnaflag) {
                                pw_opt->dna_go_penalty     = pw_go_penalty;
                                pw_opt->dna_ge_penalty     = pw_ge_penalty;
                		quickpw_opt->dna_ktup       = ktup;
                		quickpw_opt->dna_window     = window;
                		quickpw_opt->dna_signif     = signif;
                		quickpw_opt->dna_wind_gap   = wind_gap;

                        }
                        else {
                                pw_opt->prot_go_penalty     = pw_go_penalty;
                                pw_opt->prot_ge_penalty     = pw_ge_penalty;
                		quickpw_opt->prot_ktup       = ktup;
                		quickpw_opt->prot_window     = window;
                		quickpw_opt->prot_signif     = signif;
                		quickpw_opt->prot_wind_gap   = wind_gap;

                        }
 
			return;
		}
		
		switch(toupper(*lin1)) {
			case '1':
				fprintf(stdout,"Gap Open Penalty Currently: %4.2f\n",pw_go_penalty);
				pw_go_penalty=(float)getreal("Enter number",(double)0.0,(double)100.0,(double)pw_go_penalty);
				break;
			case '2':
				fprintf(stdout,"Gap Extension Penalty Currently: %4.2f\n",pw_ge_penalty);
				pw_ge_penalty=(float)getreal("Enter number",(double)0.0,(double)10.0,(double)pw_ge_penalty);
				break;
                        case '3':
                                pw_opt->matnum = set_matrix("PROTEIN",pw_opt->usermtrxname,pw_matrix_menu,pw_opt->mtrxname,pw_opt->matnum);
                                break;
                        case '4':
                                pw_opt->dnamatnum = set_matrix("DNA",pw_opt->dnausermtrxname,dnamatrix_menu,pw_opt->dnamtrxname,pw_opt->dnamatnum);
                                break;
			case '5':
                                fprintf(stdout,"Gap Penalty Currently: %d\n",(pint)wind_gap);
                                wind_gap=getint("Enter number",1,500,wind_gap);
				break;
			case '6':
                                fprintf(stdout,"K-tuple Currently: %d\n",(pint)ktup);
                                if(dnaflag)
                                     ktup=getint("Enter number",1,4,ktup);
                                else
                                     ktup=getint("Enter number",1,2,ktup);                                     
				break;
			case '7':
                                fprintf(stdout,"Top diagonals Currently: %d\n",(pint)signif);
                                signif=getint("Enter number",1,50,signif);
				break;
			case '8':
                                fprintf(stdout,"Window size Currently: %d\n",(pint)window);
                                window=getint("Enter number",1,50,window);
				break;
                        case '9': pw_opt->quick_pairalign ^= TRUE;
                                break;
			case '0': pw_opt->treealgo = treealgo_options(pw_opt);
				break;
			case '?':
			case 'H':
				get_help('3',TRUE);
				break;
			default:
				fprintf(stdout,"\n\nUnrecognised Command\n\n");
				break;
		}
	}
}





static void multi_menu(Boolean dnaflag,MULT_OPTPTR mult_opt)
{
        int catchint;
	float    gap_open,      gap_extend;
	static char lin1[MAXLINE+1];

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


	if(dnaflag) {
		gap_open   = mult_opt->dna_gap_open;
		gap_extend = mult_opt->dna_gap_extend;
	}
	else {
		gap_open   = mult_opt->prot_gap_open;
		gap_extend = mult_opt->prot_gap_extend;
	}

	while(TRUE) {

		fprintf(stdout,"\n\n\n");
		fprintf(stdout," ********* MULTIPLE ALIGNMENT PARAMETERS *********\n");
		fprintf(stdout,"\n\n");
		
		fprintf(stdout,"     1. Gap Opening Penalty              :%4.2f\n",gap_open);
		fprintf(stdout,"     2. Gap Extension Penalty            :%4.2f\n",gap_extend);

		fprintf(stdout,"     3. Delay divergent sequences        :%d %%\n\n",(pint)mult_opt->divergence_cutoff);

                fprintf(stdout,"     4. DNA Transitions Weight           :%1.2f\n\n",mult_opt->transition_weight);
                fprintf(stdout,"     5. Protein weight matrix            :%s\n"
                                        	,matrix_menu.opt[mult_opt->matnum-1].title);
                fprintf(stdout,"     6. DNA weight matrix                :%s\n"
                                        	,dnamatrix_menu.opt[mult_opt->dnamatnum-1].title);
		fprintf(stdout,"     7. Use negative matrix              :%s\n\n",(!mult_opt->neg_matrix) ? "OFF" : "ON");
                fprintf(stdout,"     8. Protein Gap Parameters\n\n");
		fprintf(stdout,"     H. HELP\n\n\n");		

		getstr("Enter number (or [RETURN] to exit)",lin1);

		if(*lin1 == EOS) {
			if(dnaflag) {
				mult_opt->dna_gap_open    = gap_open;
				mult_opt->dna_gap_extend  = gap_extend;
			}
			else {
				mult_opt->prot_gap_open   = gap_open;
				mult_opt->prot_gap_extend = gap_extend;
			}
			return;
		}
		
		switch(toupper(*lin1)) {
			case '1':
			fprintf(stdout,"Gap Opening Penalty Currently: %4.2f\n",gap_open);
				gap_open=(float)getreal("Enter number",(double)0.0,(double)100.0,(double)gap_open);
				break;
			case '2':
				fprintf(stdout,"Gap Extension Penalty Currently: %4.2f\n",gap_extend);
				gap_extend=(float)getreal("Enter number",(double)0.0,(double)10.0,(double)gap_extend);
				break;
			case '3':
				fprintf(stdout,"Min Identity Currently: %d\n",(pint)mult_opt->divergence_cutoff);
				mult_opt->divergence_cutoff=getint("Enter number",0,100,mult_opt->divergence_cutoff);
				break;
			case '4':
				fprintf(stdout,"Transition Weight Currently: %1.2f\n",(pint)mult_opt->transition_weight);
				mult_opt->transition_weight=(float)getreal("Enter number",(double)0.0,(double)1.0,(double)mult_opt->transition_weight);
				break;
			case '5':
                                mult_opt->matnum = set_matrix("PROTEIN",mult_opt->usermtrxname,matrix_menu,mult_opt->mtrxname,mult_opt->matnum);
				break;
			case '6':
                                mult_opt->dnamatnum = set_matrix("DNA",mult_opt->dnausermtrxname,dnamatrix_menu,mult_opt->dnamtrxname,mult_opt->dnamatnum);
				break;
			case '7':
				mult_opt->neg_matrix ^= TRUE;
				break;
			case '8':
                                gap_penalties_menu(mult_opt->gap_opt);
				break;
			case '?':
			case 'H':
				get_help('4',TRUE);
				break;
			default:
				fprintf(stdout,"\n\nUnrecognised Command\n\n");
				break;
		}
	}
}






static void gap_penalties_menu(MULTGAP_OPTPTR gap_opt)
{
	char c;
	sint i;
        int catchint;
	static char lin1[MAXLINE+1];
	static char lin2[MAXLINE+1];

        catchint = signal(SIGINT, SIG_IGN) != SIG_IGN;
        if (catchint) {
                if (setjmp(jmpbuf) != 0)
                        fprintf(stdout,"\n.. Interrupt\n");
#ifdef UNIX
                if (signal(SIGINT,jumper) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#else
                if (signal(SIGINT,SIG_DFL) == BADSIG)
                        fprintf(stdout,"Error: signal\n");
#endif
        }


	while(TRUE) {

		fprintf(stdout,"\n\n\n");
		fprintf(stdout," ********* PROTEIN GAP PARAMETERS *********\n");
		fprintf(stdout,"\n\n\n");

		fprintf(stdout,"     1. Toggle Residue-Specific Penalties :%s\n\n",(gap_opt->no_pref_penalties) ? "OFF" : "ON");
		fprintf(stdout,"     2. Toggle Hydrophilic Penalties      :%s\n",(gap_opt->no_hyd_penalties) ? "OFF" : "ON");
		fprintf(stdout,"     3. Hydrophilic Residues              :%s\n\n"
					,gap_opt->hyd_residues);
		fprintf(stdout,"     4. Toggle Variable Zone Penalties    :%s\n",(gap_opt->no_var_penalties) ? "OFF" : "ON");
		fprintf(stdout,"     5. Gap Separation Distance           :%d\n",(pint)gap_opt->gap_dist);
		fprintf(stdout,"     6. Toggle End Gap Separation         :%s\n\n",(!gap_opt->use_endgaps) ? "OFF" : "ON");
		fprintf(stdout,"     H. HELP\n\n\n");		

		getstr("Enter number (or [RETURN] to exit)",lin1);

		if(*lin1 == EOS) return;
		
		switch(toupper(*lin1)) {
			case '1':
				gap_opt->no_pref_penalties ^= TRUE;
				break;
			case '2':
				gap_opt->no_hyd_penalties ^= TRUE;
				break;
			case '3':
				fprintf(stdout,"Hydrophilic Residues Currently: %s\n",gap_opt->hyd_residues);

				getstr("Enter residues (or [RETURN] to quit)",lin2);
                                if (*lin2 != EOS) {
                                        for (i=0;i<strlen(gap_opt->hyd_residues) && i<26;i++) {
                                        c = lin2[i];
                                        if (isalpha(c))
                                                gap_opt->hyd_residues[i] = (char)toupper(c);
                                        else
                                                break;
                                        }
                                        gap_opt->hyd_residues[i] = EOS;
                                }
                                break;
			case '4':
				gap_opt->no_var_penalties ^= TRUE;
				break;
			case '5':
				fprintf(stdout,"Gap Separation Distance Currently: %d\n",(pint)gap_opt->gap_dist);
				gap_opt->gap_dist=getint("Enter number",0,100,gap_opt->gap_dist);
				break;
			case '6':
				gap_opt->use_endgaps ^= TRUE;
				break;
			case '?':
			case 'H':
				get_help('A',TRUE);
				break;
			default:
				fprintf(stdout,"\n\nUnrecognised Command\n\n");
				break;
		}
	}
}




static sint set_matrix(char *title,char *userfile,MatMenu menu, char *matnam, sint matn)
{
	int i;
	static char lin1[MAXLINE+1];

        while(TRUE)
        {
                fprintf(stdout,"\n\n\n");
                fprintf(stdout," ********* %s WEIGHT MATRIX MENU *********\n",title);
                fprintf(stdout,"\n\n");

		for(i=0;i<menu.noptions;i++)
                	fprintf(stdout,"     %d. %s\n",i+1,menu.opt[i].title);
                fprintf(stdout,"     H. HELP\n\n");
                fprintf(stdout,
"     -- Current matrix is the %s ",menu.opt[matn-1].title);
                if(matn == menu.noptions) fprintf(stdout,"(file = %s)",userfile);
                fprintf(stdout,"--\n");


                getstr("\n\nEnter number (or [RETURN] to exit)",lin1);
                if(*lin1 == EOS) return(matn);

                i=toupper(*lin1)-'0';
		if(i>0 && i<menu.noptions) {
                        strcpy(matnam,menu.opt[i-1].string);
                        matn=i;
		} else if (i==menu.noptions) {
                        strcpy(userfile,"");
                        if(check_user_matrix(userfile)) {
                              matn=i;
                        }
		}
		else
                switch(toupper(*lin1))  {
                        case '?':
                        case 'H':
                                get_help('8',TRUE);
                                break;
                        default:
                                fprintf(stdout,"\n\nUnrecognised Command\n\n");
                                break;
                }
        }
}


char prompt_for_yes_no(char *title,char *prompt)
{
	char line[80];
	char lin2[80];

	if(!amenus_active) return('y');

	fprintf(stdout,"\n%s\n",title);
	strcpy(line,prompt);
	strcat(line, "(y/n) ? [y]");
	getstr(line,lin2);
	if ((*lin2 != 'n') && (*lin2 != 'N'))
		return('y');
	else
		return('n');

}


/*
*	fatal()
*
*	Prints error msg to stdout and exits.
*	Variadic parameter list can be passed.
*
*	Return values:
*		none
*/

void fatal( char *msg,...)
{
	va_list ap;
	
	va_start(ap,msg);
	fprintf(stdout,"\n\nFATAL ERROR: ");
	vfprintf(stdout,msg,ap);
	fprintf(stdout,"\n\n");
	va_end(ap);
	exit(1);
}

/*
*	error()
*
*	Prints error msg to stdout.
*	Variadic parameter list can be passed.
*
*	Return values:
*		none
*/

void error( char *msg,...)
{
	va_list ap;
	
	va_start(ap,msg);
	fprintf(stdout,"\n\nERROR: ");
	vfprintf(stdout,msg,ap);
	fprintf(stdout,"\n\n");
	va_end(ap);
}

/*
*	warning()
*
*	Prints warning msg to stdout.
*	Variadic parameter list can be passed.
*
*	Return values:
*		none
*/

void warning( char *msg,...)
{
	va_list ap;
	
	va_start(ap,msg);
	fprintf(stdout,"\n\nWARNING: ");
	vfprintf(stdout,msg,ap);
	fprintf(stdout,"\n\n");
	va_end(ap);
}

/*
*	info()
*
*	Prints info msg to stdout.
*	Variadic parameter list can be passed.
*
*	Return values:
*		none
*/

void info( char *msg,...)
{
	va_list ap;
	
	va_start(ap,msg);
	fprintf(stdout,"\n");
	vfprintf(stdout,msg,ap);
	va_end(ap);
}
