/* command line interface for Clustal W  */
/* DES was here MARCH. 1994 */
/* DES was here SEPT.  1994 */
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <signal.h>
#include <setjmp.h>
#include "clustalw.h"
#include "param.h"
#include <mpi.h>

/*
*	Prototypes
*/

#ifdef UNIX
FILE    *open_path(char *);
#endif


static sint check_param(char **args,char **param_arg,char *seqname);
static void set_optional_param(char **param_arg,OPTPTR opt,Boolean dnaflag);
static sint find_match(char *probe, char *list[], sint n);
static void set_output_to_tree(ALNPTR mult_aln);
static void init_ss_options(SS_OPTPTR opt);
static void init_alnout_options(ALNOUT_OPTPTR opt);
static void init_quickpw_options(QUICKPW_OPTPTR opt);
static void init_pw_options(PW_OPTPTR opt);
static void init_mult_options(MULT_OPTPTR opt);
static void init_gap_options(MULTGAP_OPTPTR opt);
static void init_prf_options(PRF_OPTPTR opt);
static void init_tree_options(TREE_OPTPTR opt);
static void init_bstree_options(BSTREE_OPTPTR opt);
/*
*	 Global variables
*/
static sint	debug;

void init_options(OPTPTR opt)
{
	opt->explicit_type = 0;
        opt->ss_opt=(&ss_opt);
        opt->alnout_opt=(&alnout_opt);
        opt->quickpw_opt=(&quickpw_opt);
        opt->pw_opt=(&pw_opt);
        opt->mult_opt=(&mult_opt);
        (*opt->mult_opt).gap_opt=(&gap_opt);
        opt->prf_opt=(&prf_opt);
        opt->tree_opt=(&tree_opt);
        opt->bstree_opt=(&bstree_opt);

	init_ss_options(opt->ss_opt);
	init_alnout_options(opt->alnout_opt);
	init_quickpw_options(opt->quickpw_opt);
	init_pw_options(opt->pw_opt);
	init_mult_options(opt->mult_opt);
	init_gap_options((*opt->mult_opt).gap_opt);
	init_prf_options(opt->prf_opt);
	init_tree_options(opt->tree_opt);
	init_bstree_options(opt->bstree_opt);
}

static void init_bstree_options(BSTREE_OPTPTR opt)
{
/*
   bootstrap tree options
*/
	opt->boot_ntrials  = 1000;
	opt->boot_ran_seed = 111;
	opt->bootstrap_format      = BS_BRANCH_LABELS;
}

static void init_tree_options(TREE_OPTPTR opt)
{
/*
   phylogenetic tree options
*/
	opt->use_ambiguities = FALSE;
	opt->tossgaps = FALSE;
	opt->kimura = FALSE;
	opt->output_tree_clustal   = FALSE;
	opt->output_tree_phylip    = TRUE;
	opt->output_tree_distances = FALSE;
	opt->output_tree_bionj     = FALSE;
	opt->output_tree_nexus     = FALSE;
}

static void init_gap_options(MULTGAP_OPTPTR opt)
{
/*
   multiple alignment gap options
*/
	strcpy(opt->hyd_residues,"GPSNDQEKR");
	opt->gap_dist = 4;
	opt->no_hyd_penalties = FALSE;
	opt->no_var_penalties = TRUE;
	opt->no_pref_penalties = FALSE;
	opt->use_endgaps = FALSE;
	opt->endgappenalties = FALSE;

}

static void init_mult_options(MULT_OPTPTR opt)
{
/*
   multiple alignment parameters
*/
	opt->dna_gap_open = 15.0;
	opt->dna_gap_extend = 6.66;
	opt->transition_weight = 0.5;
	opt->dnamatnum = 1;
	strcpy(opt->dnamtrxname,"iub");
	strcpy(opt->dnausermtrxname,"");;
	opt->prot_gap_open = 10.0;
	opt->prot_gap_extend = 0.2;
	opt->matnum = 3;
	strcpy(opt->mtrxname,"gonnet");
	strcpy(opt->usermtrxname,"");
	opt->divergence_cutoff = 30;
	opt->no_weights = FALSE;
	opt->neg_matrix = FALSE;
	opt->reset_alignments_new  = FALSE;
	opt->reset_alignments_all  = FALSE;
	opt->propogate_motifs = FALSE;
	opt->use_motifs = FALSE;
	strcpy(opt->motif_filename,"");
}

static void init_prf_options(PRF_OPTPTR opt)
{
	opt->profile_type = PROFILE;
}

static void init_pw_options(PW_OPTPTR opt)
{
/*
   slow pairwise alignment parameters
*/
	opt->dna_go_penalty = 15.0;
	opt->dna_ge_penalty = 6.66;
	opt->transition_weight = 0.5;
	opt->prot_go_penalty = 10.0;
	opt->prot_ge_penalty = 1.0;
	opt->matnum = 3;
	strcpy(opt->mtrxname,"gonnet");
	opt->dnamatnum = 1;
	strcpy(opt->dnamtrxname,"iub");
	strcpy(opt->usermtrxname,"");
	strcpy(opt->dnausermtrxname,"");
	opt->add_motif = FALSE;
	opt->quick_pairalign = FALSE;
	opt->treealgo = NJ;
}

static void init_quickpw_options(QUICKPW_OPTPTR opt)
{
/*
   quick pairwise alignment parameters
*/
	opt->dna_ktup      = 2;   /* default parameters for DNA */
	opt->dna_wind_gap  = 5;
	opt->dna_signif    = 4;
	opt->dna_window    = 4;

	opt->prot_ktup     = 1;   /* default parameters for proteins */
	opt->prot_wind_gap = 3;
	opt->prot_signif   = 5;
	opt->prot_window   = 5;
	opt->percent=TRUE;
}

static void init_alnout_options(ALNOUT_OPTPTR opt)
{
/*
   alignment output format parameters
*/
        opt->showaln        = TRUE; /* screen display for clustalw */
        opt->output_clustal = TRUE;
        opt->output_gcg     = FALSE;
        opt->output_phylip  = FALSE;
        opt->output_nbrf    = FALSE;
        opt->output_gde     = FALSE;
        opt->output_nexus   = FALSE;
        opt->output_gscope  = FALSE;
        opt->lowercase       = TRUE; /* Flag for GDE output */
        opt->seq_numbers = FALSE;
        opt->output_order   = ALIGNED;
}

static void init_ss_options(SS_OPTPTR opt)
{
/*
   secondary structure parameters
*/
        opt->output_struct_penalties = 0;	/* output secondary structure masks with alignment */
        opt->use_ss1 = TRUE;			/* use secondary structures for profile 1 in alignment */
        opt->use_ss2 = TRUE;			/* use secondary structures for profile 2 in alignment */
        opt->helix_penalty = 4;			/* gap penalty mask value for helices */
        opt->strand_penalty = 4;		/* gap penalty mask value for beta strands */
        opt->loop_penalty = 1;			/* gap penalty mask value for non-helix, non-beta */
        opt->helix_end_minus = 3;		/* use end penalty for 3 residues within each end of helix */
        opt->helix_end_plus = 0;		/* use end penalty for 3 residues outside each end of helix */
        opt->strand_end_minus = 1;		/* use end penalty for 3 residues within each end of strand */
        opt->strand_end_plus = 1;		/* use end penalty for 3 residues outside each end of strand */
        opt->helix_end_penalty = 2;		/* gap penalty mask value for ends of helices */
        opt->strand_end_penalty = 2;		/* gap penalty mask value for ends of strands */
}

static sint check_param(char **args, char **param_arg, char *seqname)
{

/*
#ifndef MAC
        char *strtok(char *s1, const char *s2);
#endif
*/
        sint     len,i,j,k,s,n,match[MAXARGS+1];
	Boolean 	name1 = FALSE;
	char	*params[MAXARGS+1];


		if(args[0]==NULL) return(0);

		params[0]=(char *)ckalloc(strlen(args[0])*sizeof(char));
		if (args[0][0]!=COMMANDSEP)
		{
			name1 = TRUE;
			strcpy(params[0],args[0]);
		}
		else
			strcpy(params[0],&args[0][1]);

		for (i=1;i<MAXARGS;i++) {
			if(args[i]==NULL) break;
			params[i]=(char *)ckalloc(strlen(args[i])*sizeof(char));
			for(j=0;j<strlen(args[i]);j++)
				if(isprint(args[i][j])) params[i][j]=args[i][j+1];
		}
		if (i==MAXARGS) {
			fprintf(stdout,"Error: too many command line arguments\n");
			return(-1);
	}
/*
    special case - first parameter is input filename
*/
	s = 0;
	if(name1 == TRUE) {
		strcpy(seqname, params[0]);
/*  JULIE
    convert to lower case now
*/
#ifndef UNIX
		for(k=0;k<(sint)strlen(params[0]);k++) seqname[k]=tolower(params[0][k]);
#else
		for(k=0;k<(sint)strlen(params[0]);k++) seqname[k]=params[0][k];
#endif 
		s++;
		seqname[k]=EOS;
	}
	
	n = i;
    for (i=s;i<n;i++) {
        param_arg[i] = NULL;
		len = (sint)strlen(params[i]);
		for(j=0; j<len; j++)
			if(params[i][j] == '=') {
				param_arg[i] = (char *)ckalloc((len-j) * sizeof(char));
				strncpy(param_arg[i],&params[i][j+1],len-j-1);
				params[i][j] = EOS;
/*  JULIE
    convert keywords to lower case now
*/
				for(k=0;k<j;++k) params[i][k]=tolower(params[i][k]);
				param_arg[i][len-j-1] = EOS;
				break;
			}
	}
	
/*
    for each parameter given on the command line, first search the list of recognised optional 
    parameters....
*/

    for (i=0;i<n;i++) {
		if ((i==0) && (name1 == TRUE)) continue;
		j = 0;
		match[i] = -1;
		for(;;) {
			if (cmd_line_para[j].str[0] == '\0') break;
			if (!strcmp(params[i],cmd_line_para[j].str)) {
				match[i] = j;
				*cmd_line_para[match[i]].flag = i;
				if ((cmd_line_para[match[i]].type != NOARG) &&
                                    (param_arg[i] == NULL)) {
					fprintf(stdout,
                       				 "Error: parameter required for /%s\n",params[i]);
					exit(1);
				}
/*  JULIE
    convert parameters to lower case now, unless the parameter is a filename
*/
#ifdef UNIX
				else if (cmd_line_para[match[i]].type != FILARG
				         && param_arg[i] != NULL)
#endif 
				if (param_arg[i]!=0)
					{
					for(k=0;k<strlen(param_arg[i]);++k)
						param_arg[i][k]=tolower(param_arg[i][k]);
					}
				break;
			}
			j++;
		}
	}
/*
    ....then the list of recognised input files,.... 
*/
    for (i=0;i<n;i++) {
		if ((i==0) && (name1 == TRUE)) continue;
		if (match[i] != -1) continue;
		j = 0;
		for(;;) {
			if (cmd_line_file[j].str[0] == '\0') break;
			if (!strcmp(params[i],cmd_line_file[j].str)) {
				match[i] = j;
				*cmd_line_file[match[i]].flag = i;
				if ((cmd_line_file[match[i]].type != NOARG) &&
                                    (param_arg[i] == NULL)) {
					fprintf(stdout,
                       				 "Error: parameter required for /%s\n",params[i]);
					exit(1);
				}
				break;
			}
			j++;
		}
	}
/*
	....and finally the recognised verbs. 
*/
    for (i=0;i<n;i++) {
		if ((i==0) && (name1 == TRUE)) continue;
		if (match[i] != -1) continue;
		j = 0;
		for(;;) {
			if (cmd_line_verb[j].str[0] == '\0') break;
			if (!strcmp(params[i],cmd_line_verb[j].str)) {
				match[i] = j;
				*cmd_line_verb[match[i]].flag = i;
				if ((cmd_line_verb[match[i]].type != NOARG) &&
                                    (param_arg[i] == NULL)) {
					fprintf(stdout,
                       				 "Error: parameter required for /%s\n",params[i]);
					exit(1);
				}
				break;
			}
			j++;
		}
	}

/*
	check for any unrecognised parameters.
*/
    for (i=0;i<n;i++) {
		if (match[i] == -1) {
			fprintf(stdout,
                        "Error: unknown option %c%s\n",COMMANDSEP,params[i]);
			exit(1);
		}
	}
        return(n);
}

static void set_optional_param(char **param_arg,OPTPTR opt,Boolean dnaflag)
{
	int i,temp;
	int c;
	float ftemp;
	char tstr[100];

/****************************************************************************/
/* look for parameters on command line  e.g. gap penalties, k-tuple etc.    */
/****************************************************************************/

/*** ? /score=percent or /score=absolute */
        if(setscore != -1)
                if(strlen(param_arg[setscore]) > 0) {
     			temp = find_match(param_arg[setscore],score_arg,2);
                        if(temp == 0)
                                quickpw_opt.percent = TRUE;
                        else if(temp == 1)
                                quickpw_opt.percent = FALSE;
                        else
                                fprintf(stdout,"\nUnknown SCORE type: %s\n",
                                param_arg[setscore]);
                }

/*** ? /seed=n */
        if(setseed != -1) {
                temp = 0;
                if(strlen(param_arg[setseed]) > 0)
                         if (sscanf(param_arg[setseed],"%d",&temp)!=1) {
                         	fprintf(stdout,"Bad option for /seed (must be integer)\n");
                         	temp = 0;
                         }
                if(temp > 0) bstree_opt.boot_ran_seed = temp;
        fprintf(stdout,"\ntemp = %d; seed = %u;\n",(pint)temp,bstree_opt.boot_ran_seed);
        }




/*** ? /output=PIR, GCG, GDE or PHYLIP */
		if(setoutput != -1)
		if(strlen(param_arg[setoutput]) > 0) {
			temp = find_match(param_arg[setoutput],output_arg,6);
			if (temp >= 0 && temp <= 5) {
				opt->alnout_opt->output_clustal = FALSE;
				opt->alnout_opt->output_gcg     = FALSE;
				opt->alnout_opt->output_phylip  = FALSE;
				opt->alnout_opt->output_nbrf    = FALSE;
				opt->alnout_opt->output_gde     = FALSE;
				opt->alnout_opt->output_nexus   = FALSE;
				opt->alnout_opt->output_gscope     = FALSE;
			}
			switch (temp) {
				case 0: /* GCG */
					opt->alnout_opt->output_gcg     = TRUE;
					break;
				case 1: /* GDE */
					opt->alnout_opt->output_gde     = TRUE;
					break;
				case 2: /* PIR */
					opt->alnout_opt->output_nbrf    = TRUE;
					break;
				case 3: /* PHYLIP */
					opt->alnout_opt->output_phylip  = TRUE;
					break;
				case 4: /* NEXUS */
					opt->alnout_opt->output_nexus  = TRUE;
					break;
				case 5: /* GSCOPE */
					opt->alnout_opt->output_gscope  = TRUE;
					break;
				default:
					fprintf(stdout,"\nUnknown OUTPUT type: %s\n",
					param_arg[setoutput]);
			}
		}

/*** ? /outputtree=NJ or PHYLIP or DIST */
	if(setoutputtree != -1)
		if(strlen(param_arg[setoutputtree]) > 0) {
			temp = find_match(param_arg[setoutputtree],outputtree_arg,5);
			opt->tree_opt->output_tree_clustal = FALSE;
			opt->tree_opt->output_tree_phylip = FALSE;
			opt->tree_opt->output_tree_distances = FALSE;
			opt->tree_opt->output_tree_bionj = FALSE;
			opt->tree_opt->output_tree_nexus = FALSE;
			switch (temp) {
				case 0: /* NJ */
					opt->tree_opt->output_tree_clustal = TRUE;
					break;
				case 1: /* PHYLIP */
					opt->tree_opt->output_tree_phylip  = TRUE;
					break;
				case 2: /* DIST */
					opt->tree_opt->output_tree_distances = TRUE;
					break;
				case 3: /* BIONJ */
					opt->tree_opt->output_tree_bionj = TRUE;
					break;
				case 4: /* NEXUS */
					opt->tree_opt->output_tree_nexus = TRUE;
					break;
				default:
					fprintf(stdout,"\nUnknown OUTPUT TREE type: %s\n",
					param_arg[setoutputtree]);
			}
		}

/*** ? /profile (sets type of second input file to profile) */
	if(setprofile != -1)
		opt->prf_opt->profile_type = PROFILE;

/*** ? /sequences (sets type of second input file to list of sequences)  */
	if(setsequences != -1)
		opt->prf_opt->profile_type = SEQUENCE;



/*** ? /ktuple=n */
        if(setktuple != -1) {
                temp = 0;
                if(strlen(param_arg[setktuple]) > 0)
			 		if (sscanf(param_arg[setktuple],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /ktuple (must be integer)\n");
                         temp = 0;
                    }
                if(temp > 0) {
                        if(dnaflag) {
                                if(temp <= 4) {
                                        quickpw_opt.dna_ktup     = temp;
                                        quickpw_opt.dna_wind_gap = quickpw_opt.dna_ktup + 4;
                                }
                        }
                        else {
                                if(temp <= 2) {
                                        quickpw_opt.prot_ktup     = temp;
                                        quickpw_opt.prot_wind_gap = quickpw_opt.prot_ktup + 3;
                                }
                        }
                }
        }

/*** ? /pairgap=n */
        if(setpairgap != -1) {
                temp = 0;
                if(strlen(param_arg[setpairgap]) > 0)
			 		if (sscanf(param_arg[setpairgap],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /pairgap (must be integer)\n");
                         temp = 0;
                    }
                if(temp > 0)
                        if(dnaflag) {
                                if(temp > quickpw_opt.dna_ktup) {
                                        quickpw_opt.dna_wind_gap = temp;
                                }
                        }
                        else {
                                if(temp > quickpw_opt.prot_ktup) {
                                        quickpw_opt.prot_wind_gap = temp;
                                }
                        }
        }


/*** ? /topdiags=n   */
        if(settopdiags != -1) {
                temp = 0;
                if(strlen(param_arg[settopdiags]) > 0)
			 		if (sscanf(param_arg[settopdiags],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /topdiags (must be integer)\n");
                         temp = 0;
                    }
                if(temp > 0)
                        if(dnaflag) {
                                if(temp > quickpw_opt.dna_ktup) {
                                        quickpw_opt.dna_signif   = temp;
                                }
                        }
                        else {
                                if(temp > quickpw_opt.prot_ktup) {
                                        quickpw_opt.prot_signif   = temp;
                                }
                        }
        }


/*** ? /window=n  */
        if(setwindow != -1) {
                temp = 0;
                if(strlen(param_arg[setwindow]) > 0)
			 		if (sscanf(param_arg[setwindow],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /window (must be integer)\n");
                         temp = 0;
                    }
                if(temp > 0)
                        if(dnaflag) {
                                if(temp > quickpw_opt.dna_ktup) {
                                        quickpw_opt.dna_window   = temp;
                                }
                        }
                        else {
                                if(temp > quickpw_opt.prot_ktup) {
                                        quickpw_opt.prot_window   = temp;
                                }
                        }
        }

/*** ? /kimura */
	if(setkimura != -1)
		tree_opt.kimura = TRUE;

/*** ? /tossgaps */
	if(settossgaps != -1)
		tree_opt.tossgaps = TRUE;


/*** ? /negative  */
	if(setnegative != -1)
		mult_opt.neg_matrix = TRUE;

/*** ? /noweights */
	if(setnoweights != -1)
		mult_opt.no_weights = TRUE;

/*** ? /bootmatrix=ID (user's file)  */
        if(setbootmatrix != -1)
        {
                temp=strlen(param_arg[setbootmatrix]);
                if(temp > 0) {
                        for(i=0;i<temp;i++)
                                tstr[i]=param_arg[setbootmatrix][i];
                        tstr[i]='\0';
                        if(check_user_matrix(param_arg[setbootmatrix]))
                        {
                              strcpy(tree_opt.matrixfile,param_arg[setbootmatrix]);
                                tree_opt.use_matrix=TRUE;
                        }
                        else exit(1);

                }
        }
/*** ? /bootmatch= n */
        if(setbootmatch != -1) {
                temp = 0;
                if(strlen(param_arg[setbootmatch]) > 0)
                        if (sscanf(param_arg[setbootmatch],"%d",&temp)!=1) {
                 fprintf(stdout,"Bad option for /bootmatch (must be integer)\n");
                 temp = 0;
            }
                if (temp >= 0)
                        tree_opt.cutoff = temp;
        }


/*** ? /pwmatrix=ID (user's file)  */
	if(setpwmatrix != -1)
	{
		temp=strlen(param_arg[setpwmatrix]);
		if(temp > 0) {
			for(i=0;i<temp;i++)
				if (isupper(param_arg[setpwmatrix][i]))
					tstr[i]=tolower(param_arg[setpwmatrix][i]);
				else
					tstr[i]=param_arg[setpwmatrix][i];
			tstr[i]='\0';
                        if (strcmp(tstr,"blosum")==0) {
                                strcpy(pw_opt.mtrxname, tstr);
                                pw_opt.matnum = 1;
                        }
                        else if (strcmp(tstr,"pam")==0) {
                                strcpy(pw_opt.mtrxname, tstr);
                                pw_opt.matnum = 2;
                        }
                        else if (strcmp(tstr,"gonnet")==0) {
                                strcpy(pw_opt.mtrxname, tstr);
                                pw_opt.matnum = 3;
                        }
                        else if (strcmp(tstr,"bin")==0) {
                                strcpy(pw_opt.mtrxname, tstr);
                                pw_opt.matnum = 4;
                        }
                        else if (strcmp(tstr,"id")==0) {
                                strcpy(pw_opt.mtrxname, tstr);
                                pw_opt.matnum = 5;
                        }
			else {
                                if(check_user_matrix(param_arg[setpwmatrix]))
                                  {
                                     strcpy(pw_opt.mtrxname,"");
                                     strcpy(pw_opt.usermtrxname,param_arg[setpwmatrix]);
                                     pw_opt.matnum=6;
                                  }
				else exit(1);
			}

		}
	}

/*** ? /propogate  */
	if(setpropogate != -1)
	{
                mult_opt.propogate_motifs=TRUE;
	}

/*** ? /motifs=filename (user's file)  */
	if(setmotifs != -1)
	{
		temp=strlen(param_arg[setmotifs]);
		if(temp > 0) {
			for(i=0;i<temp;i++)
				tstr[i]=param_arg[setmotifs][i];
			tstr[i]='\0';
                        strcpy(mult_opt.motif_filename, tstr);
		}
                mult_opt.use_motifs=TRUE;
	}

/*** ? /matrix=ID (user's file)  */
	if(setmatrix != -1)
	{
		temp=strlen(param_arg[setmatrix]);
		if(temp > 0) {
			for(i=0;i<temp;i++)
				if (isupper(param_arg[setmatrix][i]))
					tstr[i]=tolower(param_arg[setmatrix][i]);
				else
					tstr[i]=param_arg[setmatrix][i];
			tstr[i]='\0';
                        if (strcmp(tstr,"blosum")==0) {
                                strcpy(mult_opt.mtrxname, tstr);
                                mult_opt.matnum = 1;
                        }
                        else if (strcmp(tstr,"pam")==0) {
                                strcpy(mult_opt.mtrxname, tstr);
                                mult_opt.matnum = 2;
                        }
                        else if (strcmp(tstr,"gonnet")==0) {
                                strcpy(mult_opt.mtrxname, tstr);
                                mult_opt.matnum = 3;
                        }
                        else if (strcmp(tstr,"bin")==0) {
                                strcpy(mult_opt.mtrxname, tstr);
                                mult_opt.matnum = 4;
                        }
                        else if (strcmp(tstr,"id")==0) {
                                strcpy(mult_opt.mtrxname, tstr);
                                mult_opt.matnum = 5;
                        }
			else {
                                if(check_user_matrix_series(param_arg[setmatrix],FALSE))
                                  {
                                     strcpy(mult_opt.mtrxname,"");
                                     strcpy(mult_opt.usermtrxname,param_arg[setmatrix]);
                                     mult_opt.matnum=6;
                                  }
				else exit(1);
			}

		}
	}

/*** ? /pwdnamatrix=ID (user's file)  */
	if(setpwdnamatrix != -1)
	{
		temp=strlen(param_arg[setpwdnamatrix]);
		if(temp > 0) {
			for(i=0;i<temp;i++)
				if (isupper(param_arg[setpwdnamatrix][i]))
					tstr[i]=tolower(param_arg[setpwdnamatrix][i]);
				else
					tstr[i]=param_arg[setpwdnamatrix][i];
			tstr[i]='\0';
                        if (strcmp(tstr,"iub")==0) {
                                strcpy(pw_opt.dnamtrxname, tstr);
                                pw_opt.dnamatnum = 1;
                        }
                        else if (strcmp(tstr,"clustalw")==0) {
                                strcpy(pw_opt.dnamtrxname, tstr);
                                pw_opt.dnamatnum = 2;
                        }
			else {
                                if(check_user_matrix(param_arg[setpwdnamatrix]))
                                  {
                                     strcpy(pw_opt.dnamtrxname,"");
                                     strcpy(pw_opt.dnausermtrxname,param_arg[setpwdnamatrix]);
                                     pw_opt.dnamatnum=3;
                                  }
				else exit(1);
			}

		}
	}

/*** ? /matrix=ID (user's file)  */
	if(setdnamatrix != -1)
	{
		temp=strlen(param_arg[setdnamatrix]);
		if(temp > 0) {
			for(i=0;i<temp;i++)
				if (isupper(param_arg[setdnamatrix][i]))
					tstr[i]=tolower(param_arg[setdnamatrix][i]);
				else
					tstr[i]=param_arg[setdnamatrix][i];
			tstr[i]='\0';
                        if (strcmp(tstr,"iub")==0) {
                                strcpy(mult_opt.dnamtrxname, tstr);
                                mult_opt.dnamatnum = 1;
                        }
                        else if (strcmp(tstr,"clustalw")==0) {
                                strcpy(mult_opt.dnamtrxname, tstr);
                                mult_opt.dnamatnum = 2;
                        }
			else {
                                if(check_user_matrix(param_arg[setdnamatrix]))
                                  {
                                     strcpy(mult_opt.dnamtrxname,"");
                                     strcpy(mult_opt.dnausermtrxname,param_arg[setdnamatrix]);
                                     mult_opt.dnamatnum=3;
                                  }
				else exit(1);
			}

		}
	}
/*** ? /maxdiv= n */
	if(setmaxdiv != -1) {
		temp = 0;
		if(strlen(param_arg[setmaxdiv]) > 0)
			if (sscanf(param_arg[setmaxdiv],"%d",&temp)!=1) {
                 fprintf(stdout,"Bad option for /maxdiv (must be integer)\n");
                 temp = 0;
            }
		if (temp >= 0)
			mult_opt.divergence_cutoff = temp;
	}

/*** ? /gapdist= n */
	if(setgapdist != -1) {
		temp = 0;
		if(strlen(param_arg[setgapdist]) > 0)
			if (sscanf(param_arg[setgapdist],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /gapdist (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 0)
			(*mult_opt.gap_opt).gap_dist = temp;
	}

/*** ? /debug= n */
	if(setdebug != -1) {
		temp = 0;
		if(strlen(param_arg[setdebug]) > 0)
			if (sscanf(param_arg[setdebug],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /debug (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 0)
			debug = temp;
	}

/*** ? /outfile= (user's file)  */
	if(setoutfile != -1)
		if(strlen(param_arg[setoutfile]) > 0) {
                        strcpy(opt->alnout_opt->clustal_outname, param_arg[setoutfile]);
                        strcpy(opt->alnout_opt->phylip_outname, param_arg[setoutfile]);
                        strcpy(opt->alnout_opt->nbrf_outname, param_arg[setoutfile]);
                        strcpy(opt->alnout_opt->gde_outname, param_arg[setoutfile]);
                        strcpy(opt->alnout_opt->gcg_outname, param_arg[setoutfile]);
                        strcpy(opt->alnout_opt->nexus_outname, param_arg[setoutfile]);
                        strcpy(opt->alnout_opt->gscope_outname, param_arg[setoutfile]);
		}

/*** ? /outtreefile= (user's file)  */
	if(setouttreefile != -1)
		if(strlen(param_arg[setouttreefile]) > 0) {
                        strcpy(opt->tree_opt->clustal_outname, param_arg[setouttreefile]);
                        strcpy(opt->tree_opt->phylip_outname, param_arg[setouttreefile]);
                        strcpy(opt->tree_opt->dist_outname, param_arg[setouttreefile]);
		}

/*** ? /case= lower/upper  */
	if(setcase != -1) 
		if(strlen(param_arg[setcase]) > 0) {
			temp = find_match(param_arg[setcase],case_arg,2);
			if(temp == 0) {
				opt->alnout_opt->lowercase = TRUE;
			}
			else if(temp == 1) {
				opt->alnout_opt->lowercase = FALSE;
			}
			else
				fprintf(stdout,"\nUnknown case %s\n",
				param_arg[setcase]);
		}

/*** ? /seqnos=off/on  */
	if(setseqno != -1) 
		if(strlen(param_arg[setseqno]) > 0) {
			temp = find_match(param_arg[setseqno],seqno_arg,2);
			if(temp == 0) {
				opt->alnout_opt->seq_numbers = FALSE;
			}
			else if(temp == 1) {
				opt->alnout_opt->seq_numbers = TRUE;
			}
			else
				fprintf(stdout,"\nUnknown SEQNO option %s\n",
				param_arg[setseqno]);
		}


/*** ? /gapopen=n  */
	if(setgapopen != -1) {
		ftemp = 0.0;
		if(strlen(param_arg[setgapopen]) > 0)
			if (sscanf(param_arg[setgapopen],"%f",&ftemp)!=1) {
                         fprintf(stdout,"Bad option for /gapopen (must be real number)\n");
                         ftemp = 0.0;
                    }
		if(ftemp >= 0.0)
			if(dnaflag) {
					mult_opt.dna_gap_open = ftemp;
			}
			else {
					mult_opt.prot_gap_open = ftemp;
			}
	}


/*** ? /gapext=n   */
	if(setgapext != -1) {
		ftemp = 0.0;
		if(strlen(param_arg[setgapext]) > 0)
			if (sscanf(param_arg[setgapext],"%f",&ftemp)!=1) {
                         fprintf(stdout,"Bad option for /gapext (must be real number)\n");
                         ftemp = 0.0;
                    }
		if(ftemp >= 0)
			if(dnaflag) {
					mult_opt.dna_gap_extend  = ftemp;
			}
			else {
					mult_opt.prot_gap_extend = ftemp;
			}
	}

/*** ? /transweight=n*/
	if(settransweight != -1) {
		ftemp = 0.0;
		if(strlen(param_arg[settransweight]) > 0)
			if (sscanf(param_arg[settransweight],"%f",&ftemp)!=1) {
                         fprintf(stdout,"Bad option for /transweight (must be real number)\n");
                         ftemp = 0.0;
                    }
		mult_opt.transition_weight=ftemp;
		pw_opt.transition_weight=ftemp;
	}

/*** ? /pwgapopen=n  */
	if(setpwgapopen != -1) {
		ftemp = 0.0;
		if(strlen(param_arg[setpwgapopen]) > 0)
			if (sscanf(param_arg[setpwgapopen],"%f",&ftemp)!=1) {
                         fprintf(stdout,"Bad option for /pwgapopen (must be real number)\n");
                         ftemp = 0.0;
                    }
		if(ftemp >= 0.0)
			if(dnaflag) {
                                        pw_opt.dna_go_penalty = ftemp;
			}
			else {
                                        pw_opt.prot_go_penalty = ftemp;
			}
	}


/*** ? /gapext=n   */
	if(setpwgapext != -1) {
		ftemp = 0.0;
		if(strlen(param_arg[setpwgapext]) > 0)
			if (sscanf(param_arg[setpwgapext],"%f",&ftemp)!=1) {
                         fprintf(stdout,"Bad option for /pwgapext (must be real number)\n");
                         ftemp = 0.0;
                    }
		if(ftemp >= 0)
			if(dnaflag) {
                                        pw_opt.dna_ge_penalty = ftemp;
			}
			else {
                                        pw_opt.prot_ge_penalty = ftemp;
			}
	}



/*** ? /outorder=n  */
	if(setoutorder != -1) {
		if(strlen(param_arg[setoutorder]) > 0)
			temp = find_match(param_arg[setoutorder],outorder_arg,3);
			if(temp == 0)  {	
				opt->alnout_opt->output_order   = INPUT;
			}
			else if(temp == 1)  {	
				opt->alnout_opt->output_order   = ALIGNED;
			}
			else if(temp == 2)  {	
				opt->alnout_opt->output_order   = TREEORDER;
			}
			else
				fprintf(stdout,"\nUnknown OUTPUT ORDER type %s\n",
				param_arg[setoutorder]);
	}

/*** ? /bootlabels=n  */
	if(setbootlabels != -1) {
		if(strlen(param_arg[setbootlabels]) > 0)
			temp = find_match(param_arg[setbootlabels],bootlabels_arg,2);
			if(temp == 0)  {	
				opt->bstree_opt->bootstrap_format   = BS_NODE_LABELS;
			}
			else if(temp == 1)  {	
				opt->bstree_opt->bootstrap_format   = BS_BRANCH_LABELS;
			}
			else
				fprintf(stdout,"\nUnknown bootlabels type %s\n",
				param_arg[setoutorder]);
	}

/*** ? /endgaps */
	if(setuseendgaps != -1)
		(*mult_opt.gap_opt).use_endgaps = FALSE;

/*** ? /nopgap  */
	if(setnopgap != -1)
		(*mult_opt.gap_opt).no_pref_penalties = TRUE;

/*** ? /nohgap  */
	if(setnohgap != -1)
		(*mult_opt.gap_opt).no_hyd_penalties = TRUE;

/*** ? /novgap  */
	if(setnovgap != -1)
		(*mult_opt.gap_opt).no_var_penalties = FALSE;

/*** ? /hgapresidues="string"  */
	if(sethgapres != -1)
		if(strlen(param_arg[sethgapres]) > 0) {
			for (i=0;i<strlen((*mult_opt.gap_opt).hyd_residues) && i<26;i++) {
				c = param_arg[sethgapres][i];
				if (isalpha(c))
					(*mult_opt.gap_opt).hyd_residues[i] = (char)toupper(c);
				else
					break;
			}
		}
		
		
/*** ? /nosecstr1  */
	if(setsecstr1 != -1)
		opt->ss_opt->use_ss1 = FALSE;

/*** ? /nosecstr2  */
	if(setsecstr2 != -1)
		opt->ss_opt->use_ss2 = FALSE;

/*** ? /secstroutput  */
	if(setsecstroutput != -1)
		if(strlen(param_arg[setsecstroutput]) > 0) {
			temp = find_match(param_arg[setsecstroutput],outputsecstr_arg,4);
			if(temp >= 0 && temp <= 3)
				opt->ss_opt->output_struct_penalties = temp;
			else
				fprintf(stdout,"\nUnknown case %s\n",
				param_arg[setsecstroutput]);
		}


/*** ? /helixgap= n */
	if(sethelixgap != -1) {
		temp = 0;
		if(strlen(param_arg[sethelixgap]) > 0)
			if (sscanf(param_arg[sethelixgap],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /helixgap (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 1 && temp <= 9)
			opt->ss_opt->helix_penalty = temp;

	}
	
/*** ? /strandgap= n */
	if(setstrandgap != -1) {
		temp = 0;
		if(strlen(param_arg[setstrandgap]) > 0)
			if (sscanf(param_arg[setstrandgap],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /strandgap (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 1 && temp <= 9)
			opt->ss_opt->strand_penalty = temp;
	}
	
/*** ? /loopgap= n */
	if(setloopgap != -1) {
		temp = 0;
		if(strlen(param_arg[setloopgap]) > 0)
			if (sscanf(param_arg[setloopgap],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /loopgap (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 1 && temp <= 9)
			opt->ss_opt->loop_penalty = temp;
	}

/*** ? /terminalgap= n */
	if(setterminalgap != -1) {
		temp = 0;
		if(strlen(param_arg[setterminalgap]) > 0)
			if (sscanf(param_arg[setterminalgap],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /terminalgap (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 1 && temp <= 9) {
			opt->ss_opt->helix_end_penalty = temp;
			opt->ss_opt->strand_end_penalty = temp;
		}
	}
	
/*** ? /helixendin= n */
	if(sethelixendin != -1) {
		temp = 0;
		if(strlen(param_arg[sethelixendin]) > 0)
			if (sscanf(param_arg[sethelixendin],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /helixendin (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 0 && temp <= 3)
			opt->ss_opt->helix_end_minus = temp;
	}

/*** ? /helixendout= n */
	if(sethelixendout != -1) {
		temp = 0;
		if(strlen(param_arg[sethelixendout]) > 0)
			if (sscanf(param_arg[sethelixendout],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /helixendout (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 0 && temp <= 3)
			opt->ss_opt->helix_end_plus = temp;
	}

/*** ? /strandendin= n */
	if(setstrandendin != -1) {
		temp = 0;
		if(strlen(param_arg[setstrandendin]) > 0)
			if (sscanf(param_arg[setstrandendin],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /strandendin (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 0 && temp <= 3)
			opt->ss_opt->strand_end_minus = temp;
	}

/*** ? /strandendout= n */
	if(setstrandendout != -1) {
		temp = 0;
		if(strlen(param_arg[setstrandendout]) > 0)
			if (sscanf(param_arg[setstrandendout],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /strandendout (must be integer)\n");
                         temp = 0;
                    }
		if (temp >= 0 && temp <= 3)
			opt->ss_opt->strand_end_plus = temp;
	}

}
 
#ifdef UNIX
FILE *open_path(char *fname)  /* to open in read-only file fname searching for 
				 it through all path directories */
{
#define Mxdir 70
        char dir[Mxdir+1], *path, *deb, *fin;
        FILE *fich;
        sint lf, ltot;
	char *path1;
 
        path=getenv("PATH"); 	/* get the list of path directories, 
					separated by :
    				*/

        /* added for File System Standards  - Francois */
        path1=(char *)ckalloc((strlen(path)+64)*sizeof(char));
        strcpy(path1,path);
        strcat(path1,"/usr/share/clustalx:/usr/local/share/clustalx");

        lf=(sint)strlen(fname);
        deb=path1;
        do
                {
                fin=strchr(deb,':');
                if(fin!=NULL)
                        { strncpy(dir,deb,fin-deb); ltot=fin-deb; }
                else
                        { strcpy(dir,deb); ltot=(sint)strlen(dir); }
                /* now one directory is in string dir */
                if( ltot + lf + 1 <= Mxdir)
                        {
                        dir[ltot]='/';
                        strcpy(dir+ltot+1,fname); /* now dir is appended with fi
   lename */
                        if( (fich = fopen(dir,"r") ) != NULL) break;
                        }
                else fich = NULL;
                deb=fin+1;
                }
        while (fin != NULL);
        return fich;
}
#endif

void get_help(char help_pointer,Boolean interactive)    /* Help procedure */
{	
	FILE *help_file;
	sint  i, nlines;
	Boolean found_help;
	char temp[MAXLINE+1];
	char lin2[MAXLINE+1];
	char token = '\0';
	char *digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	char *help_marker    = ">>HELP";
	char *help_file_name;

	help_file_name=get_help_file_name();
#ifdef VMS
        if((help_file=fopen(help_file_name,"r","rat=cr","rfm=var"))==NULL) {
            error("Cannot open help file [%s]",help_file_name);
            return;
        }
#else

#ifdef UNIX
        if((help_file=open_path(help_file_name))==NULL) {
             if((help_file=fopen(help_file_name,"r"))==NULL) {
                  error("Cannot open help file [%s]",help_file_name);
                  return;
             }
        }
        
#else
        if((help_file=fopen(help_file_name,"r"))==NULL) {
            error("Cannot open help file [%s]",help_file_name);
            return;
        }
#endif

#endif
	ckfree(help_file_name);
	nlines = 0;
	found_help = FALSE;

	while(TRUE) {
		if(fgets(temp,MAXLINE+1,help_file) == NULL) {
			if(!found_help)
				error("No help found in help file");
			fclose(help_file);
			return;
		}
		if(strstr(temp,help_marker)) {
                        token = ' ';
			for(i=strlen(help_marker); i<8; i++)
				if(strchr(digits, temp[i])) {
					token = temp[i];
					break;
				}
		}
		if(token == help_pointer) {
			found_help = TRUE;
			while(fgets(temp,MAXLINE+1,help_file)) {
				if(strstr(temp, help_marker)){
				  	if(interactive) {
						fprintf(stdout,"\n");
				    		getstr("Press [RETURN] to continue",lin2);
				  	}
					fclose(help_file);
					return;
				}
				if(temp[0]!='<') {
			       		fputs(temp,stdout);
			       		++nlines;
				}
			       if(interactive) {
			          if(nlines >= PAGE_LEN) {
				     	   fprintf(stdout,"\n");
			 	  	   getstr("Press [RETURN] to continue or  X  to stop",lin2);
				  	   if(toupper(*lin2) == 'X') {
						   fclose(help_file);
						   return;
				  	   }
				  	   else
						   nlines = 0;
				   }
			       }
			}
			if(interactive) {
				fprintf(stdout,"\n");
				getstr("Press [RETURN] to continue",lin2);
			}
			fclose(help_file);
		}
	}
}

void show_aln(ALNOUT_OPT alnout_opt)         /* Alignment screen display procedure */
{
        FILE *file;
        sint  nlines;
        char temp[MAXLINE+1];
        char lin2[MAXLINE+1];
        char file_name[FILENAMELEN+1];

        if(alnout_opt.output_clustal) strcpy(file_name,alnout_opt.clustal_outname);
        else if(alnout_opt.output_nbrf) strcpy(file_name,alnout_opt.nbrf_outname);
        else if(alnout_opt.output_gcg) strcpy(file_name,alnout_opt.gcg_outname);
        else if(alnout_opt.output_phylip) strcpy(file_name,alnout_opt.phylip_outname);
        else if(alnout_opt.output_gde) strcpy(file_name,alnout_opt.gde_outname);
        else if(alnout_opt.output_nexus) strcpy(file_name,alnout_opt.nexus_outname);
        else if(alnout_opt.output_gscope) strcpy(file_name,alnout_opt.gscope_outname);

#ifdef VMS
        if((file=fopen(file_name,"r","rat=cr","rfm=var"))==NULL) {
#else
        if((file=fopen(file_name,"r"))==NULL) {
#endif
                error("Cannot open file [%s]",file_name);
                return;
        }

        fprintf(stdout,"\n\n");
        nlines = 0;

        while(fgets(temp,MAXLINE+1,file)) {
                fputs(temp,stdout);
                ++nlines;
                if(nlines >= PAGE_LEN) {
                        fprintf(stdout,"\n");
                        getstr("Press [RETURN] to continue or  X  to stop",lin2);
                        if(toupper(*lin2) == 'X') {
                                fclose(file);
                                return;
                        }
                        else
                                nlines = 0;
                }
        }
        fclose(file);
        fprintf(stdout,"\n");
        getstr("Press [RETURN] to continue",lin2);
}


sint parse_params(Boolean xmenus, char **args,OPTPTR opt,ALNPTR mult_aln,Boolean interactive)
{
	sint i,j,temp;
	sint cl_error_code=0;
	sint test_pairwise;
        char path[FILENAMELEN];
        char filename[FILENAMELEN];
	char *param_arg[MAXARGS+1];
	sint numparams;
	Boolean use_tree_file=FALSE,use_tree1_file=FALSE,use_tree2_file=FALSE;
	char     clustal_tree_name[FILENAMELEN+1]="";
	char     dist_tree_name[FILENAMELEN+1]="";
	char 	phylip_tree_name[FILENAMELEN+1]="";
	char 	bionj_tree_name[FILENAMELEN+1]="";
	char 	nexus_tree_name[FILENAMELEN+1]="";
	char *cmd_line_type[] = {
                " ",
                "=n ",
                "=f ",
                "=string ",
                "=filename ",
                ""};



	Boolean do_align, do_convert, do_align_only, do_tree_only, do_tree, do_boot, do_profile, do_something;

	if(args==NULL) {
		interactive=TRUE;
		return;
	}

	do_align = do_convert = do_align_only = do_tree_only = do_tree = do_boot = do_profile = do_something = FALSE;

	filename[0]=EOS;

    	numparams = check_param(args,param_arg,filename);

	if(sethelp != -1) {
		get_help('9',FALSE);
		exit(1);
	}

	if(setoptions != -1) {
		fprintf(stdout,"clustalw option list:-\n");
		for (i=0;cmd_line_verb[i].str[0] != '\0';i++) {
			fprintf(stdout,"\t\t%c%s%s",COMMANDSEP,cmd_line_verb[i].str,cmd_line_type[cmd_line_verb[i].type]);
			if (cmd_line_verb[i].type == OPTARG) {
				if (cmd_line_verb[i].arg[0][0] != '\0')
					fprintf(stdout,"=%s",cmd_line_verb[i].arg[0]);
				for (j=1;cmd_line_verb[i].arg[j][0] != '\0';j++)
					fprintf(stdout," OR %s",cmd_line_verb[i].arg[j]);
			}
			fprintf(stdout,"\n");
		}
		for (i=0;cmd_line_file[i].str[0] != '\0';i++) {
			fprintf(stdout,"\t\t%c%s%s",COMMANDSEP,cmd_line_file[i].str,cmd_line_type[cmd_line_file[i].type]);
			if (cmd_line_file[i].type == OPTARG) {
				if (cmd_line_file[i].arg[0][0] != '\0')
					fprintf(stdout,"=%s",cmd_line_file[i].arg[0]);
				for (j=1;cmd_line_file[i].arg[j][0] != '\0';j++)
					fprintf(stdout," OR %s",cmd_line_file[i].arg[j]);
			}
			fprintf(stdout,"\n");
		}
		for (i=0;cmd_line_para[i].str[0] != '\0';i++) {
			fprintf(stdout,"\t\t%c%s%s",COMMANDSEP,cmd_line_para[i].str,cmd_line_type[cmd_line_para[i].type]);
			if (cmd_line_para[i].type == OPTARG) {
				if (cmd_line_para[i].arg[0][0] != '\0')
					fprintf(stdout,"=%s",cmd_line_para[i].arg[0]);
				for (j=1;cmd_line_para[i].arg[j][0] != '\0';j++)
					fprintf(stdout," OR %s",cmd_line_para[i].arg[j]);
			}
			fprintf(stdout,"\n");
		}
		exit(1);
	}


/*****************************************************************************/
/*  Check to see if sequence type is explicitely stated..override ************/
/* the automatic checking (DNA or Protein).   /type=d or /type=p *************/
/*****************************************************************************/
	if(settype != -1)
		if(strlen(param_arg[settype])>0) {
			temp = find_match(param_arg[settype],type_arg,2);
			if(temp == 0) {
				opt->explicit_type = 2;
				info("Sequence type explicitly set to Protein");
			}
			else if(temp == 1) {
				opt->explicit_type = 1;
				info("Sequence type explicitly set to DNA");
			}
			else
				fprintf(stdout,"\nUnknown sequence type %s\n",
				param_arg[settype]);
		}


/**************************************************/
/*  Look for /infile=file.ext on the command line */
/**************************************************/

	if(setinfile != -1) {
		if(strlen(param_arg[setinfile]) <= 0) {
			error("Bad sequence file name");
			exit(1);
		}
		strcpy(filename, param_arg[setinfile]);
	}

	if(filename[0] != EOS) {
		seq_input(filename,opt->explicit_type,FALSE,mult_aln);
		if(mult_aln->nseqs < 2) {
                        if(mult_aln->nseqs < 0) cl_error_code = 2;
                        else if(mult_aln->nseqs == 0) cl_error_code = 3;
                        else cl_error_code = 4;
                	fprintf(stdout,
			"\nNo. of seqs. read = %d. No alignment!\n",(pint)mult_aln->nseqs);
			exit(cl_error_code);
		}
		do_something = TRUE;
	}

	set_optional_param(param_arg,opt,mult_aln->dnaflag);

/*********************************************************/
/* Look for /profile1=file.ext  AND  /profile2=file2.ext */
/* You must give both file names OR neither.             */
/*********************************************************/

	if(setprofile1 != -1) {
		if(strlen(param_arg[setprofile1]) <= 0) {
			error("Bad profile 1 file name");
			exit(1);
		}
		strcpy(filename, param_arg[setprofile1]);
		profile_input(filename,1,opt->explicit_type,mult_aln);
		if(mult_aln->prf1.nseqs <= 0) {
                        if(mult_aln->nseqs<0) cl_error_code=2;
                        else if(mult_aln->nseqs==0) cl_error_code=3;
                        exit(cl_error_code);
		}
	}

	if(setprofile2 != -1) {
		if(strlen(param_arg[setprofile2]) <= 0) {
			error("Bad profile 2 file name");
			exit(1);
		}
		if(mult_aln->prf1.nseqs==0) {
			error("Only 1 profile file (profile 2) specified.");
			exit(1);
		}
		strcpy(filename, param_arg[setprofile2]);
		profile_input(filename,2,opt->explicit_type,mult_aln);
		if(mult_aln->prf2.nseqs > 0) 
			do_something = do_profile = TRUE;
		else {
                        if(mult_aln->prf2.nseqs<0) cl_error_code=2;
                        else if(mult_aln->prf2.nseqs==0) cl_error_code=3;
                        error("No sequences read from profile 2");
                        exit(cl_error_code);
		}
	}

/*************************************************************************/
/* Look for /tree or /bootstrap or /align or /usetree ******************/
/*************************************************************************/

	if (setbatch != -1)
		interactive=FALSE;

	if (setinteractive != -1) 
		interactive=TRUE;

	if (interactive) {
		settree = -1;
		setbootstrap = -1;
		setalign = -1;
		setusetree = -1;
		setusetree1 = -1;
		setusetree2 = -1;
		setnewtree = -1;
		setconvert = -1;
	}

	if(settree != -1 )
		if(mult_aln->nseqs<=0) {
			error("Cannot draw tree.  No input alignment file");
			exit(1);
		}
		else 
			do_tree = TRUE;

	if(setbootstrap != -1)
		if(mult_aln->nseqs<=0) {
			error("Cannot bootstrap tree. No input alignment file");
			exit(1);
		}
		else {
			temp = 0;
			if(param_arg[setbootstrap] != NULL)
				 if (sscanf(param_arg[setbootstrap],"%d",&temp)!=1) {
                         fprintf(stdout,"Bad option for /bootstrap (must be integer)\n");
                         temp = 0;
                    };
			if(temp > 0)          bstree_opt.boot_ntrials = temp;
			do_boot = TRUE;
		}

	if(setalign != -1)
		if(mult_aln->nseqs<=0) {
			error("Cannot align sequences.  No input file");
			exit(1);
		}
		else 
			do_align = TRUE;

	if(setconvert != -1)
		if(mult_aln->nseqs<=0) {
			error("Cannot convert sequences.  No input file");
			exit(1);
		}
		else 
			do_convert = TRUE;
 
	if(setusetree != -1)
		if(mult_aln->nseqs<=0) {
			error("Cannot align sequences.  No input file");
			exit(1);
		}
		else  {
		        if(strlen(param_arg[setusetree]) == 0) {
				error("Cannot align sequences.  No tree file specified");
				exit(1);
		        }
                        else {
			        strcpy(phylip_tree_name, param_arg[setusetree]);
				strcpy(mult_aln->treename,phylip_tree_name);
		        }
		        do_align_only = TRUE;
		}

	if(setnewtree != -1)
		if(mult_aln->nseqs<=0) {
			error("Cannot align sequences.  No input file");
			exit(1);
		}
		else  {
		        if(strlen(param_arg[setnewtree]) == 0) {
				error("Cannot align sequences.  No tree file specified");
				exit(1);
		        }
                        else {
			        strcpy(phylip_tree_name, param_arg[setnewtree]);
				strcpy(mult_aln->treename,phylip_tree_name);
		        }
			do_tree_only = TRUE;
		}
 
	if(setusetree1 != -1)
		if(mult_aln->prf1.nseqs<=0) {
			error("Cannot align profiles.  No input file");
			exit(1);
		}
		else if(opt->prf_opt->profile_type == SEQUENCE) {
			error("Invalid option /usetree1.");
			exit(1);
		}
		else  {
		        if(strlen(param_arg[setusetree1]) == 0) {
				error("Cannot align profiles.  No tree file specified");
				exit(1);
		        }
                        else {
			        strcpy(mult_aln->prf1.treename, param_arg[setusetree1]);
		        }
			use_tree1_file = TRUE;
		}

	if(setusetree2 != -1)
		if(mult_aln->nseqs-mult_aln->prf1.nseqs<=0) {
			error("Cannot align profiles.  No input file");
			exit(1);
		}
		else if(opt->prf_opt->profile_type == SEQUENCE) {
			error("Invalid option /usetree2.");
			exit(1);
		}
		else  {
		        if(strlen(param_arg[setusetree2]) == 0) {
				error("Cannot align profiles.  No tree file specified");
				exit(1);
		        }
                        else {
			        strcpy(mult_aln->prf2.treename, param_arg[setusetree2]);
		        }
			use_tree2_file = TRUE;
		}

/*** ? /pwaddmotif */
        if(setpwaddmotif != -1)
		pw_opt.add_motif = TRUE;

	if( (!do_tree) && (!do_boot) && (mult_aln->nseqs>1) && (!do_profile) && (!do_align_only) && (!do_tree_only) && (!do_convert)) 
		do_align = TRUE;

/*** ? /quicktree  */
        if(setquicktree != -1)
		pw_opt.quick_pairalign = TRUE;

/*** ? /treealgo=nj or bionj  */
        if(settreealgo != -1)
                if(strlen(param_arg[settreealgo]) > 0) {
     			temp = find_match(param_arg[settreealgo],treealgo_arg,2);
                        if(temp == 0)
                                pw_opt.treealgo = NJ;
                        else if(temp == 1)
                                pw_opt.treealgo = BIONJ;
                        else
                                fprintf(stdout,"\nUnknown TREEALGO type: %s\n",
                                param_arg[settreealgo]);
                }

	if(xmenus && setbatch==-1) {
		interactive=FALSE;
		return;
	}

	if(interactive) {
		return;
	}


	if(!do_something) {
		error("No input file(s) specified");
		exit(1);
	}




/****************************************************************************/
/* Now do whatever has been requested ***************************************/
/****************************************************************************/

	if(do_profile) {
	  if (opt->prf_opt->profile_type == PROFILE) 
	    {
	      test_pairwise=0;

	      /******************************************************************/
	      /*telling the slaves processes that there will pairwise alignments*/
	      /******************************************************************/
	      MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);

	      profile_align(mult_aln,*opt,use_tree1_file,use_tree2_file,FALSE);
	    }
	  else 
	    {
	      test_pairwise=1;

	      /******************************************************************/
	      /*telling the slaves processes that there will pairwise alignments*/
	      /******************************************************************/
	      MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);
	      
	      new_sequence_align(mult_aln,*opt,use_tree1_file,FALSE);
	    }
	}

	else if(do_align) {
	  test_pairwise=1;

	  /******************************************************************/
	  /*telling the slaves processes that there will pairwise alignments*/
	  /******************************************************************/
	  MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);align(mult_aln,*opt,FALSE);
          
	  create_alignment_output(*mult_aln,0,mult_aln->nseqs,*opt->alnout_opt,*opt->ss_opt);
	}
        else if(do_convert) {
	  test_pairwise=0;
	  
	  /***********************************************************************/
	  /*telling the slaves processes that there will be no pairwise alignment*/
	  /***********************************************************************/
	  MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);
	  
	  if(!open_alignment_output(mult_aln->filename,opt->alnout_opt)) exit(1);
	  if(mult_aln->treename[0]!='\0') set_output_to_tree(mult_aln);
	  create_alignment_output(*mult_aln,0,mult_aln->nseqs,*opt->alnout_opt,*opt->ss_opt);
        }
        else if (do_align_only) {
	  test_pairwise=0;
	  
	  /***********************************************************************/
	  /*telling the slaves processes that there will be no pairwise alignment*/
	  /***********************************************************************/
	  MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);
	  
	  align_from_tree(mult_aln,*opt,FALSE);
	}
	else if(do_tree_only) {
	  test_pairwise=1;
	  
	  /******************************************************************/
	  /*telling the slaves processes that there will pairwise alignments*/
	  /******************************************************************/
	  MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);

	  make_tree(mult_aln,*opt,FALSE);
	}
	else if(do_tree) {
	  test_pairwise=0;

	  /***********************************************************************/
	  /*telling the slaves processes that there will be no pairwise alignment*/
	  /***********************************************************************/
	  MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);

	  phylogenetic_tree(*mult_aln,phylip_tree_name,clustal_tree_name,dist_tree_name,bionj_tree_name,nexus_tree_name,FALSE,tree_opt);
	}
	else if(do_boot) {
	  test_pairwise=0;

	  /***********************************************************************/
	  /*telling the slaves processes that there will be no pairwise alignment*/
	  /***********************************************************************/
	  MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);

	  bootstrap_tree(*mult_aln,phylip_tree_name,clustal_tree_name,nexus_tree_name,FALSE,tree_opt,bstree_opt);
	}
	fprintf(stdout,"\n");

	/*modif nico*/
	return NOW_GO_HOME;

/*******whew!***now*go*home****/
}

static void set_output_to_tree(ALNPTR mult_aln)
{
        IN_TREEPTR   itree;
        sint    nsets;
        sint    **sets;
        sint    i,j,set;
	sint	tx;
        sint    status;

/* get the guide tree (tree should be in phylip format) */

        if (mult_aln->nseqs >= 2) {
                itree=(IN_TREEPTR)ckalloc(sizeof(IN_TREE));
                status = read_tree(mult_aln->treename, mult_aln->seqs, (sint)0, mult_aln->nseqs
,itree);
                if (status < 0) {
			fatal("Cannot read tree from %s\n",mult_aln->treename);
			return;
		}
        }

        sets = (sint **) ckalloc( (mult_aln->nseqs+1) * sizeof (sint *) );
        for(i=0;i<mult_aln->nseqs;i++)
                sets[i] = (sint *)ckalloc( (mult_aln->nseqs+1) * sizeof (sint) );

        nsets=create_sets(mult_aln->nseqs,sets,itree);


	tx=0;
        for (i=0;i<mult_aln->nseqs;i++)
                mult_aln->seqs[i].output_index = -1;
        for(set=0;set<nsets;set++) {
                for (i=0;i<mult_aln->nseqs;i++) {
                        if (sets[set][i] != 0) {
                                if(mult_aln->seqs[i].output_index==(-1)) {
                                        mult_aln->seqs[i].output_index = tx++;
                                }
                        }
                }
        }
}

static sint find_match(char *probe, char *list[], sint n)
{
	sint i,j,len;
	sint count,match=0;

	len = (sint)strlen(probe);
	for (i=0;i<len;i++) {
		count = 0;
		for (j=0;j<n;j++) {
			if (probe[i] == list[j][i]) {
				match = j;
				count++;
			}
		}
		if (count == 0) return((sint)-1);
		if (count == 1) return(match);
	}
	return((sint)-1);
}



