#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "clustalw.h"
#include <mpi.h>
#include "pairalign.h"

/***************************************/
/*                                     */
/*Main procedure of the slave processes*/
/*                                     */
/***************************************/

void main_processes_slaves(int argc,char **argv)
{
  /************************/
  /*variables declarations*/
  /************************/

  PW_OPT pw_opt;
  MPI_Status status;
  int i,j,irank,int_scale,gop_scale,gep_scale,istart,inseq,jstart,jnseq;
  int alnstart,alnstop,length_gather,nb_alignments,test_pairwise,nanchors;
  TMAT_SI_SJ *tmat_si_sj;
  MOTIF *motifs,motif;

  /*pw_opt structure*/
  int pw_opt_array_of_blocklengths[14]={1,1,1,1,1,1,1,FILENAMELEN+1,1,FILENAMELEN+1,
					 FILENAMELEN+1,FILENAMELEN+1,1,1};
  MPI_Aint pw_opt_array_of_displacements[14];
  MPI_Datatype pw_opt_array_of_types[14]={MPI_FLOAT,MPI_FLOAT,MPI_FLOAT,MPI_FLOAT,MPI_FLOAT,
					  MPI_INT,MPI_INT,MPI_CHAR,MPI_INT,MPI_CHAR,MPI_CHAR,
					  MPI_CHAR,MPI_CHAR,MPI_CHAR};
  MPI_Datatype pw_opt_packettype;
  /*end of pw_opt structure*/

  /*comp_matrix structure*/
  MPI_Datatype score_type;
  int comp_matrix_array_of_blocklengths[4]={1,1,27,1};
  MPI_Aint comp_matrix_array_of_displacements[4];
  MPI_Datatype comp_matrix_array_of_types[4];
  MPI_Datatype comp_matrix_packettype;
  COMP_MATRIX comp_matrix;
  /*end of comp_matrix structure*/

  /*motif structure*/
  int motif_array_of_blocklengths[6]={1,1,1,1,1,1};
  MPI_Aint motif_array_of_displacements[6];
  MPI_Datatype motif_array_of_types[6]={MPI_INT,MPI_INT,MPI_INT,MPI_INT,MPI_INT,MPI_INT};
  MPI_Datatype motif_packettype;
  /*end of motif structure*/

  /*tmat_si_sj structure*/
  int tmat_si_sj_array_of_blocklengths[3]={1,1,1};
  MPI_Aint tmat_si_sj_array_of_displacements[3];
  MPI_Datatype tmat_si_sj_array_of_types[3]={MPI_INT,MPI_INT,MPI_DOUBLE};
  MPI_Datatype tmat_si_sj_packettype;
  /*end of tmat_si_sj structure*/

  ALN mult_aln;

  /*******************************/
  /*end of variables declarations*/
  /*******************************/
  
  MPI_Comm_rank(MPI_COMM_WORLD,&irank);

  /*************************************************************************************/
  /*reception from process master of the order to do pairwise alignements (1) or no (0)*/
  /*************************************************************************************/

  MPI_Bcast(&test_pairwise,1,MPI_INT,0,MPI_COMM_WORLD);

  if(test_pairwise==1)
    {
      MPI_Bcast(&nanchors,1,MPI_INT,0,MPI_COMM_WORLD);

      /**********************************************/
      /*reception from process master of the anchors*/
      /**********************************************/
      if(nanchors>0)
	{
	  /*memory allocation*/
	  motifs=(MOTIF *)malloc(sizeof(MOTIF)*nanchors);
	  /*end of memory allocation*/

	  motif_array_of_displacements[0]=0;
	  motif_array_of_displacements[1]=
	    (char *)&motif.pos1-(char *)&motif.seq1;
	  motif_array_of_displacements[2]=
	    (char *)&motif.seq2-(char *)&motif.seq1;
	  motif_array_of_displacements[3]=
	    (char *)&motif.pos2-(char *)&motif.seq1;
	  motif_array_of_displacements[4]=
	    (char *)&motif.len-(char *)&motif.seq1;
	  motif_array_of_displacements[5]=
	    (char *)&motif.weight-(char *)&motif.seq1;
	  
	  MPI_Type_struct(6,motif_array_of_blocklengths,motif_array_of_displacements,
			  motif_array_of_types,&motif_packettype);
	  MPI_Type_commit(&motif_packettype);
	  
	  MPI_Bcast(motifs,nanchors,motif_packettype,0,MPI_COMM_WORLD);
	      
	}
      else
	{
	  motifs=NULL;
	}

      /*******************************************************/
      /*reception from process master of the pw_opt structure*/
      /*******************************************************/
      
      pw_opt_array_of_displacements[0]=0;
      pw_opt_array_of_displacements[1]=
	(char *)&pw_opt.dna_ge_penalty-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[2]=
	(char *)&pw_opt.prot_go_penalty-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[3]=
	(char *)&pw_opt.prot_ge_penalty-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[4]=
	(char *)&pw_opt.transition_weight-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[5]=
	(char *)&pw_opt.matnum-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[6]=
	(char *)&pw_opt.treealgo-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[7]=
	pw_opt.mtrxname-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[8]=
	(char *)&pw_opt.dnamatnum-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[9]=
	pw_opt.dnamtrxname-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[10]=
	pw_opt.usermtrxname-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[11]=
	pw_opt.dnausermtrxname-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[12]=
	(char *)&pw_opt.add_motif-(char *)&pw_opt.dna_go_penalty;
      pw_opt_array_of_displacements[13]=
	(char *)&pw_opt.quick_pairalign-(char *)&pw_opt.dna_go_penalty;
      
      MPI_Type_struct(14,pw_opt_array_of_blocklengths,pw_opt_array_of_displacements,
		      pw_opt_array_of_types,&pw_opt_packettype);
      MPI_Type_commit(&pw_opt_packettype);
      
      MPI_Bcast(&pw_opt,1,pw_opt_packettype,0,MPI_COMM_WORLD);
      
      /************************************************************/
      /*reception from process master of the comp_matrix structure*/
      /************************************************************/
      
      MPI_Type_vector(729,1,1,MPI_INT,&score_type);
      MPI_Type_commit(&score_type);
      
      comp_matrix_array_of_types[0]=MPI_INT;
      comp_matrix_array_of_types[1]=score_type;
      comp_matrix_array_of_types[2]=MPI_INT;
      comp_matrix_array_of_types[3]=MPI_INT;
      
      comp_matrix_array_of_displacements[0]=0;
      comp_matrix_array_of_displacements[1]=
	(char *)comp_matrix.score-(char *)&comp_matrix.format;
      comp_matrix_array_of_displacements[2]=
	(char *)comp_matrix.gapscore-(char *)&comp_matrix.format;
      comp_matrix_array_of_displacements[3]=
	(char *)&comp_matrix.mat_avscore-(char *)&comp_matrix.format;
      
      MPI_Type_struct(4,comp_matrix_array_of_blocklengths,comp_matrix_array_of_displacements,
		      comp_matrix_array_of_types,&comp_matrix_packettype);
      MPI_Type_commit(&comp_matrix_packettype);
      
      MPI_Bcast(&comp_matrix,1,comp_matrix_packettype,0,MPI_COMM_WORLD);
      
      /************************************************************************/
      /*reception from process master of the number of sequences to be aligned*/
      /************************************************************************/
      
      MPI_Bcast(&mult_aln.nseqs,1,MPI_INT,0,MPI_COMM_WORLD);
      
      /*memory allocation*/
      mult_aln.seqs=(SEQPTR)malloc(sizeof(SEQ)*mult_aln.nseqs);
      /*end of memory allocation*/
      
      /**********************************************/
      /*reception from process master of the dnaflag*/
      /**********************************************/
      
      MPI_Bcast(&(mult_aln.dnaflag),1,MPI_INT,0,MPI_COMM_WORLD);
      
      /***************************************************************************************/
      /*reception from process master of each sequence separately and before it of its length*/
      /***************************************************************************************/
      
      for(i=0;i<mult_aln.nseqs;i++)
	{
	  MPI_Bcast(&(mult_aln.seqs[i].len),1,MPI_INT,0,MPI_COMM_WORLD);
	  
	  /*memory allocation*/
	  mult_aln.seqs[i].data=(char *)malloc(sizeof(char)*mult_aln.seqs[i].len);
	  /*end of memory allocation*/
	  
	  MPI_Bcast(mult_aln.seqs[i].data,mult_aln.seqs[i].len,MPI_CHAR,0,MPI_COMM_WORLD);
	}
      
      /*********************************************************************/
      /*reception from process master of int_scale, gop_scale and gep_scale*/
      /*********************************************************************/
      
      MPI_Bcast(&int_scale,1,MPI_INT,0,MPI_COMM_WORLD);
      MPI_Bcast(&gop_scale,1,MPI_FLOAT,0,MPI_COMM_WORLD);
      MPI_Bcast(&gep_scale,1,MPI_FLOAT,0,MPI_COMM_WORLD);
      MPI_Bcast(&istart,1,MPI_INT,0,MPI_COMM_WORLD);
      MPI_Bcast(&inseq,1,MPI_INT,0,MPI_COMM_WORLD);
      MPI_Bcast(&jstart,1,MPI_INT,0,MPI_COMM_WORLD);
      MPI_Bcast(&jnseq,1,MPI_INT,0,MPI_COMM_WORLD);
      
      /*******************************************************************/
      /*reception from process master alnstart, alnstop and length_gather*/
      /*******************************************************************/
      
      MPI_Scatter(NULL,0,MPI_INT,&alnstart,1,MPI_INT,0,MPI_COMM_WORLD);
      MPI_Scatter(NULL,0,MPI_INT,&alnstop,1,MPI_INT,0,MPI_COMM_WORLD);
      
      MPI_Bcast(&length_gather,1,MPI_INT,0,MPI_COMM_WORLD);
      
      /*memory allocation*/
      tmat_si_sj=(TMAT_SI_SJ *)malloc(sizeof(TMAT_SI_SJ)*length_gather);
      /*end of memory allocation*/
      
      if(length_gather!=(alnstop-alnstart))
	{
	  tmat_si_sj[length_gather-1].si=-1.0;
	}
      
      if(alnstart!=alnstop)
	{
	  
	  /*seeking of the si and sj to be aligned*/
	  seek_si_sj_to_calculate(istart,inseq,jstart,jnseq,alnstart,alnstop,tmat_si_sj);
	  
	  /*pairwises alignments*/
	  subpairalign(mult_aln,alnstart,alnstop,tmat_si_sj,pw_opt,motifs,mult_aln.nseqs,
		       comp_matrix,int_scale,gop_scale,gep_scale);
	}      
      
      /*************************************/
      /*sending to the master of tmat_si_sj*/
      /*************************************/
      
      tmat_si_sj_array_of_displacements[0]=0;
      tmat_si_sj_array_of_displacements[1]=
	(char *)&(tmat_si_sj[0].sj)-(char *)&(tmat_si_sj[0].si);
      tmat_si_sj_array_of_displacements[2]=
	(char *)&(tmat_si_sj[0].tmat)-(char *)&(tmat_si_sj[0].si);
      
      MPI_Type_struct(3,tmat_si_sj_array_of_blocklengths,tmat_si_sj_array_of_displacements,
		      tmat_si_sj_array_of_types,&tmat_si_sj_packettype);
      MPI_Type_commit(&tmat_si_sj_packettype);
      
      MPI_Gather(tmat_si_sj,length_gather,tmat_si_sj_packettype,NULL,0,
		 tmat_si_sj_packettype,0,MPI_COMM_WORLD);
      
      /*memory freeing*/
      for(i=0;i<mult_aln.nseqs;i++)
	{
	  free(mult_aln.seqs[i].data);
	}
      free(mult_aln.seqs);
      free(tmat_si_sj);

      if(nanchors>0)
	{
	  free(motifs);
	}
      /*end of memory freeing*/
    }
}

