#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "clustalw.h"

#define MAX(a,b) ((a)>(b)?(a):(b))
#define MIN(a,b) ((a)<(b)?(a):(b))

/*
 *       Prototypes
 */

static sint count_gaps(SEQ *seqs,sint s1, sint s2, sint l);

/*
 *       Global Variables
 */
extern sint   blosum45mt[];
static sint   debug;

void aln_score(ALN mult_aln,float gap_open)
{
  static sint maxres;
  static sint  s1,s2;
  static char  c1,c2;
  static sint    ngaps;
  static sint    i,l1,l2;
  static lint    score;
  static COMP_MATRIX matrix;

/* calculate an overall score for the alignment by summing the
scores for each pairwise alignment */

  maxres = get_cl_matrix(FALSE,blosum45mt, NULL, TRUE, 100, &matrix);
  if (maxres == 0)
    {
       fprintf(stdout,"Error: matrix blosum30 not found\n");
       return;
    }

  score=0;
  for (s1=0;s1<mult_aln.nseqs;s1++)
   {
    for (s2=0;s2<s1;s2++)
      {

        l1 = mult_aln.seqs[s1].len;
        l2 = mult_aln.seqs[s2].len;
        for (i=0;i<l1 && i<l2;i++)
          {
            c1 = mult_aln.seqs[s1].data[i];
            c2 = mult_aln.seqs[s2].data[i];
            if (isalpha(c1) && isalpha(c2))
                score += matrix.score[c1-'a'][c2-'a'];
          }

        ngaps = count_gaps(mult_aln.seqs,s1, s2, l1);

        score -= 100 * gap_open * ngaps;

      }
   }

  score /= 100;

  info("Alignment Score %d", (pint)score);

}

static sint count_gaps(SEQ *seqs,sint s1, sint s2, sint l)
{
    sint i, g;
    sint q, r, *Q, *R;


    Q = (sint *)ckalloc((l+2) * sizeof(sint));
    R = (sint *)ckalloc((l+2) * sizeof(sint));

    Q[0] = R[0] = g = 0;

    for (i=0;i<l;i++)
      {
         if (!isalpha(seqs[s1].data[i])) q = 1;
         else q = 0;
         if (!isalpha(seqs[s2].data[i])) r = 1;
         else r = 0;

         if (((Q[i-1] <= R[i-1]) && (q != 0) && (1-r != 0)) ||
             ((Q[i-1] >= R[i-1]) && (1-q != 0) && (r != 0)))
             g += 1;
         if (q != 0) Q[i] = Q[i-1]+1;
         else Q[i] = 0;

         if (r != 0) R[i] = R[i-1]+1;
         else R[i] = 0;
     }
     
   Q=ckfree((void *)Q);
   R=ckfree((void *)R);

   return(g);
}
          

