#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "clustalw.h"


/*
 *       Prototypes
 */
static void align_seq_to_prf(ALNPTR mult_aln,double **tmat,sint *aligned,sint seq1,MULT_OPT mult_opt,sint output_order,Boolean use_maxid,MOTIF *motifs);
static void align_sequences_to_prf(ALNPTR mult_aln,double **tmat,sint *aligned,MULT_OPT mult_opt,sint output_order,Boolean use_maxid,MOTIF *motifs);
static sint create_upgma_sets(sint nseqs,FILE *tree,sint **sets);
static void group_seqs(INODEPTR p, sint *next_groups, sint *nsets, sint **sets,IN_TREEPTR itree);
static void mark_group1(INODEPTR p, sint *groups, IN_TREEPTR itree);
static void mark_group2(INODEPTR p, sint *groups, IN_TREEPTR itree);

static void reset_align(sint nseqs, SEQ *seqs,Boolean reset_new, Boolean reset_all);
static void reset_prf(sint prf_no,ALNPTR mult_aln,Boolean reset_new, Boolean reset_all);



/*
 *       Global Variables
 */

static sint     debug;

/*
   Do a progressive multiple alignment.
	mult_aln	the multiple alignment (sequences still unaligned usually!)
	mult_opt	multiple alignment options
	output_order	=ALIGNED if sequences to be output in alignment order, 
	             	=TREEORDER if sequences to be output in input tree order, 
			=INPUT if sequences to be output in same order as input

   Returns the number of sequences aligned
*/

sint malign(ALNPTR mult_aln,MULT_OPT mult_opt,MOTIF *motifs,sint output_order)
{
	FILE *up_tree;
	Boolean use_maxid;
	sint *aligned;
	sint ix,tx;
	sint 	*maxid;
	sint 	i,j,set;
	sint 	status,entries;
	sint	nsets;
	sint	**sets;
	lint	score = 0;
	sint	*seq_weight;
	IN_TREEPTR   itree;
	double **tmat;


	info("Start of Multiple Alignment");

/* get the guide tree (tree should be in phylip format) */

	if (mult_aln->nseqs >= 2) {
		itree=(IN_TREEPTR)ckalloc(sizeof(IN_TREE));
       		status = read_tree(mult_aln->treename, mult_aln->seqs, (sint)0, mult_aln->nseqs,itree);
       		if (status < 0) return((sint)0);
     	}

	if(status==0) use_maxid=TRUE;
	else use_maxid=FALSE;

/* group the sequences according to their relative divergence */

        sets = (sint **) ckalloc( (mult_aln->nseqs+1) * sizeof (sint *) );
        for(i=0;i<mult_aln->nseqs;i++)
           	sets[i] = (sint *)ckalloc( (mult_aln->nseqs+1) * sizeof (sint) );

        nsets=create_sets(mult_aln->nseqs,sets,itree);
/*
#ifdef VMS
	if((up_tree=fopen("upgma.tree","r","rat=cr","rfm=var"))==NULL) {
#else
        if((up_tree=fopen("upgma.tree","r"))==NULL) {
#endif
		
		error("Cannot open upgma tree [%s]","upgma_tree");
		return;
	}
	nsets=create_upgma_sets(mult_aln->nseqs,up_tree,sets);
*/
        info("There are %d groups",(pint)nsets);

/* calculate sequence weights according to branch lengths of the tree -
   weights are normalised to sum to 100 */

	seq_weight = calc_seq_weights((sint)0, mult_aln->nseqs, itree,mult_opt.no_weights);
	for(i=0;i<mult_aln->nseqs;i++)
		mult_aln->seqs[i].weight=seq_weight[i];

/* calculate tmat matrix containing percent similarities for all pairs of sequences */

	tmat = calc_similarities(mult_aln->seqs,mult_aln->nseqs,itree);
	if (tmat == NULL) return((sint)0);

/* for each sequence, find the most closely related sequence */

   	maxid = (sint *)ckalloc( (mult_aln->nseqs+1) * sizeof (sint));
   	for (i=0;i<mult_aln->nseqs;i++) {
         	maxid[i] = -1;
         	for (j=0;j<mult_aln->nseqs;j++) 
           	if (j!=i && maxid[i] < tmat[i][j]) maxid[i] = tmat[i][j];
     	}


/* clear the memory used for the phylogenetic tree */

        if (mult_aln->nseqs >= 2)
             	free_tree(itree);

/* start the multiple alignments.........  */

        info("Aligning...");

/* first pass, align closely related sequences first.... */
/* more divergent sequences will be 'delayed' until the end */

        ix = tx = 0;
        aligned = (sint *)ckalloc( (mult_aln->nseqs+1) * sizeof (sint) );
        for (i=0;i<mult_aln->nseqs;i++) aligned[i] = 0;

        if (output_order==INPUT) {
        	for (i=0;i<mult_aln->nseqs;i++) 
        		mult_aln->seqs[i].output_index = i;
        }
/*
for(set=0;set<nsets;set++) {
for (i=0;i<mult_aln->nseqs;i++) {
printf("%d ",sets[set][i]);
}
printf("\n");
}
*/
        if (output_order==TREEORDER) {
        	for (i=0;i<mult_aln->nseqs;i++) 
        		mult_aln->seqs[i].output_index = -1;
        	for(set=0;set<nsets;set++) {
			for (i=0;i<mult_aln->nseqs;i++) {
				if (sets[set][i] != 0) {
					if(mult_aln->seqs[i].output_index==(-1)) {
						mult_aln->seqs[i].output_index = tx++;
					}
				}
			}
		}
	}

        for(set=0;set<nsets;set++) {
		entries=0;
		for (i=0;i<mult_aln->nseqs;i++) {
			if (sets[set][i] != 0) {
				if (maxid[i] > mult_opt.divergence_cutoff) {
                    			entries++;
                    			if  (aligned[i] == 0) {
                          			if (output_order==ALIGNED)
							mult_aln->seqs[i].output_index = ix;
                          			aligned[i] = 1;
                              			ix++;
					}
				}
				else sets[set][i]=0;
			}
		}

        	if(entries > 0)
			score = prfalign(0,mult_aln,sets[set],tmat,use_maxid,mult_opt,motifs);
          	else score=0.0;


/* negative score means fatal error... exit now!  */

		if (score < 0) return(-1);

		if ((entries > 0) && (score > 0))
             		info("Group %d: Sequences:%4d      Score:%d",
             				(pint)set+1,(pint)entries,(pint)score);
          	else
             		info("Group %d:                     Delayed",
             				(pint)set+1);
        }

        for (i=0;i<mult_aln->nseqs;i++)
          	sets[i]=ckfree((void *)sets[i]);
        sets=ckfree(sets);
	maxid=ckfree((void *)maxid);

/* second pass - align any 'delayed' divergent sequences..... */

	align_sequences_to_prf(mult_aln,tmat,aligned,mult_opt,output_order,use_maxid,motifs);

        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = ckfree(tmat[i]);
        tmat = ckfree(tmat);

	aligned=ckfree((void *)aligned);
	seq_weight=ckfree(seq_weight);

	if(mult_aln->dnaflag)
   		aln_score(*mult_aln,mult_opt.dna_gap_open);
	else
   		aln_score(*mult_aln,mult_opt.prot_gap_open);

	return(mult_aln->nseqs);

}

/*
    Align a sequence to a profile. Mult_aln in fact contains all the
    sequences, aligned or not.
	mult_aln	the multiple alignment (including unaligned sequences)
	tmat		pairwise comparison matrix
	aligned		=1 if sequences are in profile
			=0 if sequences are unaligned
	iseq		=sequence to be aligned to profile
	mult_opt	multiple alignment options
	output_order	=ALIGNED if sequences to be output in alignment order, 
	             	=TREEORDER if sequences to be output in input tree order, 
			=INPUT if sequences to be output in same order as input
*/
	
static void align_seq_to_prf(ALNPTR mult_aln,double **tmat,sint *aligned,sint iseq,MULT_OPT mult_opt,sint output_order,Boolean use_maxid,MOTIF *motifs)
{
	sint i,j,n;
	sint *group;
	lint score = 0;

	group = (sint *)ckalloc( (mult_aln->nseqs+1) * sizeof (sint));

        for (j=0,n=0;j<mult_aln->nseqs;j++)
         	if (aligned[j] != 0) {
               		group[j] = 1;
			n++;
	}
       	group[iseq] = 2;
        aligned[iseq] = 1;

        score = prfalign(2,mult_aln,group,tmat,FALSE,mult_opt,motifs);
        info("Sequence:%d     Score:%d",(pint)iseq+1,(pint)score);
        if (output_order == INPUT) {
        	mult_aln->seqs[iseq].output_index = iseq;
        }
        else if (output_order == ALIGNED)
        	mult_aln->seqs[iseq].output_index = n;

	group=ckfree((void *)group);
}

/*
    Align remaining sequences to a profile. Mult_aln in fact contains all the
    sequences, aligned or not.
	mult_aln	the multiple alignment (including unaligned sequences)
	tmat		pairwise comparison matrix
	aligned		=1 if sequences are in profile
			=0 if sequences are unaligned
	mult_opt	multiple alignment options
	output_order	=ALIGNED if sequences to be output in alignment order, 
	             	=TREEORDER if sequences to be output in input tree order, 
			=INPUT if sequences to be output in same order as input
*/
	
static void align_sequences_to_prf(ALNPTR mult_aln,double **tmat,sint *aligned,MULT_OPT mult_opt,sint output_order,Boolean use_maxid,MOTIF *motifs)
{
	sint i,j,ix;
	sint iseq,max,sum,*maxid;
	lint	score = 0;

        ix = 0;
        for (i=0;i<mult_aln->nseqs;i++) 
		if(aligned[i]==1) 
			ix++;

/* if we haven't aligned any sequences at all yet, make sure we align the
   two most closely related sequences first */

	if(ix==0) {
        	max = -1;
		iseq = 0;
        	for (i=0;i<mult_aln->nseqs;i++) {
             		for (j=i+1;j<mult_aln->nseqs;j++) {
                  		if (max < tmat[i][j]) {
                     			max = tmat[i][j];
                     			iseq = i;
                  		}
              		}
          	}
        	aligned[iseq]=1;
         	if (output_order == INPUT) 
            		mult_aln->seqs[iseq].output_index = iseq;
         	else if (output_order == ALIGNED)
            		mult_aln->seqs[iseq].output_index = 0;
		ix=1;
	}

	maxid=(sint *)ckalloc((mult_aln->nseqs+1)*sizeof(sint));
/* for each unaligned sequence, find it's closest pair amongst the aligned sequences.  */
    	while (ix < mult_aln->nseqs) {
		for (i=0;i<mult_aln->nseqs;i++) {
                	if (aligned[i] == 0) {
                     		maxid[i] = -1;
                     		for (j=0;j<mult_aln->nseqs;j++) 
                        		if ((maxid[i] < tmat[i][j]) && (aligned[j] != 0))
                            			maxid[i] = tmat[i][j];
                  	}
              	}

/* find the most closely related sequence to those already aligned */

            	max = -1;
	    	iseq = 0;
            	for (i=0;i<mult_aln->nseqs;i++) {
                	if ((aligned[i] == 0) && (maxid[i] > max)) {
                     		max = maxid[i];
                     		iseq = i;
                  	}
              	}


/* align this sequence to the existing alignment */

		align_seq_to_prf(mult_aln,tmat,aligned,iseq,mult_opt,output_order,use_maxid,motifs);
            	ix++;
      	}

	maxid=ckfree((void *)maxid);
}

/*
   Align sequences to an existing profile
	mult_aln	the multiple alignment 
	mult_opt	multiple alignment options
	output_order	=ALIGNED if sequences to be output in alignment order, 
	             	=TREEORDER if sequences to be output in input tree order, 
			=INPUT if sequences to be output in same order as input
	istart		the first sequence that will be aligned 
			(with sequences 0 to istart-1 in mult_aln)
    Returns number of sequences in alignment
*/
sint seqalign(ALNPTR mult_aln,MULT_OPT mult_opt,MOTIF *motifs,sint output_order,sint istart)
{
	Boolean use_maxid;
	sint *aligned, *tree_weight;
	sint 	i,j,status;
	IN_TREEPTR   itree;
	double **tmat;


	info("Start of Multiple Alignment");

/* get the guide tree */

	if (mult_aln->nseqs >= 2) {
		itree=(IN_TREEPTR)ckalloc(sizeof(IN_TREE));
       		status = read_tree(mult_aln->treename, mult_aln->seqs, (sint)0, mult_aln->nseqs,itree);
       		if (status < 0) return(0);
	}
	if(status==0) use_maxid=TRUE;
	else use_maxid=FALSE;

/* calculate sequence weights according to branch lengths of the tree */

	tree_weight = calc_seq_weights((sint)0, mult_aln->nseqs, itree,mult_opt.no_weights);
	for(i=0;i<mult_aln->nseqs;i++)
		mult_aln->seqs[i].weight=tree_weight[i];
	tree_weight=ckfree(tree_weight);
   
/* calculate tmat matrix containing percent similarities for all pairs of sequences */

	tmat = calc_similarities(mult_aln->seqs,mult_aln->nseqs,itree);
	if (tmat == NULL) return((sint)0);

/* clear the memory used for the phylogenetic tree */

        if (mult_aln->nseqs >= 2)
        	free_tree(itree);

        aligned = (sint *)ckalloc( (mult_aln->nseqs+1) * sizeof (sint) );
        for (i=0;i<istart;i++) {
		aligned[i] = 1;
		mult_aln->seqs[i].output_index = i;
        }
        for (i=istart;i<mult_aln->nseqs;i++) aligned[i] = 0;

	align_sequences_to_prf(mult_aln,tmat,aligned,mult_opt,output_order,use_maxid,motifs);

        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = ckfree(tmat[i]);
        tmat = ckfree(tmat);

	aligned=ckfree((void *)aligned);

	if(mult_aln->dnaflag)
   		aln_score(*mult_aln,mult_opt.dna_gap_open);
	else
   		aln_score(*mult_aln,mult_opt.prot_gap_open);

   	return(mult_aln->nseqs);

}

/*
   Do an initial alignment of profile 1 to profile 2
   In this case, we don't have pairwise distances or guide tree
   between the 2 profiles. This subroutine does a rough alignment
   (similar in function to the pairwise alignments for
   a multiple alignment) in order to calculate distances between
   each pair of sequences.
	mult_aln	the multiple alignment (actually contains 2 profiles here!)
	mult_opt	multiple alignment options

   Returns a pairwise sequence distance matrix
*/

double ** palign1(ALNPTR mult_aln,MULT_OPT mult_opt,MOTIF *motifs)  
{
	sint 	i,j,temp;
	sint 	entries;
	sint 	*group;
	float   dscore;
	lint	score;
	double **tmat;

	info("Start of Initial Alignment");

/* calculate sequence weights according to branch lengths of the tree -
   weights are normalised to sum to INT_SCALE_FACTOR */

	temp = INT_SCALE_FACTOR/mult_aln->nseqs;
	for (i=0;i<mult_aln->nseqs;i++) mult_aln->seqs[i].weight = temp;

/* do the initial alignment.........  */

	group = (sint *)ckalloc( (mult_aln->nseqs+1) * sizeof (sint));

	for(i=0; i<mult_aln->prf1.nseqs; i++)
		group[i] = 1;
	for(i=mult_aln->prf1.nseqs; i<mult_aln->prf1.nseqs+mult_aln->prf2.nseqs; i++)
		group[i] = 2;
	entries = mult_aln->nseqs;

        tmat = (double **) ckalloc( (mult_aln->nseqs+1) * sizeof (double *) );
        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = (double *)ckalloc( (mult_aln->nseqs+1) * sizeof (double) );
        for(i=0;i<mult_aln->nseqs;i++)
                for(j=0;j<mult_aln->nseqs;j++)
                        tmat[i][j]=0.0;


	score = prfalign(1,mult_aln,group,tmat,FALSE,mult_opt,motifs);
	info("Sequences:%d      Score:%d",(pint)entries,(pint)score);
	group=ckfree((void *)group);

	for (i=0;i<mult_aln->nseqs;i++) {
		for (j=i+1;j<mult_aln->nseqs;j++) {
			dscore = countid(mult_aln->seqs[i],mult_aln->seqs[j]);
			tmat[i][j] = ((double)100.0 - (double)dscore)/(double)100.0;
			tmat[j][i] = tmat[i][j];
		}
	}

	return(tmat);
}

/*
   Do an alignment of profile 1 to profile 2
	mult_aln	the multiple alignment (actually contains 2 profiles here!)
	tmat		pairwise distance matrix
	mult_opt	multiple alignment options

   Returns the number of sequences aligned
*/
sint palign2(ALNPTR mult_aln,double **tmat,MULT_OPT mult_opt,MOTIF *motifs)  
{
	sint 	i,j,sum,entries,status;
	lint 	score;
	sint 	*group;
	sint	*maxid,*p1_weight,*p2_weight;
	IN_TREEPTR  itree;

	info("Start of Multiple Alignment");

/* get the guide trees for each profile */

	if (mult_aln->prf1.nseqs >= 2) {
		itree=(IN_TREEPTR)ckalloc(sizeof(IN_TREE));
        	status = read_tree(mult_aln->prf1.treename, mult_aln->seqs, (sint)0, mult_aln->prf1.nseqs,itree);
        	if (status < 0) return(0);
     	}

/* calculate sequence weights according to branch lengths of the tree -
   weights are normalised to sum to 100 */

	p1_weight = calc_seq_weights((sint)0, mult_aln->prf1.nseqs, itree,mult_opt.no_weights);

/* clear the memory for the phylogenetic tree */

   	if (mult_aln->prf1.nseqs >= 2)
        	free_tree(itree);

	if (mult_aln->prf2.nseqs >= 2) {
		itree=(IN_TREEPTR)ckalloc(sizeof(IN_TREE));
        	status = read_tree(mult_aln->prf2.treename, mult_aln->seqs, mult_aln->prf1.nseqs, mult_aln->nseqs,itree);
        	if (status < 0) return(0);
	}

	p2_weight = calc_seq_weights(mult_aln->prf1.nseqs,mult_aln->nseqs, itree,mult_opt.no_weights);


/* clear the memory for the phylogenetic tree */

	if (mult_aln->prf2.nseqs >= 2)
        	free_tree(itree);

/* convert tmat distances to similarities */

	for (i=0;i<mult_aln->nseqs;i++)
       		for (j=i+1;j<mult_aln->nseqs;j++) {
			tmat[i][j]=100.0-tmat[i][j]*100.0;
			tmat[j][i]=tmat[i][j];
        	}

	for (i=0;i<mult_aln->prf1.nseqs;i++) {
		mult_aln->seqs[i].weight = p1_weight[i];
	}

	for (i=mult_aln->prf1.nseqs;i<mult_aln->nseqs;i++) {
		mult_aln->seqs[i].weight = p2_weight[i];
	}

/* do the alignment.........  */

	info("Aligning...");

	group = (sint *)ckalloc( (mult_aln->nseqs+1) * sizeof (sint));

	for(i=0; i<mult_aln->prf1.nseqs; i++)
		group[i] = 1;
	for(i=mult_aln->prf1.nseqs; i<mult_aln->prf1.nseqs+mult_aln->prf2.nseqs; i++)
		group[i] = 2;
	entries = mult_aln->nseqs;

	score = prfalign(1,mult_aln,group,tmat,TRUE,mult_opt,motifs);
	info("Sequences:%d      Score:%d",(pint)entries,(pint)score);
	group=ckfree((void *)group);
	p1_weight=ckfree((void *)p1_weight);
	p2_weight=ckfree((void *)p2_weight);

	for (i=0;i<mult_aln->nseqs;i++) mult_aln->seqs[i].output_index = i;

	return(mult_aln->nseqs);
}

/*
static sint create_upgma_sets(sint nseqs,FILE *tree,sint **sets)
{
	double s;
	sint i,a,b,n;
	sint *ptr;
	sint nsets;

	ptr=(sint *)ckalloc((nseqs+1)*sizeof(ptr));

	nsets = 0;
	while(TRUE) {
		if(!read_upgma(tree,nseqs,&n,&s,&a,&b,ptr)) break;
		for(i=0;i<nseqs;i++) sets[nsets][i]=ptr[i];
		nsets++;
	}
	ckfree(ptr);
	return nsets;
}
*/

sint create_sets(sint nseq,sint **sets,IN_TREEPTR itree)
{
  sint *groups;
  sint nsets;

  nsets = 0;
  if (nseq >= 2)
     {
/*
  If there are more than three sequences....
*/
       groups = (sint *)ckalloc((nseq+1) * sizeof(sint));
       group_seqs(itree->root, groups, &nsets,sets,itree);
       groups=ckfree((void *)groups);

     }
  return nsets;

}


static void group_seqs(INODEPTR p, sint *next_groups, sint *nsets, sint **sets,IN_TREEPTR itree)
{
    sint i;
    sint *tmp_groups;

    tmp_groups = (sint *)ckalloc((itree->nleaves+1) * sizeof(sint));
    for (i=0;i<itree->nleaves;i++)
         tmp_groups[i] = 0;

    if (p->left != NULL)
      {
         if (p->left->leaf == NODE)
            {
               group_seqs(p->left, next_groups, nsets,sets,itree);
               for (i=0;i<itree->nleaves;i++)
                 if (next_groups[i] != 0) tmp_groups[i] = 1;
            }
         else
            {
               mark_group1(p->left, tmp_groups, itree);
            }
               
      }

    if (p->right != NULL)
      {
         if (p->right->leaf == NODE)
            {
               group_seqs(p->right, next_groups, nsets,sets,itree);
               for (i=0;i<itree->nleaves;i++)
                    if (next_groups[i] != 0) tmp_groups[i] = 2;
            }
         else 
            {
               mark_group2(p->right, tmp_groups, itree);
            }
    	for (i=0;i<itree->nleaves;i++)
      		sets[*nsets][i] = tmp_groups[i];
    	(*nsets)++;
      }
    for (i=0;i<itree->nleaves;i++)
      next_groups[i] = tmp_groups[i];

    tmp_groups=ckfree((void *)tmp_groups);

}

static void mark_group1(INODEPTR p, sint *groups, IN_TREEPTR itree)
{
    sint i;

    for (i=0;i<itree->nleaves;i++)
       {
         if (itree->leafptr[i] == p)
              groups[i] = 1;
         else
              groups[i] = 0;
       }
}

static void mark_group2(INODEPTR p, sint *groups, IN_TREEPTR itree)
{
    sint i;

    for (i=0;i<itree->nleaves;i++)
       {
         if (itree->leafptr[i] == p)
              groups[i] = 2;
         else if (groups[i] != 0)
              groups[i] = 1;
       }
}

/*
    Align 2 profiles. First either reads or calculates a tree for each
    profile (the user can decide to input trees for each profile if they
    exist already). A rough alignment between the 2 profiles gives
    pairwise sequence distances between the 2 profiles. This info is
    then used to calculate sequence weights, and to select an appropriate
    comparison matrix for the final profile alignment.
	mult_aln	contains the 2 profiles, one after the other
	opt		alignment options
*/

void profile_align(ALNPTR mult_aln,OPT opt,Boolean use_tree1_file,Boolean use_tree2_file,Boolean usemenu)
{
	char path[FILENAMELEN+1];
	char tree_name[FILENAMELEN+1];
	char temp[MAXLINE+1];
	Boolean use_tree1,use_tree2;
	FILE *tree;
	sint count,i,j,dscore,nanchors;
	double **tmat;
	MOTIF *motifs=NULL;
	
	if(mult_aln->prf1.nseqs<=0 || mult_aln->prf2.nseqs<=0) {
		error("No sequences in memory. Load sequences first.");
		return;
	}

	
       	if(!open_alignment_output(mult_aln->prf1.filename,opt.alnout_opt)) return;

	if(opt.mult_opt->reset_alignments_new || opt.mult_opt->reset_alignments_all) {
		reset_prf(1,mult_aln,opt.mult_opt->reset_alignments_new,opt.mult_opt->reset_alignments_all);
		reset_prf(2,mult_aln,opt.mult_opt->reset_alignments_new,opt.mult_opt->reset_alignments_all);
	}
	else fix_gaps(mult_aln);

/* get the phylogenetic tree for profile 1 */
	get_path(mult_aln->prf1.filename,path);
	tree_name[0] = EOS;
	use_tree1 = FALSE;
	if (mult_aln->prf1.nseqs >= 2) {
		if (usemenu) {
			strcpy(tree_name,path);
			strcat(tree_name,"dnd");
#ifdef VMS
        		if((tree=fopen(tree_name,"r","rat=cr","rfm=var"))!=NULL) {
#else
        		if((tree=fopen(tree_name,"r"))!=NULL) {
#endif
            			fprintf(stdout,"\nUse the existing GUIDE TREE file for Profile 1,  %s  (y/n) ? [y]: ",
                                           tree_name);
                		gets(temp);
                		if(*temp != 'n' && *temp != 'N') {
                    			strcpy(mult_aln->prf1.treename,tree_name);
                    			use_tree1 = TRUE;
                		}
                		fclose(tree);
        		}
		}
		else if (use_tree1_file && mult_aln->prf1.treename[0]!='\0') {
			use_tree1 = TRUE;
		}
	}
/* get the phylogenetic tree for profile 2 */
	tree_name[0] = EOS;
	use_tree2 = FALSE;
	get_path(mult_aln->prf2.filename,path);
	if (mult_aln->prf2.nseqs >= 2) {
		if (usemenu) {
			strcpy(tree_name,path);
			strcat(tree_name,"dnd");
#ifdef VMS
        		if((tree=fopen(tree_name,"r","rat=cr","rfm=var"))!=NULL) {
#else
        		if((tree=fopen(tree_name,"r"))!=NULL) {
#endif
            			fprintf(stdout,"\nUse the existing GUIDE TREE file for Profile 2,  %s  (y/n) ? [y]: ",
                                           tree_name);
                		gets(temp);
                		if(*temp != 'n' && *temp != 'N') {
                    			strcpy(mult_aln->prf2.treename,tree_name);
                    			use_tree2 = TRUE;
                		}
                		fclose(tree);
        		}
		}
		else if (use_tree2_file && mult_aln->prf2.treename[0]!='\0') {
			use_tree2 = TRUE;
		}
	}
				
	if (mult_aln->prf1.ss.struct_penalties == SECST)
		calc_gap_penalty_mask(mult_aln->seqs[0].len,mult_aln->prf1.ss.sec_struct_mask,mult_aln->prf1.ss.gap_penalty_mask,*opt.ss_opt);

	if (mult_aln->prf2.ss.struct_penalties == SECST)
		calc_gap_penalty_mask(mult_aln->seqs[mult_aln->prf1.nseqs].len,mult_aln->prf2.ss.sec_struct_mask,mult_aln->prf2.ss.gap_penalty_mask,*opt.ss_opt);


/* if we didn't get a phylogenetic tree for profile 1, calculate one now */
	if (use_tree1 == FALSE)
		if (mult_aln->prf1.nseqs >= 2) {
        		tmat = (double **) ckalloc( (mult_aln->prf1.nseqs+1) * sizeof (double *) );
        		for(i=0;i<mult_aln->prf1.nseqs;i++)
                		tmat[i] = (double *)ckalloc( (mult_aln->prf1.nseqs+1) * sizeof (double) );
        		for(i=0;i<mult_aln->prf1.nseqs;i++)
                		for(j=0;j<mult_aln->prf1.nseqs;j++)
                        		tmat[i][j]=0.0;

                	for (i=0;i<mult_aln->prf1.nseqs;i++) {
                        	for (j=i+1;j<mult_aln->prf1.nseqs;j++) {
       					dscore = countid(mult_aln->seqs[i],mult_aln->seqs[j]);
                                	tmat[i][j] = (100.0 - dscore)/100.0;
                                	tmat[j][i] = tmat[i][j];
                        	}
                	}
        		if (mult_aln->prf1.treename[0]!=EOS) {
                		if((tree = open_explicit_file(mult_aln->prf1.treename))==NULL) return;
        		}
        		else {
               			if((tree = open_output_file(mult_aln->prf1.filename,
                		"new GUIDE TREE","dnd", mult_aln->prf1.treename)) == NULL) return;
        		}

			guide_tree(tree,mult_aln->seqs,mult_aln->prf1.nseqs,tmat,opt.pw_opt->treealgo);
			info("Guide tree        file created:   [%s]",
               		mult_aln->prf1.treename);

        		for(i=0;i<mult_aln->prf1.nseqs;i++)
                		tmat[i] = ckfree(tmat[i]);
        		tmat = ckfree(tmat);
		}

/* if we didn't get a phylogenetic tree for profile 2, calculate one now */
	if (use_tree2 == FALSE)
		if(mult_aln->prf2.nseqs >= 2) {
        		tmat = (double **) ckalloc( (mult_aln->prf2.nseqs+1) * sizeof (double *) );
        		for(i=0;i<mult_aln->prf2.nseqs;i++)
                		tmat[i] = (double *)ckalloc( (mult_aln->prf2.nseqs+1) * sizeof (double) );
        		for(i=0;i<mult_aln->prf2.nseqs;i++)
                		for(j=0;j<mult_aln->prf2.nseqs;j++)
                        		tmat[i][j]=0.0;
                	for (i=0;i<mult_aln->prf2.nseqs;i++) {
                        	for (j=i+1;j<mult_aln->prf2.nseqs;j++) {
       					dscore = countid(mult_aln->seqs[i+mult_aln->prf1.nseqs],mult_aln->seqs[j+mult_aln->prf1.nseqs]);
                                	tmat[i][j] = (100.0 - dscore)/100.0;
                                	tmat[j][i] = tmat[i][j];
                        	}
                	}
        		if (mult_aln->prf2.treename[0]!=EOS) {
                		if((tree = open_explicit_file(mult_aln->prf2.treename))==NULL) return;
        		}
        		else {
               			if((tree = open_output_file(mult_aln->prf2.filename,
                		"new GUIDE TREE","dnd", mult_aln->prf2.treename)) == NULL) return;
        		}
			guide_tree(tree,&mult_aln->seqs[mult_aln->prf1.nseqs],mult_aln->prf2.nseqs,tmat,opt.pw_opt->treealgo);
			info("Guide tree        file created:   [%s]",
               		mult_aln->prf2.treename);
        		for(i=0;i<mult_aln->prf2.nseqs;i++)
                		tmat[i] = ckfree(tmat[i]);
        		tmat = ckfree(tmat);
		}

/* get the motif list if there is one */
	motifs=(MOTIF *)ckalloc(MOTIFBLK*sizeof(MOTIF));
	if(opt.mult_opt->use_motifs && opt.mult_opt->motif_filename[0]!='\0') {
		motifs=read_motif_list(opt.mult_opt->motif_filename,mult_aln->nseqs,mult_aln->seqs,
				opt.mult_opt->propogate_motifs,&nanchors);
		mult_aln->nanchors=nanchors;
	}

/* do an initial alignment to get the pairwise identities between the two
profiles - used to set parameters for the final alignment */

	tmat = palign1(mult_aln,*opt.mult_opt,motifs);
	if (tmat==NULL) return;

	reset_prf(1,mult_aln,TRUE,opt.mult_opt->reset_alignments_all);
	reset_prf(2,mult_aln,TRUE,opt.mult_opt->reset_alignments_all);

/* now use all the info from the trees and the distances between profiles calculated
   in palign1 to do a more accurate alignment */

	count = palign2(mult_aln,tmat,*opt.mult_opt,motifs);

        for(i=0;i<mult_aln->prf1.nseqs+mult_aln->prf2.nseqs;i++)
                tmat[i] = ckfree(tmat[i]);
        tmat = ckfree(tmat);

	if (count == 0) return;

	if(usemenu) fprintf(stdout,"\n\n\n");

	create_alignment_output(*mult_aln,0,mult_aln->prf1.nseqs+mult_aln->prf2.nseqs,*opt.alnout_opt,*opt.ss_opt);
        if (opt.alnout_opt->showaln && usemenu) show_aln(*opt.alnout_opt);

	mult_aln->prf1.treename[0]=EOS;
	mult_aln->prf2.treename[0]=EOS;

	if(motifs!=NULL) ckfree(motifs);
}

/* remove gaps from older alignments (code = GAP1) */
/* EXCEPT for gaps that were INPUT with the seqs which have  code = GAP2  */
static void reset_align(sint nseqs, SEQ *seqs,Boolean reset_new, Boolean reset_all)
{
        register sint sl;
        sint i,j;

	if(nseqs==0) return;

	if(!reset_new && !reset_all) return;

        for(i=0;i<nseqs;i++) {
                sl=0;
                for(j=0;j<seqs[i].len;j++) {
                        if(seqs[i].data[j] == GAP1 && (reset_new || reset_all)) continue;
                        if(seqs[i].data[j] == GAP2 && (reset_all)) continue;
                        seqs[i].data[sl++]=seqs[i].data[j];
                }
                seqs[i].len=sl;
        }


}

/* remove gaps from older alignments (code = GAP1) */
/* EXCEPT for gaps that were INPUT with the seqs which have  code = GAP2  */
static void reset_prf(sint prf_no,ALNPTR mult_aln,Boolean reset_new, Boolean reset_all)
{
        register sint sl;                    
        sint i,j,fseq,nseq,len;
	SEC_STRUCT_DATAPTR ss;


	if(prf_no==1) {
		fseq=0;
		nseq=mult_aln->prf1.nseqs;
		ss=(&mult_aln->prf1.ss);
	}
	else {
		fseq=mult_aln->prf1.nseqs;
		nseq=mult_aln->prf1.nseqs+mult_aln->prf2.nseqs;
		ss=(&mult_aln->prf2.ss);

	}
	if(nseq==0) return;

        len=mult_aln->seqs[fseq].len;
        if (ss->struct_penalties != NONE) {
                sl=0;
                for (j=0;j<len;j++) {
                        if (ss->gap_penalty_mask[j] == GAP1 && (reset_new || reset_all)) continue;
                        if (ss->gap_penalty_mask[j] == GAP2 && (reset_all)) continue;
                        ss->gap_penalty_mask[sl++]=ss->gap_penalty_mask[j];
                }
        }

        if (ss->struct_penalties == SECST) {
                sl=0;
                for (j=0;j<len;j++) {
                        if (ss->sec_struct_mask[j] == GAP1 && (reset_new || reset_all)) continue;
                        if (ss->sec_struct_mask[j] == GAP2 && (reset_all)) continue;
                        ss->sec_struct_mask[sl++]=ss->sec_struct_mask[j];
                }
        }

        for(i=fseq;i<nseq;i++) {
                sl=0;
                for(j=0;j<len;j++) {
                        if(mult_aln->seqs[i].data[j] == GAP1 && (reset_new || reset_all)) continue;
                        if(mult_aln->seqs[i].data[j] == GAP2 && (reset_all)) continue;
                        mult_aln->seqs[i].data[sl++]=mult_aln->seqs[i].data[j];
                }
                mult_aln->seqs[i].len=sl;
        }


}

void fix_gaps(ALNPTR mult_aln)   /* fix gaps introduced in older alignments (code = GAP1) */
{
        sint i,j;

	if(mult_aln->nseqs==0) return;

        if (mult_aln->prf1.ss.struct_penalties != NONE) {
                for (j=0;j<mult_aln->seqs[0].len;j++) {
                        if (mult_aln->prf1.ss.gap_penalty_mask[j] == GAP1)
                                mult_aln->prf1.ss.gap_penalty_mask[j]=GAP2;
                }
        }

        if (mult_aln->prf1.ss.struct_penalties == SECST) {
                for (j=0;j<mult_aln->seqs[0].len;j++) {
                        if (mult_aln->prf1.ss.sec_struct_mask[j] == GAP1)
                                mult_aln->prf1.ss.sec_struct_mask[j]=GAP2;
                }
        }

        for(i=0;i<mult_aln->nseqs;i++) {
                for(j=0;j<mult_aln->seqs[i].len;j++) {
                        if(mult_aln->seqs[i].data[j] == GAP1)
                                mult_aln->seqs[i].data[j]=GAP2;
                }
        }
}





void align(ALNPTR mult_aln,OPT opt,Boolean usemenu)
{ 
	char path[FILENAMELEN+1];
	FILE *nj_tree,*up_tree;
	sint i,j,count,nanchors;
	MOTIF *motifs=NULL;
	double **tmat;

	nanchors=0;
	
	if(mult_aln->nseqs<=0) {
		error("No sequences in memory. Load sequences first.");
		return;
	}

	if(mult_aln->nseqs==1) {
		error("Less than 2 sequences in memory. Cannot align sequences.");
		return;
	}

       	if(!open_alignment_output(mult_aln->filename,opt.alnout_opt)) return;

       	if (mult_aln->treename[0]!=EOS) {
               	if((nj_tree = open_explicit_file(
               	mult_aln->treename))==NULL) return;
       	}
       	else {
               	if((nj_tree = open_output_file(mult_aln->filename,
               	"new GUIDE TREE","dnd",mult_aln->treename)) == NULL) return;
       	}

/* get the motif list if there is one */
	if(opt.mult_opt->use_motifs && opt.mult_opt->motif_filename[0]!='\0') {
		motifs=read_motif_list(opt.mult_opt->motif_filename,mult_aln->nseqs,mult_aln->seqs,
				opt.mult_opt->propogate_motifs,&nanchors);
		mult_aln->nanchors=nanchors;
	}

	reset_align(mult_aln->nseqs,mult_aln->seqs,opt.mult_opt->reset_alignments_new,opt.mult_opt->reset_alignments_all);

        info("Start of Pairwise alignments");
        info("Aligning...");

        tmat = (double **) ckalloc( (mult_aln->nseqs+1) * sizeof (double *) );
        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = (double *)ckalloc( (mult_aln->nseqs+1) * sizeof (double) );
        for(i=0;i<mult_aln->nseqs;i++)
                for(j=0;j<mult_aln->nseqs;j++)
                        tmat[i][j]=0.0;

        if (opt.pw_opt->quick_pairalign)
           show_pair(*mult_aln,(sint)0,mult_aln->nseqs,(sint)0,mult_aln->nseqs,tmat,*opt.quickpw_opt);
        else
           pairalign(*mult_aln,(sint)0,mult_aln->nseqs,(sint)0,mult_aln->nseqs,tmat,*opt.pw_opt,motifs,nanchors);

	guide_tree(nj_tree,mult_aln->seqs,mult_aln->nseqs,tmat,opt.pw_opt->treealgo);
	info("Guide tree        file created:   [%s]", mult_aln->treename);
/*
	upgma_tree("upgma.tree",mult_aln->seqs,mult_aln->nseqs,tmat);
	info("Upgma tree        file created:   [%s]", "upgma.tree");
*/
        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = ckfree(tmat[i]);
        tmat = ckfree(tmat);
	
	count = malign(mult_aln,*opt.mult_opt,motifs,opt.alnout_opt->output_order);
	
	if (count <= 0) return;

	if (usemenu) fprintf(stdout,"\n\n\n");
/*	create_alignment_output(*mult_aln,0,mult_aln->nseqs,*opt.alnout_opt,*opt.ss_opt);*/
        if (opt.alnout_opt->showaln && usemenu) show_aln(*opt.alnout_opt);
	mult_aln->treename[0]=EOS;

	if(motifs!=NULL) ckfree(motifs);
}





void new_sequence_align(ALNPTR mult_aln,OPT opt,Boolean use_tree_file,Boolean usemenu)
{ 
	char path[FILENAMELEN+1];
	char tree_name[FILENAMELEN+1],temp[MAXLINE+1];
	Boolean use_tree;
	FILE *tree;
	sint i,j,count,new_seq,nanchors;
	float dscore;
	double **tmat;
	MOTIF *motifs=NULL;

	nanchors=0;
	
	if((mult_aln->prf1.nseqs<=0) && usemenu) {
		error("No profile in memory. Input 1st profile first.");
		return;
	}

	if((mult_aln->prf2.nseqs<=0) && usemenu) {
		error("No sequences in memory. Input sequences first.");
		return;
	}


       	if(!open_alignment_output(mult_aln->prf2.filename,opt.alnout_opt)) return;

	new_seq = mult_aln->prf1.nseqs;

/* check for secondary structure information for list of sequences */

        get_path(mult_aln->prf2.filename,path);
	if (mult_aln->prf2.ss.struct_penalties != NONE && opt.ss_opt->use_ss2 == TRUE && (mult_aln->prf2.nseqs > 1)) {
		if (mult_aln->prf2.ss.struct_penalties == SECST) 
			warning("Warning: ignoring secondary structure for a list of sequences");
		else if (mult_aln->prf2.ss.struct_penalties == GMASK)
			warning("Warning: ignoring gap penalty mask for a list of sequences");
		mult_aln->prf2.ss.struct_penalties = FALSE;
	}

        tmat = (double **) ckalloc( (mult_aln->nseqs+1) * sizeof (double *) );
        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = (double *)ckalloc( (mult_aln->nseqs+1) * sizeof (double) );
        for(i=0;i<mult_aln->nseqs;i++)
                for(j=0;j<mult_aln->nseqs;j++)
                        tmat[i][j]=0.0;

	for (i=0;i<new_seq;i++) {
     		for (j=i+1;j<new_seq;j++) {
       			dscore = countid(mult_aln->seqs[i],mult_aln->seqs[j]);
       			tmat[i][j] = ((double)100.0 - (double)dscore)/(double)100.0;
       			tmat[j][i] = tmat[i][j];
     		}
   	}

	tree_name[0] = EOS;
	use_tree = FALSE;
	if (usemenu) {
		strcpy(tree_name,path);
		strcat(tree_name,"dnd");
#ifdef VMS
        	if((tree=fopen(tree_name,"r","rat=cr","rfm=var"))!=NULL) {
#else
        	if((tree=fopen(tree_name,"r"))!=NULL) {
#endif
		if (usemenu)
            	fprintf(stdout,"\nUse the existing GUIDE TREE file,  %s  (y/n) ? [y]: ",
                                           tree_name);
                gets(temp);
                if(*temp != 'n' && *temp != 'N') {
                    strcpy(mult_aln->treename,tree_name);
                    use_tree = TRUE;
                }
                fclose(tree);
        	}
	}
	else if (!usemenu && use_tree_file && mult_aln->treename[0]!='\0') {
		use_tree = TRUE;
	}
	
	if(opt.mult_opt->reset_alignments_new || opt.mult_opt->reset_alignments_all) {
/*
		reset_prf(1,mult_aln);
*/
		reset_prf(2,mult_aln,opt.mult_opt->reset_alignments_new,opt.mult_opt->reset_alignments_all);
	}
	else fix_gaps(mult_aln);

	if (mult_aln->prf1.ss.struct_penalties == SECST)
		calc_gap_penalty_mask(mult_aln->seqs[0].len,mult_aln->prf1.ss.sec_struct_mask,mult_aln->prf1.ss.gap_penalty_mask,*opt.ss_opt);

	if (mult_aln->prf2.ss.struct_penalties == SECST)
		calc_gap_penalty_mask(mult_aln->seqs[mult_aln->prf1.nseqs].len,mult_aln->prf2.ss.sec_struct_mask,mult_aln->prf2.ss.gap_penalty_mask,*opt.ss_opt);

/* get the motif list if there is one */
	if(opt.mult_opt->use_motifs && opt.mult_opt->motif_filename[0]!='\0') {
		motifs=read_motif_list(opt.mult_opt->motif_filename,mult_aln->nseqs,mult_aln->seqs,
				opt.mult_opt->propogate_motifs,&nanchors);
		mult_aln->nanchors=nanchors;
	}


/* create the new tree file, if necessary */

	if (use_tree == FALSE) {

        	get_path(mult_aln->prf2.filename,path);
        	if (mult_aln->prf2.treename[0]!=EOS) {
                	if((tree = open_explicit_file(
                	mult_aln->prf2.treename))==NULL) return;
        	}
        	else {
               		if((tree = open_output_file(mult_aln->prf2.filename,
                	"new GUIDE TREE","dnd", mult_aln->treename)) == NULL) return;
        	}
        	info("Start of Pairwise alignments");
        	info("Aligning...");

        	if (opt.pw_opt->quick_pairalign)
           		show_pair(*mult_aln,(sint)0,mult_aln->nseqs,new_seq,mult_aln->nseqs,tmat,*opt.quickpw_opt);
        	else
           		pairalign(*mult_aln,(sint)0,mult_aln->nseqs,new_seq,mult_aln->nseqs,tmat,*opt.pw_opt,motifs,nanchors);

		guide_tree(tree,mult_aln->seqs,mult_aln->nseqs,tmat,opt.pw_opt->treealgo);
		info("Guide tree        file created:   [%s]", mult_aln->treename);
	}
        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = ckfree(tmat[i]);
        tmat = ckfree(tmat);
	
	count = seqalign(mult_aln,*opt.mult_opt,motifs,opt.alnout_opt->output_order,new_seq);
	
	if (count <= 0) return;

	if (usemenu) fprintf(stdout,"\n\n\n");
	
	create_alignment_output(*mult_aln,0,mult_aln->nseqs,*opt.alnout_opt,*opt.ss_opt);
        if (opt.alnout_opt->showaln && usemenu) show_aln(*opt.alnout_opt);

	mult_aln->treename[0]=EOS;

	if(motifs!=NULL) ckfree(motifs);

}





void make_tree(ALNPTR mult_aln,OPT opt,Boolean usemenu)
{
	char path[FILENAMELEN+1];
	FILE *tree;
	sint i,j,nanchors;
	MOTIF *motifs=NULL;
	double **tmat;

	nanchors=0;
	
	if(mult_aln->nseqs<=0) {
		error("No sequences in memory. Load sequences first.");
		return;
	}


	if (mult_aln->nseqs == 1) {
		error("Less than 2 sequences in memory. Phylogenetic tree cannot be built.");
		return;
	}

/* get the motif list if there is one */
	if(opt.mult_opt->use_motifs && opt.mult_opt->motif_filename[0]!='\0') {
		motifs=read_motif_list(opt.mult_opt->motif_filename,mult_aln->nseqs,mult_aln->seqs,
				opt.mult_opt->propogate_motifs,&nanchors);
		mult_aln->nanchors=nanchors;
	}

	reset_align(mult_aln->nseqs,mult_aln->seqs,opt.mult_opt->reset_alignments_new,opt.mult_opt->reset_alignments_all);

	info("Start of Pairwise alignments");
	info("Aligning...");
   
        tmat = (double **) ckalloc( (mult_aln->nseqs+1) * sizeof (double *) );
        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = (double *)ckalloc( (mult_aln->nseqs+1) * sizeof (double) );
        for(i=0;i<mult_aln->nseqs;i++)
                for(j=0;j<mult_aln->nseqs;j++)
                        tmat[i][j]=0.0;

        if (opt.pw_opt->quick_pairalign)
          show_pair(*mult_aln,(sint)0,mult_aln->nseqs,(sint)0,mult_aln->nseqs,tmat,*opt.quickpw_opt);
        else
          pairalign(*mult_aln,(sint)0,mult_aln->nseqs,(sint)0,mult_aln->nseqs,tmat,*opt.pw_opt,motifs,nanchors);

       	if (mult_aln->treename[0]!=EOS) {
               	if((tree = open_explicit_file(
               	mult_aln->treename))==NULL) return;
       	}
       	else {
               	if((tree = open_output_file(mult_aln->filename,
                	"new GUIDE TREE","dnd", mult_aln->treename)) == NULL) return;
       	}

	guide_tree(tree,mult_aln->seqs,mult_aln->nseqs,tmat,opt.pw_opt->treealgo);
	info("Guide tree        file created:   [%s]", mult_aln->treename);
	
	mult_aln->treename[0]=EOS;

        for(i=0;i<mult_aln->nseqs;i++)
                tmat[i] = ckfree(tmat[i]);
        tmat = ckfree(tmat);

	if(motifs!=NULL) ckfree(motifs);
}









void align_from_tree(ALNPTR mult_aln,OPT opt,Boolean usemenu)
{
	char path[FILENAMELEN+1],temp[MAXLINE+1];
	sint i,j,count,nanchors;
	MOTIF *motifs=NULL;
	
	if(mult_aln->nseqs<=0) {
		error("No sequences in memory. Load sequences first.");
		return;
	}

        if (mult_aln->nseqs == 1) {
		error("Less than 2 sequences in memory. Load sequences first.");
		return;
	}

       	if(!open_alignment_output(mult_aln->filename,opt.alnout_opt)) return;

	reset_align(mult_aln->nseqs,mult_aln->seqs,opt.mult_opt->reset_alignments_new,opt.mult_opt->reset_alignments_all);

        get_path(mult_aln->filename,path);
       	if(usemenu) {
		strcpy(mult_aln->treename,path);
       		strcat(mult_aln->treename,"dnd");

            	fprintf(stdout,"\nEnter a name for the guide tree file [%s]: ",
                                           mult_aln->treename);
               	gets(temp);
               	if(*temp != EOS) strcpy(mult_aln->treename,temp);
       	}

/* get the motif list if there is one */
	motifs=NULL;
	if(opt.mult_opt->use_motifs && opt.mult_opt->motif_filename[0]!='\0') {
		motifs=read_motif_list(opt.mult_opt->motif_filename,mult_aln->nseqs,mult_aln->seqs,
				opt.mult_opt->propogate_motifs,&nanchors);
		mult_aln->nanchors=nanchors;
	}

	count = malign(mult_aln,*opt.mult_opt,motifs,opt.alnout_opt->output_order);
	if (count <= 0) return;

	if (usemenu) fprintf(stdout,"\n\n\n");

	create_alignment_output(*mult_aln,0,mult_aln->nseqs,*opt.alnout_opt,*opt.ss_opt);
        if (opt.alnout_opt->showaln && usemenu) show_aln(*opt.alnout_opt);

	mult_aln->treename[0]=EOS;

	if(motifs!=NULL) ckfree(motifs);
}

