#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "clustalw.h"

#define MIN(a,b) ((a)<(b)?(a):(b))
#define MAX(a,b) ((a)>(b)?(a):(b))

/*
 *   Prototypes
 */
static MOTIF *read_dialign_list(FILE *fd,sint *nanchors);
static MOTIF *read_ballast_list(FILE *fd,sint nseqs,SEQ *seqs,float max,Boolean propogate,sint *nanchors);
static sint start_pos(char *seq,sint p);
static sint next_pos(char *seq,sint len,sint p);
static Boolean compare_motifs(MOTIF motif1,MOTIF motif2, sint *offset);
static Boolean motif_exists(MOTIF *motifs,sint nmotifs,MOTIF motif1,MOTIF motif2);

static sint debug=0;

MOTIF *read_motif_list(char *filename,sint nseqs,SEQ *seqs,Boolean propogate,sint *nanchors)
{
	MOTIF *motifs=NULL;
	sint n,n1;
	float max;
	FILE *fd;
	char inline1[1024];

	motifs=NULL;
	(*nanchors)=n1=0;
	if ((fd=fopen(filename,"r"))==NULL) {
        	error("cannot open motif file %s", filename);
        	return(NULL);
	}

	if (fgets(inline1,1024,fd) != NULL) {
		if(keyword(inline1,"Ballast")) {
			if((n=sscanf(&inline1[7],"%f",&max))!=1) {
				error("wrong format in ballast file %s", filename);
        			return(NULL);
			}
			motifs=read_ballast_list(fd,nseqs,seqs,max,propogate,&n1);
		}
		else {
			rewind(fd);
			motifs=read_dialign_list(fd,&n1);
		}
	}

	fclose(fd);

	*nanchors=n1;

	return motifs;
}

static MOTIF *read_ballast_list(FILE *fd,sint nseqs,SEQ *seqs,float max,Boolean propogate,sint *nanchors)
{
	MOTIF *motifs;
	sint i,j,n,nmotifs=0,nblocks=1;
	sint seq1,seq2,pos1,pos2,len,offset,peak;
	char name1[MAXNAMES],name2[MAXNAMES];
	char sname[MAXNAMES];
	char s[1024];
	float weight;
	char inline1[1024];
	char *tmp;

	motifs=(MOTIF *)ckalloc(MOTIFBLK*sizeof(MOTIF));
	while (fgets(inline1,1024,fd) != NULL) {
		if(keyword(inline1,"Ballast")) {
			if((n=sscanf(&inline1[7],"%f",&max))!=1)
				warning("wrong format in ballast motif file");
		}
		else if((tmp=strstr(inline1,"seq:"))!=NULL) {
			tmp+=4;
			if((n=sscanf(tmp,"%s %s %s %d %s %d %d %s %d %s %f",
				name1,name2,s,&peak,s,&pos1,&pos2,s,&len,s,&weight))!=11) {
				continue;
			}
			/*if(max<10) continue;
			if(nseqs<=5 && max<25 && weight<50) continue;
			if(max<100 && weight<20) continue;
			if(nseqs>10 && max<100) continue;
			if(nseqs>10 && max<300 && weight<30) continue;
			if(nseqs>10 && max<500 && weight<15) continue;
			if(nseqs>20 && max<200) continue;*/
			lowstr(name1);
			lowstr(name2);
			seq1=-1;
			for(i=0;i<nseqs;i++) {
				strcpy(sname,seqs[i].name);
				lowstr(sname);
				if(strcmp(name1,sname)==0) {
					seq1=i;
					break;
				}
			}
/*
			if (seq1==-1) {
				error("sequence %s not found in sequence file\n",name1);
			}
*/
			seq2=-1;
			for(i=0;i<nseqs;i++) {
				strcpy(sname,seqs[i].name);
				lowstr(sname);
				if(strcmp(name2,sname)==0) {
					seq2=i;
					break;
				}
			}
/*
			if (seq2==-1) {
				error("sequence %s not found in sequence file\n",name2);
			}
*/
			if(seq1 == -1 || seq2 == -1) continue;
			if(pos1+len>seqs[seq1].len) continue;
			if(pos2+len>seqs[seq2].len) continue;
			motifs[nmotifs].seq1=seq1+1;
			motifs[nmotifs].seq2=seq2+1;
			motifs[nmotifs].pos1=pos1;
			motifs[nmotifs].pos2=pos2;
			motifs[nmotifs].len=len;
			if(len>=3) {
				/*weight*=(float)max/3000.0;*/
				if(weight>5) { 
					/*if(weight>30) weight*=2.0;*/
if(debug>2) printf("%s %s %d %d %d %f\n",name1,name2,pos1,pos2,len,weight);
					motifs[nmotifs].weight=(weight)*INT_SCALE_FACTOR;
					if(nmotifs>=nblocks*MOTIFBLK-2) {
						nblocks++;
						motifs=(MOTIF *)ckrealloc(motifs,nblocks*MOTIFBLK*sizeof(MOTIF));
					}
					nmotifs++;
				}
			}
		}
	} 

if(debug>2) fprintf(stdout,"nmotifs %d\n",nmotifs);
/* extrapolate the motifs through the other sequences */
	n=0;
	if(propogate) {
		for(i=0;i<nmotifs;i++) {
			for(j=i+1;j<nmotifs;j++) {
				if(compare_motifs(motifs[i],motifs[j],&offset)) {
					motifs[nmotifs+n].seq1=motifs[i].seq2;
					motifs[nmotifs+n].seq2=motifs[j].seq2;
					motifs[nmotifs+n].pos1=motifs[i].pos2+offset;
					motifs[nmotifs+n].pos2=motifs[j].pos2;
					motifs[nmotifs+n].len=MIN(motifs[i].len,motifs[j].len)-offset;
					if(motifs[nmotifs+n].pos1<=0 || motifs[nmotifs+n].pos2<=0) continue;
					if(motifs[nmotifs+n].len<=0) continue;
					if(motifs[nmotifs+n].pos1+motifs[nmotifs+n].len>seqs[motifs[nmotifs+n].seq1].len) continue;
					if(motifs[nmotifs+n].pos2+motifs[nmotifs+n].len>seqs[motifs[nmotifs+n].seq2].len) continue;
					motifs[nmotifs+n].weight=MAX(motifs[i].weight,motifs[j].weight);
if (debug>2) printf("%d %d %d %d %d %d\n",nmotifs+n,motifs[i].seq2,motifs[j].seq2,motifs[i].pos2+offset,motifs[j].pos2,motifs[nmotifs+n].weight);
					n++;
					if(nmotifs+n>=nblocks*MOTIFBLK-2) {
						nblocks++;
						motifs=(MOTIF *)ckrealloc(motifs,nblocks*MOTIFBLK*sizeof(MOTIF));
					}
				}
			}
		}
	}
	motifs[nmotifs+n].seq1=0;
	motifs[nmotifs+n].seq2=0;
	motifs[nmotifs+n].pos1=0;
	motifs[nmotifs+n].pos2=0;
	motifs[nmotifs+n].len=0;
	motifs[nmotifs+n].weight=0;

	*nanchors=nmotifs+n;

	return motifs;
}

static Boolean motif_exists(MOTIF *motifs,sint nmotifs,MOTIF motif1,MOTIF motif2) 
{
	sint i;

	for(i=0;i<nmotifs;i++) {
		if (motifs[i].seq1==motif1.seq2 && motifs[i].seq2==motif2.seq2
		  && motif1.pos2>=motifs[i].pos1 && motif1.pos2<motifs[i].pos1+motifs[i].len
		  && motif2.pos2>=motifs[i].pos2 && motif2.pos2<motifs[i].pos2+motifs[i].len)
		return TRUE;
		
	}
	return FALSE;
}

static Boolean compare_motifs(MOTIF motif1,MOTIF motif2,sint *offset)
{
	Boolean ret;
	

	if(motif1.pos1==motif2.pos1 && motif1.pos1<motif2.pos1+motif2.len) {
		(*offset)=0; 
		ret=TRUE;
	}
	else if(motif1.pos1>motif2.pos1 && motif1.pos1<motif2.pos1+motif2.len) {
		(*offset)=motif2.pos1-motif1.pos1; 
		ret=TRUE;
	}
	else if(motif2.pos1>motif1.pos1 && motif2.pos1<motif1.pos1+motif1.len) {
		(*offset)=motif2.pos1-motif1.pos1;
		ret=TRUE;
	}
	else ret=FALSE;
	return ret;

}


static MOTIF *read_dialign_list(FILE *fd,sint *nanchors)
{ 
	MOTIF *motifs;
	sint n,nmotifs=0,nblocks=1;
	sint seq1,seq2,pos1,pos2,len;
	char s1[1024],s2[1024],s3[1024];
	float weight;
	char inline1[1024];
	char *tmp;

	motifs=(MOTIF *)ckalloc(MOTIFBLK*sizeof(MOTIF));
	while (fgets(inline1,1024,fd) != NULL) {
		if((tmp=strstr(inline1,"seq:"))!=NULL) {
			tmp+=4;
			if((n=sscanf(tmp,"%d %d %s %d %d %s %d %s %f\n",
				&seq1,&seq2,s1,&pos1,&pos2,s2,&len,s3,&weight))!=9) {
				error("Wrong format in motif list file\n");
				fclose(fd);
				return(NULL);
			}
			motifs[nmotifs].seq1=seq1;
			motifs[nmotifs].seq2=seq2;
			motifs[nmotifs].pos1=pos1;
			motifs[nmotifs].pos2=pos2;
			motifs[nmotifs].len=len;
			if(weight>1 && len>5) {
			motifs[nmotifs].weight=weight*INT_SCALE_FACTOR;
			if(nmotifs>=nblocks*MOTIFBLK-2) {
				nblocks++;
				motifs=(MOTIF *)ckrealloc(motifs,nblocks*MOTIFBLK*sizeof(MOTIF));
			}
			nmotifs++;
			}
		}
	} 

	if(nmotifs<=0) {
		motifs=ckfree(motifs);
	}
	*nanchors=nmotifs;
	return motifs;
}

sint calc_motif_scores(SEQ *seqs, sint nseqs, sint *group, MOTIF *motifs, sint **motif_score, sint prf_length1, sint prf_length2)
{

	char c;
	sint i,j, m,n1,n2;
	sint seq1, seq2, pos1, pos2, len, weight;
	sint ix1,ix2;
   
	for (i=0; i<prf_length1; i++)
		for (j=0; j<prf_length2; j++)
			motif_score[i][j]=0;

/*
   count number of sequences in each profile 
*/
	n1=n2=0;
	for(i=0;i<nseqs;i++)
		if(group[i]==1) n1++;
		else if(group[i]==2) n2++;

/*
   Check for motifs
*/
	if(motifs==NULL) return;

/*     check the list of motifs */
	for(m=0;motifs[m].weight!=0;m++) {
		seq1=motifs[m].seq1-1;
		seq2=motifs[m].seq2-1;
		pos1=motifs[m].pos1;
		pos2=motifs[m].pos2;
		len=motifs[m].len;
		if(group[seq1]==1 && group[seq2]==2) {
			ix1=start_pos(seqs[seq1].data,pos1);
			ix2=start_pos(seqs[seq2].data,pos2);
			weight=motifs[m].weight;
if(debug>0) fprintf(stdout,"1. %d %d %d %d %d %d weight %d len %d\n",seq1,seq2,pos1,pos2,ix1,ix2,weight,len);
			for(i=0;i<len;i++) {
				if(ix1>=prf_length1) break;
				if(ix2>=prf_length2) break;
				if(weight>motif_score[ix1][ix2]) motif_score[ix1][ix2]=weight;
				ix1=next_pos(seqs[seq1].data,prf_length1,ix1);
				ix2=next_pos(seqs[seq2].data,prf_length2,ix2);
			}
		}
		else if(group[seq2]==1 && group[seq1]==2) {
			ix1=start_pos(seqs[seq1].data,pos1);
			ix2=start_pos(seqs[seq2].data,pos2);
			weight=motifs[m].weight;
if(debug>0) fprintf(stdout,"2. %d %d %d %d %d %d weight %d len %d\n",seq1,seq2,pos1,pos2,ix1,ix2,weight,len);
			for(i=0;i<len;i++) {
				if(ix1>=prf_length2) break;
				if(ix2>=prf_length1) break;
				if(weight>motif_score[ix2][ix1]) motif_score[ix2][ix1]=weight;
				ix1=next_pos(seqs[seq1].data,prf_length2,ix1);
				ix2=next_pos(seqs[seq2].data,prf_length1,ix2);
			}
		}
	}
if(debug>5) {
fprintf(stdout,"\n");
for(i=0;i<prf_length1;i++) {
for(j=0;j<prf_length2;j++)
fprintf(stdout,"%d ",motif_score[i][j]);
fprintf(stdout,"\n");
}
}
	return m;
}

/* index into the sequence including gaps of the pth residue */
static sint start_pos(char *seq,sint p)
{
	sint i,ix=0;

	for(i=0;seq[i]!='\0';i++) {
		if(isalpha(seq[i])) ix++;	
		if(ix>=p) break;
	}

	return i;
}

static sint next_pos(char *seq,sint len,sint p)
{
	p++;
	if(p>=len) return (len-1);
	while(!isalpha(seq[p]) && p<len) p++;
	return p;
}

void calc_pw_motif_scores(SEQ *seqs,sint s1, sint s2, sint len1, sint len2, MOTIF *motifs, sint **motif_score)
{
	char c;
	sint i,j, m;
	sint pos1, pos2, len;
	sint ix1,ix2;
   
	for (i=0; i<len1; i++)
		for (j=0; j<len2; j++)
			motif_score[i][j]=0;

/*
   Check for motifs
*/
	if(motifs==NULL) return;

/*     check the list of motifs */
	for(m=0;motifs[m].weight!=0;m++) {
		pos1=motifs[m].pos1;
		pos2=motifs[m].pos2;
		len=motifs[m].len;
		if(motifs[m].seq1-1 == s1 && motifs[m].seq2-1 == s2) {
			ix1=start_pos(seqs[s1].data,pos1);
			ix2=start_pos(seqs[s2].data,pos2);
if(debug>1) fprintf(stdout,"1. %d %d %d %d %d %d weight %d\n",s1,s2,pos1,pos2,ix1,ix2,motifs[m].weight);
			for(i=0;i<len;i++) {
				if(ix1>=len1) break;
				if(ix2>=len2) break;
				if (motifs[m].weight>motif_score[ix1][ix2]) 
					motif_score[ix1][ix2]=motifs[m].weight;
				ix1=next_pos(seqs[s1].data,len1,ix1);
				ix2=next_pos(seqs[s2].data,len2,ix2);
			}
		}
		else if(motifs[m].seq1-1 == s2 && motifs[m].seq2-1 == s1) {
			ix1=start_pos(seqs[s1].data,pos1);
			ix2=start_pos(seqs[s2].data,pos2);
if(debug>1) fprintf(stdout,"2. %d %d %d %d %d %d weight %d\n",s1,s2,pos1,pos2,ix1,ix2,motifs[m].weight);
			for(i=0;i<len;i++) {
				if(ix1>=len2) break;
				if(ix2>=len1) break;
				if (motifs[m].weight>motif_score[ix2][ix1]) 
					motif_score[ix2][ix1]=motifs[m].weight;
				ix1=next_pos(seqs[s1].data,len2,ix1);
				ix2=next_pos(seqs[s2].data,len1,ix2);
			}
		}
	}
if(debug>=5) {
fprintf(stdout,"\n");
for(i=0;i<len1;i++) {
for(j=0;j<len2;j++)
fprintf(stdout,"%d ",motif_score[i][j]);
fprintf(stdout,"\n");
}
}
}

