#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "clustalw.h"
#ifdef MPI
#include <mpi.h>
#endif
#include "pairalign.h"

#define MIN(a,b) ((a)<(b)?(a):(b))
#define MAX(a,b) ((a)>(b)?(a):(b))

/*
*       Prototypes
*/
static float forward_pass(char *seq1, char *seq2, sint n, sint m,sint gop, sint gep,  COMP_MATRIX matrix);
static float forward_passmotif(char *seq1, char *seq2, sint n, sint m,sint gop, sint gep,  COMP_MATRIX matrix, sint **motif_score,Boolean add_motif);

/*
 *   Global variables
 */
#ifdef MAC
#define pwint   sint
#else
#define pwint   int
#endif
static sint 	debug=0;

/* hard-coded comparison matrices, in matrices.h */
extern sint 	blosum30mt[],pam350mt[],idmat[],pw_usermat[],pw_userdnamat[];
extern sint    clustalvdnamt[],swgapdnamt[];
extern sint    gon250mt[];
extern sint    bin30mt[],bin30gap[];

/*
   Do pairwise alignments of all the sequences selected. The aim is only to calculate
   percent identity scores, so the alignments are not actually produced.

	mult_aln	the sequences
	istart		first sequence in first list
	inseq		no. of sequences in first list
	jstart		first sequence in second list
	jnseq		no. of sequences in second list
	tmat		the distance matrix that will contain the pairwise identity scores
	pw_opt		the pairwise alignment options

   Returns 1 on successful completion.
*/

#ifdef MPI
sint pairalign(ALN mult_aln,sint istart, sint inseq, sint jstart, sint jnseq, double **tmat,
	       PW_OPT pw_opt,MOTIF *motifs,sint nanchors)
{
	static sint    	si, sj, i, j;
	static sint    	n,m,len1,len2;
	static sint    	nseqs,minlen,maxres;
	static char    	c;
	static float   	gop_scale,gep_scale;
	static sint   	*matptr,*gapptr;
	static float 	pcidcount;
	static sint 	gop, gep;
	static COMP_MATRIX matrix;
	static sint	int_scale;
	static sint	**motif_score;

	FILE *fichier_debug;

	/***************************************/
	/*                                     */
	/*variable declarations for parallelism*/
	/*                                     */
	/***************************************/

	int npes,nb_motifs,nb_alignments,length_gather;
	int *nb_alignments_for_processes,*alnstart_for_processes,*alnstop_for_processes;
	int factor;
	TMAT_SI_SJ *tmat_si_sj,*tmat_si_sj_total;
	MPI_Status status;
	time_t temps_debut,temps_fin;

	/*pw_opt structure*/
	int pw_opt_array_of_blocklengths[14]={1,1,1,1,1,1,1,FILENAMELEN+1,1,FILENAMELEN+1,
					      FILENAMELEN+1,FILENAMELEN+1,1,1};
	MPI_Aint pw_opt_array_of_displacements[14];
	MPI_Datatype pw_opt_array_of_types[14]={MPI_FLOAT,MPI_FLOAT,MPI_FLOAT,MPI_FLOAT,MPI_FLOAT,
						MPI_INT,MPI_INT,MPI_CHAR,MPI_INT,MPI_CHAR,MPI_CHAR,
						MPI_CHAR,MPI_CHAR,MPI_CHAR};
	MPI_Datatype pw_opt_packettype;
	/*end of pw_opt structure*/

	/*comp_matrix structure*/
	MPI_Datatype score_type;
	int comp_matrix_array_of_blocklengths[4]={1,1,27,1};
	MPI_Aint comp_matrix_array_of_displacements[4];
	MPI_Datatype comp_matrix_array_of_types[4];
	MPI_Datatype comp_matrix_packettype;	
	/*end of comp_matrix structure*/

	/*motif structure*/
	MPI_Datatype motif_type;
	int motif_array_of_blocklengths[6]={1,1,1,1,1,1};
	MPI_Aint motif_array_of_displacements[6];
	MPI_Datatype motif_array_of_types[6]={MPI_INT,MPI_INT,MPI_INT,MPI_INT,MPI_INT,MPI_INT};
	MPI_Datatype motif_packettype;
	/*end of motif structure*/
	
	/*tmat_si_sj structure*/
	int tmat_si_sj_array_of_blocklengths[3]={1,1,1};
	MPI_Aint tmat_si_sj_array_of_displacements[3];
	MPI_Datatype tmat_si_sj_array_of_types[3]={MPI_INT,MPI_INT,MPI_DOUBLE};
	MPI_Datatype tmat_si_sj_packettype;
	/*end of tmat_si_sj structure*/


	/**********************************************/
	/*                                            */
	/*end of variable declarations for parallelism*/
	/*                                            */
	/**********************************************/

	temps_debut=time(NULL);

	nseqs=inseq-istart;
	int_scale = 100;

	matrix.format=0;

	gop_scale=gep_scale=1.0;
	if (mult_aln.dnaflag) {
	  if (debug>1) fprintf(stdout,"matrix %s\n",pw_opt.dnamtrxname);
		if(pw_opt.dnamtrxname[0]=='\0')
			maxres = get_user_matrix(pw_opt.dnausermtrxname, TRUE, int_scale, &matrix);
        	else {
		        if (strcmp(pw_opt.dnamtrxname, "iub") == 0)
			    matptr=swgapdnamt;
       			else if (strcmp(pw_opt.dnamtrxname, "clustalw") == 0) { 
             			gop_scale=0.6667;
             			gep_scale=0.751;
				matptr=clustalvdnamt;
	  		}
			maxres=get_cl_matrix(mult_aln.dnaflag,matptr,gapptr,TRUE,int_scale,
					     &matrix);
        	}
		if (maxres == 0) return((sint)-1);
/* transition weights for A-G, C-T, C-U */
		matrix.score[0][6]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[6][0]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[2][19]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[19][2]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[2][20]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[20][2]=pw_opt.transition_weight*matrix.score[0][0];
    	}
	else {
if (debug>1) fprintf(stdout,"matrix %s\n",pw_opt.mtrxname);
		if(pw_opt.mtrxname[0]=='\0')
             		maxres = get_user_matrix(pw_opt.usermtrxname, TRUE, int_scale, &matrix);
		else {
       			if (strcmp(pw_opt.mtrxname, "blosum") == 0)
             			matptr=blosum30mt;
       			else if (strcmp(pw_opt.mtrxname, "pam") == 0)
             			matptr=pam350mt;
       			else if (strcmp(pw_opt.mtrxname, "gonnet") == 0) {
             			int_scale /= 10;
             			matptr=gon250mt;
          		}
       			else if (strcmp(pw_opt.mtrxname, "bin") == 0) {
             			int_scale /= 100;
             			matptr=bin30mt;
				gapptr=bin30gap;
				matrix.format=1;
          		}
       			else if (strcmp(pw_opt.mtrxname, "id") == 0)
             			matptr=idmat;
           		maxres = get_cl_matrix(mult_aln.dnaflag, matptr, gapptr, TRUE, int_scale, &matrix);
        	}
    	}
       	if (maxres == 0) return((sint)-1);

	/********************************************/
	/*                                          */
	/*              Parallelisation             */
	/*                                          */
	/********************************************/

	/*recuperation of the number of processes npes*/
	MPI_Comm_size(MPI_COMM_WORLD,&npes);
	
	nb_alignments=inseq*(inseq-1)/2-jstart*(jstart-1)/2;

	/*********************************************************/
	/*Sending to the slave processes the motifs if nanchors>0*/
	/*********************************************************/

	MPI_Bcast(&nanchors,1,MPI_INT,0,MPI_COMM_WORLD);

	if(nanchors>0)
	  {
	    motif_array_of_displacements[0]=0;
	    motif_array_of_displacements[1]=
	      (char *)&(motifs[0].pos1)-(char *)&(motifs[0].seq1);
	    motif_array_of_displacements[2]=
	      (char *)&(motifs[0].seq2)-(char *)&(motifs[0].seq1);
	    motif_array_of_displacements[3]=
	      (char *)&(motifs[0].pos2)-(char *)&(motifs[0].seq1);
	    motif_array_of_displacements[4]=
	      (char *)&(motifs[0].len)-(char *)&(motifs[0].seq1);
	    motif_array_of_displacements[5]=
	      (char *)&(motifs[0].weight)-(char *)&(motifs[0].seq1);
	   
	    MPI_Type_struct(6,motif_array_of_blocklengths,motif_array_of_displacements,
			    motif_array_of_types,&motif_packettype);
	    MPI_Type_commit(&motif_packettype);
	    
	    MPI_Bcast(motifs,nanchors,motif_packettype,0,MPI_COMM_WORLD);
	  }

	/**************************************************/
	/*sending to the slave processes the pw_opt struct*/
	/**************************************************/

	pw_opt_array_of_displacements[0]=0;
	pw_opt_array_of_displacements[1]=
	  (char *)&pw_opt.dna_ge_penalty-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[2]=
	  (char *)&pw_opt.prot_go_penalty-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[3]=
	  (char *)&pw_opt.prot_ge_penalty-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[4]=
	  (char *)&pw_opt.transition_weight-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[5]=
	  (char *)&pw_opt.matnum-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[6]=
	  (char *)&pw_opt.treealgo-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[7]=
	  pw_opt.mtrxname-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[8]=
	  (char *)&pw_opt.dnamatnum-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[9]=
	  pw_opt.dnamtrxname-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[10]=
	  pw_opt.usermtrxname-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[11]=
	  pw_opt.dnausermtrxname-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[12]=
	  (char *)&pw_opt.add_motif-(char *)&pw_opt.dna_go_penalty;
	pw_opt_array_of_displacements[13]=
	  (char *)&pw_opt.quick_pairalign-(char *)&pw_opt.dna_go_penalty;

	MPI_Type_struct(14,pw_opt_array_of_blocklengths,pw_opt_array_of_displacements,
			pw_opt_array_of_types,&pw_opt_packettype);
	MPI_Type_commit(&pw_opt_packettype);
 
	MPI_Bcast(&pw_opt,1,pw_opt_packettype,0,MPI_COMM_WORLD);
	
	/*******************************************************/
	/*sending to the slave processes the comp_matrix struct*/
	/*******************************************************/
	
	MPI_Type_vector(729,1,1,MPI_INT,&score_type);
	MPI_Type_commit(&score_type);	
	
	comp_matrix_array_of_types[0]=MPI_INT;
	comp_matrix_array_of_types[1]=score_type;
	comp_matrix_array_of_types[2]=MPI_INT;
	comp_matrix_array_of_types[3]=MPI_INT;
	
	comp_matrix_array_of_displacements[0]=0;
	comp_matrix_array_of_displacements[1]=
	  (char *)matrix.score-(char *)&matrix.format;
	comp_matrix_array_of_displacements[2]=
	  (char *)matrix.gapscore-(char *)&matrix.format;
	comp_matrix_array_of_displacements[3]=
	  (char *)&matrix.mat_avscore-(char *)&matrix.format;
	
	MPI_Type_struct(4,comp_matrix_array_of_blocklengths,comp_matrix_array_of_displacements,
		 	comp_matrix_array_of_types,&comp_matrix_packettype);
	MPI_Type_commit(&comp_matrix_packettype);

	MPI_Bcast(&matrix,1,comp_matrix_packettype,0,MPI_COMM_WORLD);
	
	/************************************************************/
	/*sending of the number of sequences to the slaves processes*/
	/************************************************************/

	MPI_Bcast(&(mult_aln.nseqs),1,MPI_INT,0,MPI_COMM_WORLD);
 
	/************************************************/
	/*sending of the dnaflag to the slaves processes*/
	/************************************************/

	MPI_Bcast(&(mult_aln.dnaflag),1,MPI_INT,0,MPI_COMM_WORLD);
	
	/************************************************************/
	/*sending to the slave processes of each sequence separately*/
	/*and before it its length                                  */
	/************************************************************/

	for(j=0;j<mult_aln.nseqs;j++)
	  {
	    MPI_Bcast(&(mult_aln.seqs[j].len),1,MPI_INT,0,MPI_COMM_WORLD);
	    MPI_Bcast(mult_aln.seqs[j].data,mult_aln.seqs[j].len,MPI_CHAR,0,MPI_COMM_WORLD);
	  }	  
	
	/*********************************************************/
	/*sending to the slave processes of int_scale, gop_scale,*/
	/*gep_scale, istart, inseq, jstart and jnseq             */
	/*********************************************************/
	
	MPI_Bcast(&int_scale,1,MPI_INT,0,MPI_COMM_WORLD);
	MPI_Bcast(&gop_scale,1,MPI_FLOAT,0,MPI_COMM_WORLD);
	MPI_Bcast(&gep_scale,1,MPI_FLOAT,0,MPI_COMM_WORLD);
	MPI_Bcast(&istart,1,MPI_INT,0,MPI_COMM_WORLD);
	MPI_Bcast(&inseq,1,MPI_INT,0,MPI_COMM_WORLD);
	MPI_Bcast(&jstart,1,MPI_INT,0,MPI_COMM_WORLD);
	MPI_Bcast(&jnseq,1,MPI_INT,0,MPI_COMM_WORLD);
	
	/***************************************************************************/
	/*sending to the slave processes the number of alignments they must process*/
	/***************************************************************************/

	

	/*memory allocation*/
	nb_alignments_for_processes=(int *)malloc(sizeof(int)*npes);
	alnstart_for_processes=(int *)malloc(sizeof(int)*npes);
	alnstop_for_processes=(int *)malloc(sizeof(int)*npes);
	/*end of memory allocation*/

	for(i=0;i<npes;i++)
	  {
	    nb_alignments_for_processes[i]=nb_alignments/npes;
	  }
	
	for(i=0;i<(nseqs*(nseqs-1)/2)%npes;i++)
	  {
	    (nb_alignments_for_processes[i])++;
	  }

	for(i=0;i<npes;i++)
	  {
	    printf("nb_alignements par process : %d\n",nb_alignments_for_processes[i]);
	  }

	alnstart_for_processes[0]=0;
	for(i=1;i<npes;i++)
	  {
	    alnstart_for_processes[i]=
	      alnstart_for_processes[i-1]+nb_alignments_for_processes[i-1];
	  }
	for(i=0;i<npes;i++)
	  {
	    alnstop_for_processes[i]=
	      alnstart_for_processes[i]+nb_alignments_for_processes[i];
	  }

	MPI_Scatter(alnstart_for_processes,1,MPI_INT,&(alnstart_for_processes[0]),1,MPI_INT,
		    0,MPI_COMM_WORLD);
	MPI_Scatter(alnstop_for_processes,1,MPI_INT,&(alnstop_for_processes[0]),1,MPI_INT,
		    0,MPI_COMM_WORLD);

	length_gather=nb_alignments/npes;
	if((nb_alignments%npes)!=0)
	  {
	    length_gather++;
	  }

	MPI_Bcast(&length_gather,1,MPI_INT,0,MPI_COMM_WORLD);
	
     	/*memory allocation*/
	tmat_si_sj=(TMAT_SI_SJ *)malloc(sizeof(TMAT_SI_SJ)*(alnstop_for_processes[0]-
							    alnstart_for_processes[0]));
	tmat_si_sj_total=(TMAT_SI_SJ *)malloc(sizeof(TMAT_SI_SJ)*length_gather*npes);
	/*end of memory allocation*/
	
	/*seeking of the si and sj to be aligned*/
	seek_si_sj_to_calculate(istart,inseq,jstart,jnseq,alnstart_for_processes[0],
				alnstop_for_processes[0],tmat_si_sj);

	/*pairwise alignments*/
	subpairalign(mult_aln,alnstart_for_processes[0],alnstop_for_processes[0],tmat_si_sj,
		     pw_opt,motifs,nseqs,matrix,int_scale,gop_scale,gep_scale);

	/***********************************************/
	/*reception from the slaves of their tmat_si_sj*/
	/***********************************************/
	
       	tmat_si_sj_array_of_displacements[0]=0;
	tmat_si_sj_array_of_displacements[1]=
	  (char *)&(tmat_si_sj[0].sj)-(char *)&(tmat_si_sj[0].si);
	tmat_si_sj_array_of_displacements[2]=
	  (char *)&(tmat_si_sj[0].tmat)-(char *)&(tmat_si_sj[0].si);
	
	MPI_Type_struct(3,tmat_si_sj_array_of_blocklengths,tmat_si_sj_array_of_displacements,
			tmat_si_sj_array_of_types,&tmat_si_sj_packettype);
	MPI_Type_commit(&tmat_si_sj_packettype);
	
	MPI_Gather(tmat_si_sj,length_gather,tmat_si_sj_packettype,tmat_si_sj_total,
		   length_gather,tmat_si_sj_packettype,0,MPI_COMM_WORLD);
	
	/*for(i=0;i<nseqs;i++)
	  {
	  tmat[i][i]=1.0;
	  }*/

	if(npes>nb_alignments)
	  {
	    factor=nb_alignments;
	  }
	else
	  {
	    factor=npes;
	  }

	for(i=0;i<length_gather*factor;i++)
	  {
	    if(tmat_si_sj_total[i].si>-1)
	      {
		tmat[tmat_si_sj_total[i].si][tmat_si_sj_total[i].sj]=
		  tmat_si_sj_total[i].tmat;
		tmat[tmat_si_sj_total[i].sj][tmat_si_sj_total[i].si]=
		  tmat[tmat_si_sj_total[i].si][tmat_si_sj_total[i].sj];
	      }
	  }
	  
	fichier_debug=fopen("distances_par","w");
	for(i=istart;i<istart+inseq;i++)
	  {
	    for(j=jstart;j<jstart+jnseq;j++)
	      {
		fprintf(fichier_debug,"%.5f ",tmat[i][j]);
	      }
	    fprintf(fichier_debug,"\n");
	  }
	fclose(fichier_debug);

	/*modif nico*/
	/*for(i=0;i<NUMRES;i++) 
	  matrix.score[i]=ckfree((void *)matrix.score[i]);
	  matrix.score=ckfree((void *)matrix.score);
	  matrix.gapscore=ckfree((void *)matrix.gapscore);*/

	/*memory desallocation*/
	free(nb_alignments_for_processes);
	free(alnstart_for_processes);
	free(alnstop_for_processes);
	free(tmat_si_sj);
	free(tmat_si_sj_total);
	/*end of memory desallocation*/
	
	/***********************************************/
	/*                                             */
	/*              End of Parallelisation         */
	/*                                             */
	/***********************************************/
	temps_fin=time(NULL);
	
	printf("temps d'execution de la partie parallele : %d\n",temps_fin-temps_debut);
	return((sint)1);
}


/*alignment of the pairs number alnstart to alnstop*/

void subpairalign(ALN mult_aln,sint alnstart,sint alnstop,TMAT_SI_SJ *tmat_si_sj,PW_OPT pw_opt,
		  MOTIF *motifs,sint nseqs,COMP_MATRIX matrix,sint int_scale,float gop_scale,
		  float gep_scale)
{
  static sint si,sj,i,j; 
  static sint n,m,len1,len2;
  static char c; 
  static sint gop,gep; 
  static sint **motif_score;
  static float pcidcount;
  int pairalign_count,nb_alignments;

  nb_alignments=alnstop-alnstart;
  for(pairalign_count=0;pairalign_count<nb_alignments;pairalign_count++)
    { 
      si=tmat_si_sj[pairalign_count].si;
      sj=tmat_si_sj[pairalign_count].sj;
     
      n = mult_aln.seqs[si].len;
      len1 = 0;
      for (i=0;i<n;i++) {
	c = mult_aln.seqs[si].data[i];
	if (isalpha(c)) len1++;
      }
      
      m = mult_aln.seqs[sj].len;
      if(n==0 || m==0) {
	tmat_si_sj[pairalign_count].tmat=1.0;
	continue;
      }
      
      if(si<sj)
	{
	  len2 = 0;
	  for(i=0;i<m;i++) 
	    {
	      c = mult_aln.seqs[sj].data[i];
	      if (isalpha(c)) len2++;
	    }
	  
	  if (debug>0) fprintf(stdout,"mat_avscore %d %d %d\n",matrix.mat_avscore,n,m);
	  if (mult_aln.dnaflag) {
	    gop = 2 * (float)pw_opt.dna_go_penalty * int_scale*gop_scale;
	    gep = pw_opt.dna_ge_penalty * int_scale*gep_scale;
	  }
	  else {
	    if(matrix.format==1) {
	      gop = matrix.mat_avscore * pw_opt.prot_go_penalty;
	      gep = matrix.mat_avscore * pw_opt.prot_ge_penalty;
	    }
	    else if(matrix.mat_avscore<=0) {
	      gop = 2 * (float)(pw_opt.prot_go_penalty + log((double)(MIN(n,m))));
	      gep = pw_opt.prot_ge_penalty;
	    }
	    else {
	      gop = 3 * matrix.mat_avscore * (float)(pw_opt.prot_go_penalty + log((double)(MIN(n,m))));
	      gep = pw_opt.prot_ge_penalty;
	    }
	  }
	  
	  
	  if(debug>0) fprintf(stdout,"go %d ge %d\n",(pint)gop,(pint)gep);
	  
	  /*
	    calculate the local motif scores
	  */
	  
	  if(motifs!=NULL) {
	    motif_score = (sint **) ckalloc( (n+1) * sizeof (sint *) );
	    for(i=0; i<n; i++)
	      motif_score[i] = (sint *) ckalloc( (m+2) * sizeof(sint) );
	    
	    calc_pw_motif_scores(mult_aln.seqs,si,sj,n,m,motifs,motif_score);
	    
	    pcidcount=forward_passmotif(&mult_aln.seqs[si].data[0],
					&mult_aln.seqs[sj].data[0],
					n, m,gop, gep, matrix, motif_score,pw_opt.add_motif);
	    for (i=0;i<n;i++)
	      motif_score[i]=ckfree((void *)motif_score[i]);
	    motif_score=ckfree((void *)motif_score);
	  }
	  else
	    {
	      pcidcount=forward_pass(&mult_aln.seqs[si].data[0],
				     &mult_aln.seqs[sj].data[0],
				     n,m,gop,gep,matrix);
	    }      
	  
	  pcidcount*=100.0;
	  if(len1==0 || len2==0) pcidcount=0;
	  else
	    pcidcount/=(float)MIN(len1,len2);
	  
	  
	  info("Sequences (%d:%d) Aligned. Score:  %d",(pint)si+1,(pint)sj+1, (pint)pcidcount);
	  
	  tmat_si_sj[pairalign_count].tmat=((float)100.0-pcidcount)/(float)100.0;
	  tmat[sj][si] = tmat[si][sj] = ((float)100.0 - pcidcount)/(float)100.0;
	  
	}
    }
}


/***********************************************************************/
/*                                                                     */
/*Procedure which seeks the si and sj for which tmat must be calculated*/
/*                                                                     */
/***********************************************************************/

void seek_si_sj_to_calculate(int istart,int inseq,int jstart,int jnseq,int alnstart,
			     int alnstop,TMAT_SI_SJ *tmat_si_sj)
{
  /*variable declarations*/
  int i,si,sj,offset,first_sj,width,pairalign_count;
  /*end of variable declarations*/

  width=jnseq-jstart;
  offset=alnstart+1;
  si=istart;
  sj=jstart;
  first_sj=jstart;

  if(si>=sj)
    {
      first_sj++;
      sj=first_sj;
      width--;
    }
  
  while(1)
    {
      if(offset>width)
	{
	  offset-=width;
	  si++;
	  if(si>=sj)
	    {
	      first_sj++;
	      sj=first_sj;
	      width--;
	    }
	}
      else
	{
	  sj+=(offset-1);
	  break;
	}
    }
  
  pairalign_count=0;
  while(1)
    {
      if(si<sj)
	{
	  tmat_si_sj[pairalign_count].si=si;
	  tmat_si_sj[pairalign_count].sj=sj;
	 
	  pairalign_count++;
	  if(pairalign_count==(alnstop-alnstart))
	    {
	      break;
	    }
	}
      si_sj_update(inseq,jstart,&si,&sj);
    }
}

/*************************************/
/*                                   */
/*update of si and sj in subpairalign*/
/*                                   */
/*************************************/

void si_sj_update(int inseq,int jstart,int *si,int *sj)
{
  if((*sj+1)==inseq)
    {
      *sj=jstart;
      (*si)++;
    }
  else
    {
      (*sj)++;
    }
}

#else /*non-MPI*/

sint pairalign(ALN mult_aln,sint istart, sint inseq, sint jstart, sint jnseq, double **tmat,PW_OPT pw_opt,MOTIF *motifs,sint nanchors)
{
	static sint    	si, sj, i, j;
	static sint    	n,m,len1,len2;
	static sint    	nseqs,minlen,maxres;
	static char    	c;
	static float   	gop_scale,gep_scale;
	static sint   	*matptr,*gapptr;
	static float 	pcidcount;
	static sint 	gop, gep;
	static COMP_MATRIX matrix;
	static sint	int_scale;
	static sint	**motif_score;

	nseqs=inseq-istart;
	int_scale = 100;

	matrix.format=0;

	gop_scale=gep_scale=1.0;
	if (mult_aln.dnaflag) {
if (debug>1) fprintf(stdout,"matrix %s\n",pw_opt.dnamtrxname);
		if(pw_opt.dnamtrxname[0]=='\0')
			maxres = get_user_matrix(pw_opt.dnausermtrxname, TRUE, int_scale, &matrix);
        	else {
			if (strcmp(pw_opt.dnamtrxname, "iub") == 0)
             			matptr=swgapdnamt;
       			else if (strcmp(pw_opt.dnamtrxname, "clustalw") == 0) { 
             			gop_scale=0.6667;
             			gep_scale=0.751;
				matptr=clustalvdnamt;
	  		}
			maxres = get_cl_matrix(mult_aln.dnaflag, matptr, gapptr, TRUE, int_scale, &matrix);
        	}
		if (maxres == 0) return((sint)-1);
/* transition weights for A-G, C-T, C-U */
		matrix.score[0][6]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[6][0]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[2][19]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[19][2]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[2][20]=pw_opt.transition_weight*matrix.score[0][0];
		matrix.score[20][2]=pw_opt.transition_weight*matrix.score[0][0];
    	}
	else {
if (debug>1) fprintf(stdout,"matrix %s\n",pw_opt.mtrxname);
		if(pw_opt.mtrxname[0]=='\0')
             		maxres = get_user_matrix(pw_opt.usermtrxname, TRUE, int_scale, &matrix);
		else {
       			if (strcmp(pw_opt.mtrxname, "blosum") == 0)
             			matptr=blosum30mt;
       			else if (strcmp(pw_opt.mtrxname, "pam") == 0)
             			matptr=pam350mt;
       			else if (strcmp(pw_opt.mtrxname, "gonnet") == 0) {
             			int_scale /= 10;
             			matptr=gon250mt;
          		}
       			else if (strcmp(pw_opt.mtrxname, "bin") == 0) {
             			int_scale /= 100;
             			matptr=bin30mt;
				gapptr=bin30gap;
				matrix.format=1;
          		}
       			else if (strcmp(pw_opt.mtrxname, "id") == 0)
             			matptr=idmat;
           		maxres = get_cl_matrix(mult_aln.dnaflag, matptr, gapptr, TRUE, int_scale, &matrix);
        	}
    	}
       	if (maxres == 0) return((sint)-1);

	for (si=MAX(0,istart);si<nseqs && si<inseq;si++) {
		n = mult_aln.seqs[si].len;
		len1 = 0;
		for (i=0;i<n;i++) {
			c = mult_aln.seqs[si].data[i];
			if (isalpha(c)) len1++;
		}

		for (sj=MAX(si+1,jstart);sj<nseqs && sj<jnseq;sj++) {
			m = mult_aln.seqs[sj].len;
			if(n==0 || m==0) {
				tmat[si][sj]=1.0;
				tmat[sj][si]=1.0;
				continue;
			}
			len2 = 0;
			for (i=0;i<m;i++) {
				c = mult_aln.seqs[sj].data[i];
				if (isalpha(c)) len2++;
			}

if (debug>0) fprintf(stdout,"mat_avscore %d %d %d\n",matrix.mat_avscore,n,m);
        		if (mult_aln.dnaflag) {
           			gop = 2 * (float)pw_opt.dna_go_penalty * int_scale*gop_scale;
           			gep = pw_opt.dna_ge_penalty * int_scale*gep_scale;
        		}
        		else {
                                if(matrix.format==1) {
              				gop = matrix.mat_avscore * pw_opt.prot_go_penalty;
           				gep = matrix.mat_avscore * pw_opt.prot_ge_penalty;
                                }
				else if(matrix.mat_avscore<=0) {
              				gop = 2 * (float)(pw_opt.prot_go_penalty + log((double)(MIN(n,m))));
           				gep = pw_opt.prot_ge_penalty;
				}
				else {
              				gop = 3 * matrix.mat_avscore * (float)(pw_opt.prot_go_penalty + log((double)(MIN(n,m))));
           				gep = pw_opt.prot_ge_penalty;
				}
        		}


if(debug>0) fprintf(stdout,"go %d ge %d\n",(pint)gop,(pint)gep);

/*
   calculate the local motif scores
*/

			if(motifs!=NULL) {
        			motif_score = (sint **) ckalloc( (n+1) * sizeof (sint *) );
        			for(i=0; i<n; i++)
                			motif_score[i] = (sint *) ckalloc( (m+2) * sizeof(sint) );

        			calc_pw_motif_scores(mult_aln.seqs,si,sj,n,m,motifs,motif_score);

        			pcidcount=forward_passmotif(&mult_aln.seqs[si].data[0],
						 &mult_aln.seqs[sj].data[0],
						 n, m,gop, gep, matrix, motif_score,pw_opt.add_motif);
        			for (i=0;i<n;i++)
                			motif_score[i]=ckfree((void *)motif_score[i]);
        			motif_score=ckfree((void *)motif_score);
			}
			else {

        			pcidcount=forward_pass(&mult_aln.seqs[si].data[0],
						 &mult_aln.seqs[sj].data[0],
						 n, m,gop, gep, matrix);

			}



			pcidcount*=100.0;
			if(len1==0 || len2==0) pcidcount=0;
			else
				pcidcount /= (float)MIN(len1,len2);


        		info("Sequences (%d:%d) Aligned. Score:  %d",
                                      (pint)si+1,(pint)sj+1, 
                                      (pint)pcidcount);

        		tmat[sj][si] = tmat[si][sj] = ((float)100.0 - pcidcount)/(float)100.0;
   		}
	}

  	return((sint)1);
}

#endif /*non-MPI*/

/*
  Modified Needleman-Wunsch dynamic programming. The percent identity of the alignment
  is calculated along the way.
  seq1	the first sequence
  seq2	the second sequence
  n	the length of seq1
  m	the length of seq2
  gop	the gap opening penalty
  gep	the gap extension penalty
  matrix	the comparison matrix
  
  Returns a total count of the residue identities.
*/

static float forward_pass(char *seq1, char *seq2, sint n, sint m,sint gop, sint gep, COMP_MATRIX matrix)
{

  	sint i,j,s;
  	sint maxscore;
  	pwint INS,match,previous,t;
  	pwint ppc,ppc1,tmp;
	static pwint 	*MATCH, *DEL;
	static pwint	*pcid;
  	float pcidcount;

	pcid = (pwint *)ckalloc((n+m+1) * sizeof(pwint));
	MATCH = (pwint *)ckalloc((n+m+1) * sizeof(pwint));
	DEL = (pwint *)ckalloc((n+m+1) * sizeof(pwint));

  	maxscore = 0;
  	for (i=0;i<=m;i++) {
       		MATCH[i] = 0;
       		DEL[i] = -gop;
    		}

	pcidcount=0;

  	for (i=1;i<=n;i++) {
        	match = previous = 0;
		INS = -gop;
		ppc = ppc1 = 0;

        	for (j=1;j<=m;j++) {
              		INS -= gep; 
              		t = match - gop - gep;
              		if (INS<t) INS = t;

              		DEL[j] -= gep;
              		t = MATCH[j] - gop - gep;
              		if (DEL[j]<t) DEL[j] = t;

			s=0;
			tmp=ppc1;			/* ppc1 = pcid[i-1][j-1] */
      			if(isalpha(seq1[i-1]) && isalpha(seq2[j-1])) {
	      			s=matrix.score[(int)seq1[i-1]-'a'][(int)seq2[j-1]-'a'];
      				if(seq1[i-1]==seq2[j-1]) 
					tmp+=1;			
			}
	      		match = previous + s;
       			if (match<INS) {
				match = INS;
				tmp=ppc;			/* ppc = pcid[i][j-1] */
			}
       			if (match<DEL[j]) {
				match = DEL[j];
				tmp=pcid[j];			/* pcid[j] = pcid[i-1][j] */
			}
       			if (match<0) 
				match = tmp = 0;

              		previous = MATCH[j];
              		MATCH[j] = match;
	      		ppc=tmp;
	      		ppc1=pcid[j];
	      		pcid[j]=tmp;

              		if (match > maxscore) {
		   		pcidcount=pcid[j];
                   		maxscore = match;
                	}
           	}
     	}

   	pcid=ckfree((void *)pcid);
   	MATCH=ckfree((void *)MATCH);
   	DEL=ckfree((void *)DEL);

	return pcidcount;
}


/*
   Modified Needleman-Wunsch dynamic programming. The percent identity of the alignment
   is calculated along the way.
	seq1	the first sequence
	seq2	the second sequence
	n	the length of seq1
	m	the length of seq2
	gop	the gap opening penalty
	gep	the gap extension penalty
	matrix	the comparison matrix

   Returns a total count of the residue identities.
*/

static float forward_passmotif(char *seq1, char *seq2, sint n, sint m,sint gop, sint gep,  COMP_MATRIX matrix, sint **motif_score, Boolean add_motif)
{

  	sint i,j,s;
  	sint maxscore;
  	pwint INS,match,previous,t;
  	pwint ppc,ppc1,tmp;
	static pwint 	*MATCH, *DEL;
	static pwint	*pcid;
  	float pcidcount;

	pcid = (pwint *)ckalloc((n+m+1) * sizeof(pwint));
	MATCH = (pwint *)ckalloc((n+m+1) * sizeof(pwint));
	DEL = (pwint *)ckalloc((n+m+1) * sizeof(pwint));

  	maxscore = 0;
  	for (i=0;i<=m;i++) {
       		MATCH[i] = 0;
       		DEL[i] = -gop;
    		}

	pcidcount=0;

  	for (i=1;i<=n;i++) {
        	match = previous = 0;
		INS = -gop;
		ppc = ppc1 = 0;

        	for (j=1;j<=m;j++) {
              		INS -= gep; 
              		t = match - gop - gep;
              		if (INS<t) INS = t;

              		DEL[j] -= gep;
              		t = MATCH[j] - gop - gep;
              		if (DEL[j]<t) DEL[j] = t;


			s=0;
			tmp=ppc1;			/* ppc1 = pcid[i-1][j-1] */
      			if(isalpha(seq1[i-1]) && isalpha(seq2[j-1])) {
	      			s=matrix.score[(int)seq1[i-1]-'a'][(int)seq2[j-1]-'a']+motif_score[i-1][j-1];
      				if(seq1[i-1]==seq2[j-1]) 
					tmp+=1;			
				else if(add_motif && motif_score[i-1][j-1]>0)  
					tmp+=1;
			}
	      		match = previous + s;
       			if (match<INS) {
				match = INS;
				tmp=ppc;			/* ppc = pcid[i][j-1] */
			}
       			if (match<DEL[j]) {
				match = DEL[j];
				tmp=pcid[j];			/* pcid[j] = pcid[i-1][j] */
			}
       			if (match<0) 
				match = tmp = 0;

              		previous = MATCH[j];
              		MATCH[j] = match;
	      		ppc=tmp;
	      		ppc1=pcid[j];
	      		pcid[j]=tmp;

              		if (match > maxscore) {
		   		pcidcount=pcid[j];
                   		maxscore = match;
                	}
           	}
     	}

   	pcid=ckfree((void *)pcid);
   	MATCH=ckfree((void *)MATCH);
   	DEL=ckfree((void *)DEL);

	return pcidcount;
}
