#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <ctype.h>
#include "clustalw.h"

static char revision_level[10]="";
static char help_file_name[FILENAMELEN]="";
static Boolean usemenu=FALSE;

void set_usemenu(Boolean flag)
{
	usemenu=flag;
}

Boolean get_usemenu(void)
{
	return usemenu;
}


void set_revision_level(char *rev)
{
	strcpy(revision_level,rev);
}

char * get_revision_level(void)
{
	char *rev;

	rev=(char *)ckalloc((strlen(revision_level)+2)*sizeof(char));
	strcpy(rev,revision_level);

	return rev;
}

void set_help_file_name(char *filename)
{
	strcpy(help_file_name,filename);
}

char * get_help_file_name(void)
{
        char *file_name;

        file_name=ckalloc((strlen(help_file_name)+2)*sizeof(char));
        strcpy(file_name,help_file_name);

        return file_name;
}

/*
*	ckalloc()
*
*	Tries to allocate "bytes" bytes of memory. Exits program if failed.
*	Return value:
*		Generic pointer to the newly allocated memory.
*/

void *ckalloc(size_t bytes)
{
	register void *ret;
	
	if( (ret = calloc(bytes, sizeof(char))) == NULL)
		fatal("Out of memory\n");
	else
		return ret;	

	return ret;	
}

/*
*	ckrealloc()
*
*	Tries to reallocate "bytes" bytes of memory. Exits program if failed.
*	Return value:
*		Generic pointer to the re-allocated memory.
*/

void *ckrealloc(void *ptr, size_t bytes)
{
	register void *ret=NULL;

	if (ptr == NULL)	
		fatal("Bad call to ckrealloc\n");
	else if( (ret = realloc(ptr, bytes)) == NULL)
		fatal("Out of memory\n");
	else
		return ret;	

	return ret;	
}

/*
*	ckfree()
*
*	Tries to free memory allocated by ckalloc.
*	Return value:
*		None.
*/

void *ckfree(void *ptr)
{
	if (ptr == NULL)
		warning("Bad call to ckfree\n");
	else {
	 	free(ptr);
		ptr = NULL;
	}
	return ptr;
}


/*
*	rtrim()
*
*	Removes trailing blanks from a string
*
*	Return values:
*		Pointer to the processed string
*/

char * rtrim(char *str)
{
	register int p;

	p = strlen(str) - 1;
	
	while ( isspace(str[p]) )
		p--;
		
	str[p + 1] = EOS;
	
	return str;
}


/*
*	blank_to_()
*
*	Replace blanks in a string with underscores
*
*       Also replaces , ; : ( or ) with _
*
*	Return value:
*		Pointer to the processed string
*/

char * blank_to_(char *str)
{
	int i,p;

	p = strlen(str);
	
	for(i=0;i<p;i++) 
		if(
                     (str[i]==' ') ||
                     (str[i]==';') ||
                     (str[i]==',') ||
                     (str[i]=='(') ||
                     (str[i]==')') ||
                     (str[i]==':')
                  )
                      str[i] = '_';
	
	return str;
}


/*
*	upstr()
*
*	Converts string str to uppercase.
*	Return values:
*		Pointer to the converted string.
*/

char * upstr(char *str)
{
	register char *s = str;
	
	while( (*s = toupper(*s)) )
		s++;
		
	return str;
}

/*
*	lowstr()
*
*	Converts string str to lower case.
*	Return values:
*		Pointer to the converted string.
*/

char * lowstr(char *str)
{
	sint i,len;

	len=strlen(str);
	for(i=0;i<len;i++) {
		if(isupper(str[i])) str[i] = tolower(str[i]);
	}
		
	return str;
}

void getstr(char *instr,char *outstr)
{	
	fprintf(stdout,"%s: ",instr);
	gets(outstr);
}

double getreal(char *instr,double minx,double maxx,double def)
{
	int status;
	float ret;
	char line[MAXLINE];	
	
	while(TRUE) {
		fprintf(stdout,"%s (%.1f-%.1f)   [%.1f]: ",instr,minx,maxx,def);
		gets(line);
		status=sscanf(line,"%f",&ret);
		if(status == EOF) return def;
		if(ret>maxx) {
			fprintf(stdout,"ERROR: Max. value=%.1f\n\n",maxx);
			continue;
		}
		if(ret<minx) {
			fprintf(stdout,"ERROR: Min. value=%.1f\n\n",minx);
			continue;
		}
		break;
	}
	return (double)ret;
}


int getint(char *instr,int minx,int maxx, int def)
{
	int ret,status;
	char line[MAXLINE];	

	while(TRUE) {
		fprintf(stdout,"%s (%d..%d)    [%d]: ",
		instr,(pint)minx,(pint)maxx,(pint)def);
		gets(line);
		status=sscanf(line,"%d",&ret);
		if(status == EOF) return def;
		if(ret>maxx) {
			fprintf(stdout,"ERROR: Max. value=%d\n\n",(pint)maxx);
			continue;
		}
		if(ret<minx) {
			fprintf(stdout,"ERROR: Min. value=%d\n\n",(pint)minx);
			continue;
		}
		break;
	}
	return ret;
}

void do_system(void)
{
	char line[MAXLINE];
	
	getstr("\n\nEnter system command",line);
	if(*line != EOS)
		system(line);
	fprintf(stdout,"\n\n");
}


Boolean linetype(char *line,char *code)
{
	return( strncmp(line,code,strlen(code)) == 0 );
}

Boolean keyword(char *line,char *code)
{
	int i;
	char key[MAXLINE];

	for(i=0;!isspace(line[i]) && line[i]!=EOS;i++)
		key[i]=line[i];
	key[i]=EOS;
	return( strcmp(key,code) == 0 );
}

Boolean blankline(char *line)
{
	int i;

	for(i=0;line[i]!='\n' && line[i]!=EOS;i++) {
		if( isdigit(line[i]) ||
		    isspace(line[i]) ||
		    (line[i] == '*') ||
		    (line[i] == ':') ||
                    (line[i] == '.')) 
			;
		else
			return FALSE;
	}
	return TRUE;
}


void get_path(char *str,char *path)
{
	register int i;
	
	strcpy(path,str);
	for(i=strlen(path)-1;i>-1;--i) {
		if(str[i]==DIRDELIM) {
			i = -1;
			break;
		}
		if(str[i]=='.') break;
	}
	if(i<0)
		strcat(path,".");
	else
		path[i+1]=EOS;
}

void alloc_aln(sint nseqs,ALNPTR mult_aln)
{
	sint i;

	mult_aln->seqs = (SEQ *)ckalloc((nseqs+1) * sizeof (SEQ));
	for(i=0;i<nseqs;i++) {
		mult_aln->seqs[i].name = (char *)ckalloc((MAXNAMES+1) * sizeof (char));
		mult_aln->seqs[i].title = (char *)ckalloc((MAXTITLES+1) * sizeof (char));
                mult_aln->seqs[i].data=NULL;
                mult_aln->seqs[i].mask=NULL;
		mult_aln->seqs[i].weight=100;
		mult_aln->seqs[i].len=0;
	}
	mult_aln->prf1.ss.struct_penalties=NONE;
	mult_aln->prf1.ss.name=NULL;
	mult_aln->prf1.ss.gap_penalty_mask=NULL;
	mult_aln->prf1.ss.sec_struct_mask=NULL;
	mult_aln->prf2.ss.struct_penalties=NONE;
	mult_aln->prf2.ss.name=NULL;
	mult_aln->prf2.ss.gap_penalty_mask=NULL;
	mult_aln->prf2.ss.sec_struct_mask=NULL;

}

void realloc_aln(sint first_seq,sint nseqs,ALNPTR mult_aln)
{
	sint i;

	mult_aln->seqs = (SEQ *)ckrealloc(mult_aln->seqs,(first_seq+nseqs+1) * sizeof (SEQ));
	for(i=first_seq;i<first_seq+nseqs;i++) {
                mult_aln->seqs[i].name = (char *)ckalloc((MAXNAMES+1) * sizeof (char));
                mult_aln->seqs[i].title = (char *)ckalloc((MAXTITLES+1) * sizeof (char));
                mult_aln->seqs[i].data=NULL;
                mult_aln->seqs[i].mask=NULL;
                mult_aln->seqs[i].weight=100;
                mult_aln->seqs[i].len=0;
        }
	mult_aln->prf2.ss.name=NULL;
	mult_aln->prf2.ss.gap_penalty_mask=NULL;
	mult_aln->prf2.ss.sec_struct_mask=NULL;

}

void free_aln(ALNPTR mult_aln)
{
	sint i;

	if(mult_aln->nseqs<=0) return;

        for(i=0;i<mult_aln->nseqs;i++) {
		ckfree(mult_aln->seqs[i].name);
		ckfree(mult_aln->seqs[i].title);
		ckfree(mult_aln->seqs[i].data);
		ckfree(mult_aln->seqs[i].mask);
	}
	ckfree(mult_aln->seqs);
}

void alloc_seq(SEQ *seq,sint length)
{
	seq->data = (char *)ckalloc((length+2) * sizeof (char));
	seq->mask = (char *)ckalloc((length+2) * sizeof (char));
}

void realloc_seq(SEQ *seq,sint length)
{
	seq->data = (char *)ckrealloc(seq->data, (length+2) * sizeof (char));
	seq->mask = (char *)ckrealloc(seq->mask, (length+2) * sizeof (char));
}

int getargs(char *inline1,char *args[],int max)
{

        char    *inptr;
/*
#ifndef MAC
        char    *strtok(char *s1, const char *s2);
#endif
*/
        int     i;

        inptr=inline1;
        for (i=0;i<=max;i++)
        {
                if ((args[i]=strtok(inptr," \t\n"))==NULL)
                        break;
                inptr=NULL;
        }

        return(i);
}


/* 
   count the number of identities between two sequences
	seq1	first sequence
	seq2	second sequence
*/
float countid(SEQ seq1,SEQ seq2)
{
   char c1,c2;
   sint i;
   sint count,total;
   float score;

   count = total = 0;
   for (i=0;i<seq1.len && i<seq2.len;i++) {
     c1 = seq1.data[i];
     c2 = seq2.data[i];
     if (isalpha(c1)) {
       total++;
       if (c1 == c2) count++;
     }

   }

   if(total==0) score=0;
   else
   score = 100.0 * (float)count / (float)total;
   return(score);

}

void create_parameter_output(OPT opt,ALN mult_aln)
{
	char parname[FILENAMELEN+1], temp[FILENAMELEN+1];
	char path[FILENAMELEN+1];
	FILE *parout;
	Boolean usemenu;

        get_path(mult_aln.filename,path);
        strcpy(parname,path);
        strcat(parname,"par");

	usemenu=get_usemenu();
	if(usemenu) {
        	fprintf(stdout,"\nEnter a name for the parameter output file [%s]: ",
                                           parname);
               	gets(temp);
               	if(*temp != EOS)
                       	strcpy(parname,temp);
       	}

/* create a file with execute permissions first */
	remove(parname);
/*
	fd = creat(parname, 0777);
	close(fd);
*/

        if((parout = open_explicit_file(parname))==NULL) return;

        fprintf(parout,"clustalw \\\n");
	if ((mult_aln.nseqs>0) && (mult_aln.prf1.nseqs<=0)) fprintf(parout,"-infile=%s \\\n",mult_aln.filename);
	if (mult_aln.prf1.nseqs>0) fprintf(parout,"-profile1=%s\\\n",mult_aln.prf1.filename);
	if (mult_aln.prf2.nseqs>0) fprintf(parout,"-profile2=%s\\\n",mult_aln.prf2.filename);
	if (mult_aln.dnaflag == TRUE) fprintf(parout,"-type=dna \\\n");
	else                 fprintf(parout,"-type=protein \\\n");

	if (opt.pw_opt->quick_pairalign) {
		fprintf(parout,"-quicktree \\\n");
		if (!mult_aln.dnaflag) {
			fprintf(parout,"-ktuple=%d \\\n",(pint)opt.quickpw_opt->dna_ktup);
     			fprintf(parout,"-window=%d \\\n",(pint)opt.quickpw_opt->dna_window);
     			fprintf(parout,"-pairgap=%d \\\n",(pint)opt.quickpw_opt->dna_wind_gap);
     			fprintf(parout,"-topdiags=%d \\\n",(pint)opt.quickpw_opt->dna_signif);    
		}
		else {
			fprintf(parout,"-ktuple=%d \\\n",(pint)opt.quickpw_opt->prot_ktup);
     			fprintf(parout,"-window=%d \\\n",(pint)opt.quickpw_opt->prot_window);
     			fprintf(parout,"-pairgap=%d \\\n",(pint)opt.quickpw_opt->prot_wind_gap);
     			fprintf(parout,"-topdiags=%d \\\n",(pint)opt.quickpw_opt->prot_signif);    
		}
     		if (opt.quickpw_opt->percent) fprintf(parout,"/score=percent \\\n");      
     		else         fprintf(parout,"-score=absolute \\\n");      
	}
	else {
		if (!mult_aln.dnaflag) {
			fprintf(parout,"-pwmatrix=%s \\\n",opt.pw_opt->mtrxname);
			fprintf(parout,"-pwgapopen=%.2f \\\n",opt.pw_opt->prot_go_penalty);
			fprintf(parout,"-pwgapext=%.2f \\\n",opt.pw_opt->prot_ge_penalty);
		}
		else {
			fprintf(parout,"-pwgapopen=%.2f \\\n",opt.pw_opt->dna_go_penalty);
			fprintf(parout,"-pwgapext=%.2f \\\n",opt.pw_opt->dna_ge_penalty);
		}
	}

	if (!mult_aln.dnaflag) {
		fprintf(parout,"-matrix=%s \\\n",opt.mult_opt->mtrxname);
		fprintf(parout,"-gapopen=%.2f \\\n",opt.mult_opt->prot_gap_open);
		fprintf(parout,"-gapext=%.2f \\\n",opt.mult_opt->prot_gap_extend);
	}
	else {
		fprintf(parout,"-gapopen=%.2f \\\n",opt.mult_opt->dna_gap_open);
		fprintf(parout,"-gapext=%.2f \\\n",opt.mult_opt->dna_gap_extend);
	}

	fprintf(parout,"-maxdiv=%d \\\n",(pint)opt.mult_opt->divergence_cutoff);
	if (!opt.mult_opt->gap_opt->use_endgaps) fprintf(parout,"-endgaps \\\n");    

	if (!mult_aln.dnaflag) {
     		if (opt.mult_opt->neg_matrix) fprintf(parout,"-negative \\\n");   
     		if (opt.mult_opt->gap_opt->no_pref_penalties) fprintf(parout,"-nopgap \\\n");     
     		if (opt.mult_opt->gap_opt->no_hyd_penalties) fprintf(parout,"-nohgap \\\n");     
     		if (opt.mult_opt->gap_opt->no_var_penalties) fprintf(parout,"/novgap \\\n");     
    		fprintf(parout,"-hgapresidues=%s \\\n",opt.mult_opt->gap_opt->hyd_residues);
     		fprintf(parout,"-gapdist=%d \\\n",(pint)opt.mult_opt->gap_opt->gap_dist);     
	}
	else {
		fprintf(parout,"-transweight=%.2f \\\n",opt.mult_opt->transition_weight);
	}

     	if (opt.alnout_opt->output_gcg) fprintf(parout,"-output=gcg \\\n");
     	else if (opt.alnout_opt->output_gde) fprintf(parout,"-output=gde \\\n");
     	else if (opt.alnout_opt->output_nbrf) fprintf(parout,"-output=pir \\\n");
     	else if (opt.alnout_opt->output_phylip) fprintf(parout,"-output=phylip \\\n");
     	if (opt.alnout_opt->output_order==ALIGNED) fprintf(parout,"-outorder=aligned \\\n");  
     	else                      fprintf(parout,"-outorder=input \\\n");  
     	if (opt.alnout_opt->output_gde)
		if (opt.alnout_opt->lowercase) fprintf(parout,"-case=lower \\\n");
		else           fprintf(parout,"-case=upper \\\n");


        fprintf(parout,"-interactive\n");

	fclose(parout);

}




FILE *open_explicit_file(char *file_name)
{
        FILE * file_handle;

        if (*file_name == EOS) {
                error("Bad output file [%s]",file_name);
                return NULL;
        }
#ifdef VMS
        if((file_handle=fopen(file_name,"w","rat=cr","rfm=var"))==NULL) {
#else
        if((file_handle=fopen(file_name,"w"))==NULL) {
#endif
                error("Cannot open output file [%s]",file_name);
                return NULL;
        }
        return file_handle;
}

