#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <vibrant.h>
#include <document.h>

#include "clustalw.h"
#include "xmenu.h"


static void RemoveWin(WindoW w);
static void QuitWinW(WindoW w);
static void QuitWinI(IteM i);
static void QuitHelpW(WindoW w);
static void QuitHelpB(ButtoN b);
static void SearchStrWin (IteM item);
static void SavePSSeqWin (IteM item);
static void SavePSPrf1Win (IteM item);
static void SavePSPrf2Win (IteM item);
static void SaveSeqFileWin (IteM item);
static void SavePrf1FileWin (IteM item);
static void SavePrf2FileWin (IteM item);
static void OpenColorParWin (IteM item);
static void SearchStr(ButtoN but);
static void SavePSSeqFile(ButtoN but);
static void SavePSPrf1File(ButtoN but);
static void SavePSPrf2File(ButtoN but);
static void SaveSeqFile(ButtoN but);
static void SavePrf1File(ButtoN but);
static void SavePrf2File(ButtoN but);
static void SaveScoresWin (IteM item);
static void SaveScores(ButtoN but);
static void OpenColorPar(ButtoN but);
static void CancelWin(ButtoN but);
static void SaveTreeWin (IteM item);
static void MakeBallastMotifWin (IteM item);
static void CAlignWin (IteM item);
static void RealignSeqsWin (IteM item);
static void RealignSeqRangeWin (IteM item);
static void DrawTreeWin (IteM item);
static void AlignFromTreeWin(IteM item);
static void PrfPrfAlignWin(IteM item);
static void PrfPrfTreeAlignWin(IteM item);
static void SeqPrfAlignWin(IteM item);
static void SeqPrfTreeAlignWin(IteM item);
static void BootstrapTreeWin (IteM item);
static void CreateAlignTree(ButtoN but);
static void MakeBallastMotifs(ButtoN but);
static void CompleteAlign(ButtoN but);
static void RealignSeqs(ButtoN but);
static void RealignSeqRange(ButtoN but);
static void DrawTree(ButtoN but);
static void AlignFromTree(ButtoN but);
static void PrfPrfAlign(ButtoN but);
static void PrfPrfTreeAlign(ButtoN but);
static void SeqPrfAlign(ButtoN but);
static void SeqPrfTreeAlign(ButtoN but);
static void BootstrapTree(ButtoN but);
static void OpenSeqFile (IteM item);
static void AppendSeqFile (IteM item);
static void OpenPrf1File (IteM item);
static void OpenPrf2File (IteM item);
static void ScoreWin(IteM item);
static void SegmentWin(IteM item);
static void PWParameters(IteM item);
static void MotifParameters(IteM item);
static void MultiParameters(IteM item);
static void GapParameters(IteM item);
static void SSParameters(IteM item);
static void OutputParameters(IteM item);
static void OutputTreeParameters(IteM item);
static void HelpProc(IteM item);
static void DefColorPar(IteM item);
static void BlackandWhite(IteM item);
static void set_reset_new_gaps(IteM i);
static void set_reset_all_gaps(IteM i);
static void SearchStringAgain(ButtoN but);

static PopuP make_toggle(GrouP g,CharPtr title,CharPtr true_text, CharPtr false_text,
				 Boolean *value,PupActnProc SetProc);
static PrompT make_scale(GrouP g,CharPtr title,int length,int value,int max,BarScrlProc SetProc);
static PrompT make_prompt(GrouP g,CharPtr title);

static void CutSequences(IteM item);
static void PasteSequences(IteM item);
static void RemoveGaps(IteM item);
static void RemoveGapPos(IteM item);

static void SelectSeqs(IteM item);
static void SelectPrf1(IteM item);
static void SelectPrf2(IteM item);
static void MergeProfiles(IteM item);
static void ClearSeqs(IteM item);
 
static void cut_multiplem(void);
static void cut_profile1(void);
static void cut_profile2(void);
static void ssave(SEQ seq);
static void sscpy(SEQ *seq1, SEQ seq2);
static void sload(SEQ *seq);
static void clear_seqrange(spanel p);
static void select_seqs(spanel p,Boolean flag);
static void remove_gap_pos(sint prf_no,ALN mult_aln);
static void reload_alignment(sint nseqs,SEQ *seqs);

static void make_menu_headers(WindoW w);
static void make_help_menu(void);
static void make_score_menu(void);
static void make_file_menu(void);
static void make_edit_menu(void);
static void make_motif_menu(void);
static void make_align_menu(void);
static void make_tree_menu(void);
static void make_color_menu(void);

static void save_aln_window(ALN mult_aln,int prf_no,char *title,char *prompt,void save_proc(ButtoN but));
static void save_ps_window(int prf_no,char *prompt,void save_proc(ButtoN but));
static void read_file_window(char *title,char *prompt,char *filename,void read_proc(ButtoN but));
static void do_align_window(WindoW *ralignw,TexT *rtreetext,Boolean treestatus,char *title,void align_proc(ButtoN but),ALN mult_aln);
static void do_palign_window(WindoW *ralignw,TexT *rtree1text,TexT *rtree2text,Boolean treestatus,char *title,void align_proc(ButtoN but),ALN mult_aln);
static Boolean open_aln_files(ALN mult_aln);
static void write_file(int fseq, int lseq, int fres, int lres,ALN mult_aln);

static OPT opt;

char revision_level[10];

int    mheader = 2; /* maximum header lines */
int    mfooter = 1; /* maximum footer lines */
int max_mlines = 20;      /*   multiple align display length */
int min_mlines = 10;      /*   multiple align display length */
int max_plines = 8;     /*   profile align display length */
int min_plines1 = 5;     /*   profile align display length */
int min_plines2 = 4;     /*   profile align display length */

Boolean xmenus_active = FALSE;	/* tells xutils that xmenus have started */
Boolean aln_mode = MULTIPLEM;
Boolean window_displayed = FALSE;

int    save_format = CLUSTAL;
Boolean fixed_prf_scroll = FALSE;
int loffset,boffset,toffset;
int roffset;
int poffset;

int score_cutoff=5;    /* cutoff for residue exceptions */
int score_hwin=5;    /* half window for summing alignment column scores */
int score_scale=5;
int segment_dnascale=5;
int length_cutoff=1;    /* length cutoff for segment exceptions */
Boolean residue_exceptions=FALSE;
Boolean segment_exceptions=FALSE;
int score_matnum=4;
char score_mtrxname[FILENAMELEN];
int segment_matnum=3;
char segment_mtrxname[FILENAMELEN];
int score_dnamatnum=1;
char score_dnamtrxname[FILENAMELEN];
int segment_dnamatnum=1;
char segment_dnamtrxname[FILENAMELEN];

Boolean output_ss;
Boolean output_gp;


extern Boolean use_tree_file;

MatMenu dnamatrix_menu = {3,
                "IUB","iub",
                "CLUSTALW(1.6)","clustalw",
                "User defined",""
                };

MatMenu matrix_menu = {6,
                "BLOSUM series","blosum",
                "PAM series","pam",
                "Gonnet series","gonnet",
		"Bin series","bin",
                "Identity matrix","id",
                "User defined",""
                };

MatMenu pw_matrix_menu = {6,
                "BLOSUM 30","blosum",
                "PAM 350","pam",
                "Gonnet 250","gonnet",
		"Bin 30","bin",
                "Identity matrix","id",
                "User defined",""
                };


Boolean  save_parameters=FALSE;


PrompT   message;           /* used in temporary message window */

static Char filename[FILENAMELEN]; /* used in temporary file selection window */

Boolean mess_output=TRUE;
Boolean save_log=FALSE;
FILE *save_log_fd=NULL;
static char save_log_filename[FILENAMELEN];
static IteM save_item1,save_item2;

spanel  seq_panel;        /* data for multiple alignment area */
spanel  prf_panel[2];       /* data for profile alignment areas */
spanel  active_panel;       /* 'in-use' panel -scrolling,clicking etc. */
int firstres, lastres;	/* range of alignment for saving as ... */
 
/* data for Search function */

char find_string[MAXFINDSTR]="";
aln_pos find_pos;

/* arrays for storing clustalw data for cut-and-paste sequences */
static sint     *saveseqlen_array=NULL;
static char     **saveseq_array=NULL;
static char     **savenames=NULL, **savetitles=NULL;
sint     ncutseqs=0;

FonT datafont,helpfont;
WindoW mainw=NULL;
WindoW messagew=NULL;
WindoW readfilew=NULL;
WindoW savealnw=NULL;
WindoW savescoresw=NULL;
WindoW savepsw=NULL;
WindoW findw=NULL;
WindoW calignw=NULL;
WindoW ralignw=NULL;
WindoW rralignw=NULL;
WindoW talignw=NULL;
WindoW palignw=NULL;
WindoW salignw=NULL;
WindoW scorew=NULL;
WindoW exceptionw=NULL;
TexT savealntext;
TexT savescorestext;
TexT savepstext;
TexT findtext;
TexT pspartext;
TexT ctreetext;
TexT rtreetext;
TexT rrtreetext;
TexT ttreetext;
TexT ptree1text,ptree2text;
TexT streetext;
TexT readfiletext;
WindoW savetreew=NULL;
TexT savetreetext;
WindoW makeballastmotifw=NULL;
TexT makeballastmotiftext;
WindoW drawtreew=NULL;
TexT drawnjtreetext;
TexT drawphtreetext;
TexT drawdsttreetext;
TexT drawbionjtreetext;
TexT drawnxstreetext;
WindoW boottreew=NULL;
TexT bootnjtreetext;
TexT bootphtreetext;
TexT bootnxstreetext;
TexT blocklentext;
PrompT mattext,motiftext,pwmattext,dnamattext,pwdnamattext,scoremattext,segmentmattext;
PrompT scorednamattext,segmentdnamattext;
GrouP seg_matrix_list,score_matrix_list;
GrouP seg_dnamatrix_list,score_dnamatrix_list;
GrouP matrix_list,pw_matrix_list,dnamatrix_list,pw_dnamatrix_list;

TexT cl_outtext,pir_outtext,msf_outtext,phylip_outtext,gde_outtext,nexus_outtext;
GrouP slow_para,fast_para;
GrouP  seq_display,prf1_display,prf2_display;

MenU   filem,alignm,editm,motifm,treem,colorm;
menu_item file_item,align_item,motif_item,edit_item,tree_item,color_item;
MenU   scorem,helpmenu;
menu_item score_item,help_item;
IteM bw_item,defcol_item,usercol_item;
IteM new_gaps_item,all_gaps_item;
WindoW helpw[MAXHELPW];
int numhelp=0;

PopuP modetext,flisttext;
ButtoN pscrolltext;
PopuP show_seg_toggle;
PrompT residue_cutofftext;
PrompT length_cutofftext;
PrompT scorescaletext;
PrompT segmentdnascaletext;

#define MAXFONTS 6
int nfonts=5;
int av_font[MAXFONTS]={8,10,12,14,18};
int font_size=1;

char def_protpar_file[]="colprot.par";
char def_dnapar_file[]="coldna.par";
char *explicit_par_file = NULL;
char *par_file = NULL;
int    inverted = TRUE;
int usebw=FALSE,usedefcolors=TRUE,useusercolors=FALSE;

char ps_par_file[FILENAMELEN]="colprint.par";
int pagesize=A4;
int orientation=LANDSCAPE;
Boolean ps_header=TRUE;
Boolean ps_ruler=TRUE;
Boolean resize=TRUE;
int first_printres=0,last_printres=0,blocklen;
Boolean ps_curve=TRUE;
Boolean ps_resno=TRUE;
PoinT display_pos;
int namewidth,seqwidth; /* fixed widths of sequence display areas */

Boolean         realign_endgappenalties=TRUE;
Boolean         align_endgappenalties=FALSE;

char helptext[MAXHELPLENGTH];

ALN mult_aln;

/* main subroutine called from clustalx.c, initialises windows and enters a
   forever loop monitoring user input */

void x_menu(OPT aln_opt,ALN mult_aln_data,char *rev)
{
	int i,n;
	char font[30];
	char tstr[30];
	int height;
	PrompT   fsize;
	RecT wr,r,r1;
 

/*  make the pulldown menu bar  */

#ifdef WIN_MAC
	MenU   m;

        m=AppleMenu (NULL);
        DeskAccGroup (m);
	make_menu_headers(NULL);
#endif
#ifdef WIN_MSWIN
	ProcessUpdatesFirst(FALSE);
#endif

	xmenus_active = TRUE;
	opt=aln_opt;
	mult_aln=mult_aln_data;
	strcpy(revision_level,rev);

	sprintf(tstr,"LeClustal%s",revision_level);
/*#ifdef WIN_MSWIN
	mainw = FixedWindow (-50,-33,-10,-10,tstr,QuitWinW);
#else*/
	mainw = DocumentWindow (-50,-33,-10,-10,tstr,QuitWinW,ResizeWindowProc);
/*#endif*/	SetGroupSpacing(mainw,0,10);
	SetGroupSpacing(mainw,0,10);
 

#ifndef WIN_MAC
	make_menu_headers(mainw);
#endif
/* decide if we're starting in profile or sequence mode */
	if (mult_aln.prf1.nseqs>0) aln_mode=PROFILEM;
	else aln_mode=MULTIPLEM;

	make_file_menu();
	make_edit_menu();
	make_align_menu();
	make_motif_menu();
	make_tree_menu();
	make_color_menu();
	make_score_menu();
	make_help_menu();

/*  add a button to switch between multiple and profile alignment modes */

	modetext=PopupList(mainw,TRUE,set_aln_mode);
	PopupItem(modetext,"Multiple Alignment Mode");
	PopupItem(modetext,"Profile Alignment Mode");
	if(aln_mode==MULTIPLEM)
		SetValue(modetext,1);
	else
		SetValue(modetext,2);

	sprintf(font, "%s,%d,%c", "courier", av_font[font_size], 'm');
        datafont=ParseFont(font);

	sprintf(font, "%s,%d,%c", "courier", 10, 'm');
        helpfont=ParseFont(font);

	Advance(mainw);
	shift(mainw,20,0);

/*  add a button to select font size */
	fsize=StaticPrompt(mainw,"Font Size:",0,dialogTextHeight,systemFont,'r');
	Advance(mainw);
	flisttext=PopupList(mainw,TRUE,set_font_size);
	for(i=0;i<nfonts;i++)
	{
		sprintf(tstr,"%d",av_font[i]);
		PopupItem(flisttext,tstr);
	}
	SetValue(flisttext,font_size+1);

	Advance(mainw);
	shift(mainw,20,0);

/*  add a button to switch profile scrolling modes */
        pscrolltext=CheckBox(mainw,"Lock Scroll",set_pscroll_mode);
	if(fixed_prf_scroll) SetStatus(pscrolltext,TRUE);
	Break(mainw);

/*  initialise the multiple alignment display area */

	SelectFont(datafont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
 
	GetNextPosition(mainw,&display_pos);

/* calculate initial pixel width and height of displays */
	namewidth=(DNAMES+DNUMBER+1)*stdCharWidth;
	seqwidth=(DCOLS+2*MARGIN)*stdCharWidth+2;
	n=screenRect.right-screenRect.left;
	if(seqwidth+namewidth>n) seqwidth=n-namewidth;

	height=(max_mlines+mfooter+MARGIN)*stdLineHeight+2+SCOREHEIGHT;
	n=screenRect.bottom-screenRect.top;
	if(height>n) height=n;

	seq_display=make_scroll_area(mainw,0,namewidth,seqwidth,height,0,mult_aln.nseqs,&seq_panel);
	position_scrollbars(seq_panel);

/*  initialise the profile alignment display area */
 
	SetNextPosition(mainw,display_pos);
	height=(max_plines+MARGIN)*stdLineHeight+2+SCOREHEIGHT;
	if(height>n) height=n;
	prf1_display=make_scroll_area(mainw,1,namewidth,seqwidth,height,0,mult_aln.prf1.nseqs,&prf_panel[0]);
	position_scrollbars(prf_panel[0]);

	prf2_display=make_scroll_area(mainw,2,namewidth,seqwidth,height,mult_aln.prf1.nseqs,mult_aln.nseqs-mult_aln.prf1.nseqs,&prf_panel[1]);
	position_scrollbars(prf_panel[1]);

/*  add the message line */
	Break(mainw);
	Advance(mainw);
	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	message = StaticPrompt(mainw, "",500, 0,systemFont,'l');

/* save some pixel sizes for future resizing events */
	if(aln_mode==PROFILEM)
	{
		Hide(seq_display);
		Show(prf1_display);
		Show(prf2_display);
		Show(pscrolltext);
		active_panel=prf_panel[0];
		Select(prf1_display);
		load_aln(prf_panel[0],1,0,mult_aln.prf1.nseqs,TRUE);
		load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,TRUE);

		Show(mainw);
		ObjectRect(mainw,&wr);
		ObjectRect(prf_panel[0].names,&r);
		ObjectRect(prf_panel[1].names,&r1);
		boffset=wr.bottom-wr.top-r1.bottom;
		loffset=r.left;
		toffset=r.top;
		ObjectRect(prf_panel[0].seqs,&r);
		roffset=wr.right-wr.left-r.right;
	}
	else
	{
		Hide(prf1_display);
		Hide(prf2_display);
		Hide(pscrolltext);
		Show(seq_display);
		active_panel=seq_panel;

		Select(seq_display);
		load_aln(seq_panel,0,0,mult_aln.nseqs,TRUE);

		Show(mainw);
		ObjectRect(mainw,&wr);
		ObjectRect(seq_panel.names,&r);
		boffset=wr.bottom-wr.top-r.bottom;
		loffset=r.left;
		toffset=r.top;
		ObjectRect(seq_panel.seqs,&r);
		roffset=wr.right-wr.left-r.right;
	}
	ObjectRect(prf_panel[0].names,&r);
	ObjectRect(prf_panel[1].names,&r1);
	poffset=r1.top-r.bottom;

/* initialise some variables before we display the window */
        if(orientation==LANDSCAPE)
        {
                if(pagesize==A4) blocklen=150;
                else blocklen=250;
        }
        else
        {
                if(pagesize==A4) blocklen=80;
                else blocklen=150;
        }

/* ok - Go! */
	window_displayed=TRUE;
	ProcessEvents();

}


static void RemoveWin(WindoW w)
{
	Remove(w);
}


static void QuitWinW(WindoW w)
{
	if(aln_mode == MULTIPLEM)
	{
		if(seq_panel.modified)
			if (Message(MSG_YN,"Alignment has not been saved.\n"
			"Quit program anyway?")==ANS_NO) return;
	}
	else if(aln_mode == PROFILEM)
	{
		if(prf_panel[0].modified)
			if (Message(MSG_YN,"Profile 1 has not been saved.\n"
			"Quit program anyway?")==ANS_NO) return;
		if(prf_panel[1].modified)
			if (Message(MSG_YN,"Profile 2 has not been saved.\n"
			"Quit program anyway?")==ANS_NO) return;
	}
	QuitProgram ();
}

static void SearchStrWin (IteM item)
{
	int i;
	Boolean sel=FALSE;
	GrouP findgr;
	ButtoN find_can,find_ok;
	panel_data data;

	GetPanelExtra(active_panel.names,&data);
	if (data.nseqs==0)
	{
		Message(MSG_OK,"No file loaded.");
		return;
	}
	for (i=0;i<data.nseqs;i++)
		if(data.selected[i]==TRUE)
		{
			sel=TRUE;
			break;
		}
	if(sel==FALSE)
	{
		Message(MSG_OK,"Select sequences by clicking on the names.");
		return;
	}

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	findw=FixedWindow(-50, -33, -10, -10, "SEARCH IN SELECTED SEQUENCES",RemoveWin);
        stdLineHeight=18;
        SelectFont(programFont);
	findtext=DialogText(findw, "", 35, NULL);
	Break(findw);
	find_ok=PushButton(findw, "SEARCH FROM START", SearchStr);
	Break(findw);
	find_ok=PushButton(findw, "SEARCH AGAIN", SearchStringAgain);
	Break(findw);
	find_can=PushButton(findw, "CLOSE", CancelWin);

	Show(findw);
}

static void SavePSSeqWin (IteM item)
{
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);
	if (mult_aln.nseqs<=0)
	{
		error("No file loaded");
		return;
	}
	save_ps_window(0,"WRITE SEQUENCES TO:",SavePSSeqFile);
}

static void SavePSPrf1Win (IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);
	if (mult_aln.prf1.nseqs<=0)
	{
		error("No file loaded");
		return;
	}
	save_ps_window(1,"WRITE PROFILE 1 TO:",SavePSPrf1File);
}

static void SavePSPrf2Win (IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[1].seqs,&data);
	if (mult_aln.prf1.nseqs==mult_aln.nseqs)
	{
		error("No file loaded");
		return;
	}
	save_ps_window(2,"WRITE PROFILE 2 TO:",SavePSPrf2File);
}

static void save_ps_window(int prf_no,char *prompt,void save_proc(ButtoN but))
{
	GrouP savegr;
	ButtoN save_can,save_ok;
	PopuP ps,or;
	char path[FILENAMELEN];
	char str[FILENAMELEN];
	panel_data data;

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	savepsw=FixedWindow(-50, -33, -10, -10, "WRITE POSTSCRIPT FILE",RemoveWin);
	make_prompt(savepsw, prompt);
        stdLineHeight=18;
        SelectFont(programFont);
	savepstext=DialogText(savepsw, "", 35, NULL);
	Break(savepsw);
	make_prompt(savepsw, "PS Colors File :");
	pspartext=DialogText(savepsw, ps_par_file, 35, NULL);
	Break(savepsw);
       	make_prompt(savepsw, "Page Size");
	Advance(savepsw);
	ps=PopupList(savepsw,TRUE,set_pagesize);
	PopupItem(ps,"A4");
	PopupItem(ps,"A3");
	if (pagesize == A4)
		SetValue(ps,1);
	else if (pagesize == A3)
		SetValue(ps,2);
	Break(savepsw);
       	make_prompt(savepsw, "Orientation");
	Advance(savepsw);
	or=PopupList(savepsw,TRUE,set_orientation);
	PopupItem(or,"LANDSCAPE");
	PopupItem(or,"PORTRAIT");
	if (orientation == LANDSCAPE)
		SetValue(or,1);
	else if (orientation == PORTRAIT)
		SetValue(or,2);
	Break(savepsw);
	make_toggle(savepsw,"Print Header :","YES","NO",&ps_header,set_header);
	Advance(savepsw);
	make_toggle(savepsw,"Print Quality Curve :","YES","NO",&ps_curve,set_curve);
	Break(savepsw);
	make_toggle(savepsw,"Print Ruler :","YES","NO",&ps_ruler,set_ruler);
	Advance(savepsw);
	make_toggle(savepsw,"Print Residue Numbers :","YES","NO",&ps_resno,set_resno);
	Break(savepsw);
	make_toggle(savepsw,"Resize to fit page:","YES","NO",&resize,set_resize);
	Break(savepsw);
	first_printres=1;
	if (prf_no==0)
		GetPanelExtra(seq_panel.seqs,&data);
	else if (prf_no==1)
		GetPanelExtra(prf_panel[0].seqs,&data);
	else
		GetPanelExtra(prf_panel[1].seqs,&data);
	last_printres=data.ncols;
        make_prompt(savepsw, "Print from position :");
	Advance(savepsw);
	sprintf(str,"%5d",first_printres);
        DialogText(savepsw, str, 5,set_fpres);
	Advance(savepsw);
        make_prompt(savepsw, "to :");
	Advance(savepsw);
	sprintf(str,"%5d",last_printres);
        DialogText(savepsw, str, 5,set_lpres);
	Break(savepsw);
        make_prompt(savepsw, "Use block length :");
	Advance(savepsw);
	sprintf(str,"%5d",blocklen);
        blocklentext=DialogText(savepsw, str, 5,set_blocklen);
	Break(savepsw);
	savegr=HiddenGroup(savepsw, 2, 0, NULL);
	shift(savegr, 60, 20);
	save_ok=PushButton(savegr, "  OK  ", save_proc);
	shift(savegr, 20,0);
	save_can=PushButton(savegr, "CLOSE", CancelWin);

	if(prf_no==0)
		get_path(mult_aln.filename,path);
	else if(prf_no==1)
		get_path(mult_aln.prf1.filename,path);
	else if(prf_no==2)
		get_path(mult_aln.prf2.filename,path);
	strcat(path,"ps");
	SetTitle(savepstext, path);
	Show(savepsw);
}

static void SaveScoresWin (IteM item)
{
	int i;
	Boolean sel=FALSE;
	GrouP scoregr;
	ButtoN score_can,score_ok;
	char path[FILENAMELEN];
	panel_data data;


	GetPanelExtra(active_panel.names,&data);
	if (data.nseqs<=0)
	{
		error("No file loaded");
		return;
	}

	for (i=0;i<data.nseqs;i++)
		if(data.selected[i]==TRUE)
		{
			sel=TRUE;
			break;
		}
	if(sel==FALSE)
	{
		Message(MSG_OK,"Select sequences to be written by clicking on the names.");
		return;
	}

	GetPanelExtra(active_panel.seqs,&data);
        get_path(mult_aln.filename,path);
	strcat(path,"qscores");

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	savescoresw=FixedWindow(-50, -33, -10, -10, "SAVE QUALITY SCORES",RemoveWin);
        stdLineHeight=18;
        SelectFont(programFont);
	make_prompt(savescoresw, "SAVE QUALITY SCORES TO:");
        stdLineHeight=18;
        SelectFont(programFont);
	Break(savescoresw);
	savescorestext=DialogText(savescoresw, "", 35, NULL);
	Break(savescoresw);
	scoregr=HiddenGroup(savescoresw, 2, 0, NULL);
	shift(scoregr, 60, 20);
	score_ok=PushButton(scoregr, "  OK  ", SaveScores);
	shift(scoregr, 20,0);
	score_can=PushButton(scoregr, "CANCEL", CancelWin);

	SetTitle(savescorestext, path);
	Show(savescoresw);

	Advance(savescoresw);
	Show(savescoresw);
}

static void SaveScores(ButtoN but)
{
	char c,val;
	int i,j;
	int length=0;
	FILE *outfile;
	panel_data ndata,sdata;
	Boolean gap;

	GetPanelExtra(active_panel.names,&ndata);
	GetPanelExtra(active_panel.seqs,&sdata);

	GetTitle(savescorestext, filename, FILENAMELEN);
	stripspace(filename);

	outfile=open_explicit_file(filename); 

/* get the maximum length of the selected sequences */
        for (i=1;i<=mult_aln.nseqs;i++)
           if (ndata.selected[i-1]==TRUE && length < mult_aln.seqs[i].len) length = mult_aln.seqs[i].len;

	for(j=1;j<=length;j++)
	{
/* first check for a column of gaps */
		gap=TRUE;
        	for (i=1;i<=mult_aln.nseqs;i++)
           		if (ndata.selected[i-1]==TRUE)
			{
                                val = mult_aln.seqs[i].data[j];
                                if(j<=mult_aln.seqs[i].len && isalpha(val))
				{
					gap=FALSE;
					break;
				}
			}
		if(gap==FALSE)
		{
        		for (i=1;i<=mult_aln.nseqs;i++)
			{
           			if (ndata.selected[i-1]==TRUE)
				{
                                	val = mult_aln.seqs[i].data[j];
                                	if(j>mult_aln.seqs[i].len || !isalpha(val))
                                        	c = '-';
                                	else {
                                        	c = val;
                                	}
	 
					fprintf(outfile,"%c ",c);
				}
			}
			fprintf(outfile,"\t%3d\n",sdata.colscore[j-1]);
		}

	}
	fclose(outfile);

        if (Visible(savescoresw))
        {
                Remove(savescoresw);
                savescoresw=NULL;
        }



	info("File %s saved",filename);
}

static void SaveSeqFileWin (IteM item)
{
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);
	if (mult_aln.nseqs<=0)
	{
		error("No file loaded");
		return;
	}
	save_aln_window(mult_aln,0,"SAVE SEQUENCES","SAVE SEQUENCES AS:",SaveSeqFile);
}
static void SavePrf1FileWin (IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);
	if (mult_aln.prf1.nseqs<=0)
	{
		error("No file loaded");
		return;
	}
	save_aln_window(mult_aln,1,"SAVE PROFILE","SAVE PROFILE 1 AS:",SavePrf1File);
}
static void SavePrf2FileWin (IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[1].seqs,&data);
	if (mult_aln.prf1.nseqs==mult_aln.nseqs)
	{
		error("No file loaded");
		return;
	}
	save_aln_window(mult_aln,2,"SAVE PROFILE","SAVE PROFILE 2 AS:",SavePrf2File);
}

static void save_aln_window(ALN mult_aln,int prf_no,char *title,char *prompt,void save_proc(ButtoN but))
{
	GrouP savegr;
	ButtoN save_ok, save_can;
	GrouP maing;
        GrouP format_list;
	ButtoN formatb[6];
	PopuP case_toggle,snos_toggle;
	char path[FILENAMELEN];
	char str[FILENAMELEN];

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();

	savealnw=FixedWindow(-50, -33, -10, -10, title,RemoveWin);

        format_list=NormalGroup(savealnw,3,0,"Format",systemFont,set_format);
        formatb[0]=RadioButton(format_list,"CLUSTAL");
        formatb[1]=RadioButton(format_list,"NBRF/PIR");
        formatb[2]=RadioButton(format_list,"GCG/MSF");
        formatb[3]=RadioButton(format_list,"PHYLIP");
        formatb[4]=RadioButton(format_list,"GDE");
        formatb[5]=RadioButton(format_list,"NEXUS");
	if(prf_no==0)
        	get_path(mult_aln.filename,path);
	else if(prf_no==1)
        	get_path(mult_aln.prf1.filename,path);
	else if(prf_no==2)
        	get_path(mult_aln.prf2.filename,path);
	if (save_format==CLUSTAL)
	{
        	SetValue(format_list,1);
		strcat(path,"aln");
	}
	else if (save_format==PIR)
	{
        	SetValue(format_list,2);
		strcat(path,"pir");
	}
	else if (save_format==MSF)
	{
        	SetValue(format_list,3);
		strcat(path,"msf");
	}
	else if (save_format==PHYLIP)
	{
        	SetValue(format_list,4);
		strcat(path,"phy");
	}
	else if (save_format==GDE)
	{
        	SetValue(format_list,5);
		strcat(path,"gde");
	}
	else if (save_format==NEXUS)
	{
        	SetValue(format_list,6);
		strcat(path,"nxs");
	}
 
 
	maing=HiddenGroup(savealnw,0,0,NULL);
	SetGroupSpacing(maing,0,10);

	case_toggle=make_toggle(maing,"GDE output case :","Lower","Upper",&(*opt.alnout_opt).lowercase,set_case);
	Break(maing);
	snos_toggle=make_toggle(maing,"CLUSTALW sequence numbers :","ON","OFF",&(*opt.alnout_opt).seq_numbers,set_snos);

	Break(maing);
        make_prompt(maing, "Save from residue :");
	Advance(maing);
	sprintf(str,"%5d",firstres);
        DialogText(maing, str, 5,set_fres);
	Advance(maing);
        make_prompt(maing, "to :");
	Advance(maing);
	sprintf(str,"%5d",lastres);
        DialogText(maing, str, 5,set_lres);

	Break(maing);
	shift(savealnw, 0, 20);
	make_prompt(savealnw, prompt);
        stdLineHeight=18;
        SelectFont(programFont);
	Break(savealnw);
	savealntext=DialogText(savealnw, "", 35, NULL);
	Break(savealnw);
	savegr=HiddenGroup(savealnw, 2, 0, NULL);
	shift(savegr, 60, 20);
	save_ok=PushButton(savegr, "  OK  ", save_proc);
	shift(savegr, 20,0);
	save_can=PushButton(savegr, "CANCEL", CancelWin);

	SetTitle(savealntext, path);
	Show(savealnw);
}

static void read_file_window(char *title,char *prompt,char *fname,void read_proc(ButtoN but))
{
	GrouP readgr;
	ButtoN read_ok, read_can;
	GrouP maing;

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	readfilew=FixedWindow(-50, -33, -10, -10, title,RemoveWin);

	maing=HiddenGroup(readfilew,2,0,NULL);
	SetGroupSpacing(maing,0,10);

	shift(readfilew, 0, 20);
	make_prompt(readfilew, prompt);
        stdLineHeight=18;
        SelectFont(programFont);
	Break(readfilew);
	readfiletext=DialogText(readfilew, "", 35, NULL);
	if (fname != NULL) SetTitle(readfiletext, fname);
	Break(readfilew);
	readgr=HiddenGroup(readfilew, 2, 0, NULL);
	shift(readgr, 60, 20);
	read_ok=PushButton(readgr, "  OK  ", read_proc);
	shift(readgr, 20,0);
	read_can=PushButton(readgr, "CANCEL", CancelWin);

	Show(readfilew);
}

static void CancelWin (ButtoN but)
{
	Remove(ParentWindow(but));
}

static void SearchStr(ButtoN but)
{

/* reset the current position */

	find_pos.seq=0;
	find_pos.res=(-1);

/* find the next occurrence of the string */
	SearchStringAgain(but);


}

static void SearchStringAgain(ButtoN but)
{
	int i,ix,length;
	int seq,res,start_res;
	Boolean in_string,found;
	panel_data ndata,sdata;

	GetTitle(findtext, filename, FILENAMELEN);
	stripspace(filename);

	strncpy(find_string,filename,MAXFINDSTR);
	length=strlen(find_string);
	if(length==0) return;
	for(i=0;i<length;i++)
		find_string[i]=toupper(find_string[i]);

        GetPanelExtra(active_panel.names,&ndata);
        GetPanelExtra(active_panel.seqs,&sdata);

	in_string=FALSE;
	found=FALSE;
	start_res=0;
	ix=0;
	seq=find_pos.seq+ndata.firstseq;
	res=find_pos.res+1;
        while (seq<ndata.firstseq+ndata.nseqs)
	{
                if(ndata.selected[seq]==TRUE)
                {
        		while (res<sdata.ncols)
			{
				if(mult_aln.seqs[seq].data[res]==find_string[ix])
				{
					if(in_string==FALSE) 
						start_res=res;
					ix++;
					in_string=TRUE;
				}
				else if(in_string==TRUE)
				{
					res=start_res;
					ix=0;
					in_string=FALSE;
				}
				if(ix==length)
				{
					find_pos.seq=seq;
					find_pos.res=start_res;
					found=TRUE;
					break;
				}
				res++;
				while(res<sdata.ncols && mult_aln.seqs[seq].data[res]=='-')
					res++;
			}
                }
		if(found) break;
		seq++;
		res=0;
	}


	if(find_pos.seq==-1)
		info("String %s not found",find_string);
	else
	{
		info("String %s in sequence %s, column %d",find_string,mult_aln.seqs[find_pos.seq].name,find_pos.res+1);
	}
}

static void SavePSSeqFile(ButtoN but)
{
	char *ps_file;

	GetTitle(savepstext, filename, FILENAMELEN);
	stripspace(filename);

        ps_file=(char *)ckalloc(FILENAMELEN*sizeof(char));
	strcpy(ps_file,filename); 

	GetTitle(pspartext, filename, FILENAMELEN);
	stripspace(filename);

	strcpy(ps_par_file,filename); 

	write_ps_file(seq_panel,mult_aln,ps_file,ps_par_file,pagesize,orientation,
		ps_header,ps_ruler,ps_resno,
		resize,first_printres,last_printres,revision_level,
		inverted,blocklen,ps_curve,
		residue_exceptions,segment_exceptions);

	info("Postscript file %s written",ps_file);
	ckfree(ps_file);

}

static void SavePSPrf1File(ButtoN but)
{
	char *ps_file;

	GetTitle(savepstext, filename, FILENAMELEN);
	stripspace(filename);

        ps_file=(char *)ckalloc(FILENAMELEN*sizeof(char));
	strcpy(ps_file,filename); 

	GetTitle(pspartext, filename, FILENAMELEN);
	stripspace(filename);

	strcpy(ps_par_file,filename); 

	write_ps_file(prf_panel[0],mult_aln,ps_file,ps_par_file,pagesize,orientation,
		ps_header,ps_ruler,ps_resno,
		resize,first_printres,last_printres,revision_level,
		inverted,blocklen,ps_curve,
		residue_exceptions,segment_exceptions);

	info("Postscript file %s written",ps_file);

}

static void SavePSPrf2File(ButtoN but)
{
	char *ps_file;

	GetTitle(savepstext, filename, FILENAMELEN);
	stripspace(filename);

        ps_file=(char *)ckalloc(FILENAMELEN*sizeof(char));
	strcpy(ps_file,filename); 

	GetTitle(pspartext, filename, FILENAMELEN);
	stripspace(filename);

	strcpy(ps_par_file,filename); 

	write_ps_file(prf_panel[1],mult_aln,ps_file,ps_par_file,pagesize,orientation,
		ps_header,ps_ruler,ps_resno,
		resize,first_printres,last_printres,revision_level,
		inverted,blocklen,ps_curve,
		residue_exceptions,segment_exceptions);

	info("Postscript file %s written",ps_file);

}

static void SaveSeqFile(ButtoN but)
{
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);
	write_file(0,mult_aln.nseqs,firstres,lastres,mult_aln);
	seq_panel.modified=FALSE;
	info("File %s saved",filename);
}

static void SavePrf1File(ButtoN but)
{
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);
	write_file(0,mult_aln.prf1.nseqs,firstres,lastres,mult_aln);
	prf_panel[0].modified=FALSE;
	info("File %s saved",filename);
}

static void SavePrf2File(ButtoN but)
{
	panel_data data;

	GetPanelExtra(prf_panel[1].seqs,&data);
	write_file(mult_aln.prf1.nseqs,mult_aln.nseqs,firstres,lastres,mult_aln);
	prf_panel[1].modified=FALSE;
	info("File %s saved",filename);
}

/* this is equivalent to open_alignment_output(), but uses the window
interface to input file names */

static Boolean open_aln_files(ALN mult_aln)
{
	char path[FILENAMELEN];

	if(!(*opt.alnout_opt).output_clustal && !(*opt.alnout_opt).output_nbrf && !(*opt.alnout_opt).output_gcg &&
		 !(*opt.alnout_opt).output_phylip && !(*opt.alnout_opt).output_gde && !(*opt.alnout_opt).output_nexus ) {
		error("You must select an alignment output format");
		return FALSE;
	}

	if((*opt.alnout_opt).output_clustal) {
		GetTitle(cl_outtext,filename,FILENAMELEN);
		stripspace(filename);
		if(((*opt.alnout_opt).clustal_outfile=open_explicit_file(filename))==NULL) return FALSE;
		strcpy((*opt.alnout_opt).clustal_outname,filename);
	}
	if((*opt.alnout_opt).output_nbrf) {
		GetTitle(pir_outtext,filename,FILENAMELEN);
		stripspace(filename);
		if(((*opt.alnout_opt).nbrf_outfile=open_explicit_file(filename))==NULL) return FALSE;
		strcpy((*opt.alnout_opt).nbrf_outname,filename);
	}
	if((*opt.alnout_opt).output_gcg) {
		GetTitle(msf_outtext,filename,FILENAMELEN);
		stripspace(filename);
		if(((*opt.alnout_opt).gcg_outfile=open_explicit_file(filename))==NULL) return FALSE;
		strcpy((*opt.alnout_opt).gcg_outname,filename);
	}
	if((*opt.alnout_opt).output_phylip) {
		GetTitle(phylip_outtext,filename,FILENAMELEN);
		stripspace(filename);
		if(((*opt.alnout_opt).phylip_outfile=open_explicit_file(filename))==NULL) return FALSE;
		strcpy((*opt.alnout_opt).phylip_outname,filename);
	}
	if((*opt.alnout_opt).output_gde) {
		GetTitle(gde_outtext,filename,FILENAMELEN);
		stripspace(filename);
		if(((*opt.alnout_opt).gde_outfile=open_explicit_file(filename))==NULL) return FALSE;
		strcpy((*opt.alnout_opt).gde_outname,filename);
	}
	if((*opt.alnout_opt).output_nexus) {
		GetTitle(nexus_outtext,filename,FILENAMELEN);
		stripspace(filename);
		if(((*opt.alnout_opt).nexus_outfile=open_explicit_file(filename))==NULL) return FALSE;
		strcpy((*opt.alnout_opt).nexus_outname,filename);
	}

	if(save_log)
	{
		get_path(mult_aln.filename,path);
		strcpy(save_log_filename,path);
		strcat(save_log_filename,"log");
		if ((save_log_fd=fopen(save_log_filename,"a"))==NULL)
			error("Cannot open log file %s",save_log_filename);
	}

	return TRUE;
}

/* writes the sequences to a single alignment file specified by the user */

static void write_file(int fseq, int lseq, int fres, int lres,ALN mult_aln)
{
	int i,length=0;
	FILE *outfile;

	GetTitle(savealntext, filename, FILENAMELEN);
	stripspace(filename);

	outfile=open_explicit_file(filename); 

	for (i=fseq;i<=lseq;i++)
	   if (length < mult_aln.seqs[i].len) length = mult_aln.seqs[i].len;

	if(fres<0) fres=0;
	if(lres<1) lres=length;
	length=lres-fres+1;
 
	if(save_format==CLUSTAL) {
		clustal_out(outfile, fres, length, fseq, lseq,(*opt.alnout_opt).seq_numbers,mult_aln,*opt.ss_opt);
		fclose(outfile);
		info("CLUSTAL format file created  [%s]",filename);
	}
	else if(save_format==PIR)  {
		nbrf_out(outfile, fres, length, fseq, lseq,mult_aln);
		fclose(outfile);
		info("NBRF/PIR format file created  [%s]",filename);
	}
	else if(save_format==MSF)  {
		gcg_out(outfile, fres, length, fseq, lseq,mult_aln);
		fclose(outfile);
		info("GCG/MSF format file created  [%s]",filename);
	}
	else if(save_format==PHYLIP)  {
		phylip_out(outfile, fres, length, fseq, lseq,mult_aln);
		fclose(outfile);
		info("PHYLIP format file created  [%s]",filename);
	}
	else if(save_format==GDE)  {
		gde_out(outfile, fres, length, fseq, lseq,(*opt.alnout_opt).lowercase,mult_aln,*opt.ss_opt);
		fclose(outfile);
		info("GDE format file created  [%s]",filename);
	}
	else if(save_format==NEXUS)  {
		nexus_out(outfile, fres, length, fseq, lseq,mult_aln);
		fclose(outfile);
		info("NEXUS format file created  [%s]",filename);
	}


	if (Visible(savealnw))
	{
		Remove(savealnw);
		savealnw=NULL;
	}


}

static void MakeBallastMotifWin (IteM item)
{
	GrouP savegr;
	ButtoN save_ok, save_can;
	char path[FILENAMELEN];
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);

	if (mult_aln.nseqs<=0)
	{
		error("No file loaded");
		return;
	}
        if (mult_aln.nseqs < 2)
	{
                error("Alignment has only %d sequences",mult_aln.nseqs);
                return;
        }

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	makeballastmotifw=FixedWindow(-50, -33, -10, -10, "CREATE BALLAST MOTIF FILE",RemoveWin);
	shift(makeballastmotifw, 0, 20);
	make_prompt(makeballastmotifw, "SAVE MOTIFS AS :");
	Advance(makeballastmotifw);
	shift(makeballastmotifw, 0, -10);
	stdLineHeight=18;
	SelectFont(programFont);
	makeballastmotiftext=DialogText(makeballastmotifw, "", 35, NULL);
	SelectFont(systemFont);
	stdLineHeight=15;
	Break(makeballastmotifw);
	savegr=HiddenGroup(makeballastmotifw, 2, 0, NULL);
	shift(savegr, 140, 20);
	save_ok=PushButton(savegr, "  OK  ", MakeBallastMotifs);
	shift(savegr, 20, 0);
	save_can=PushButton(savegr, "CANCEL", CancelWin);

	get_path(mult_aln.filename,path);
	strcat(path,"ballast.motifs");
  
	SetTitle(makeballastmotiftext, path);
	Show(makeballastmotifw);
}

static void SaveTreeWin (IteM item)
{
	GrouP savegr;
	ButtoN save_ok, save_can;
	char path[FILENAMELEN];
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);

	if (mult_aln.nseqs<=0)
	{
		error("No file loaded");
		return;
	}
        if (mult_aln.nseqs < 2)
	{
                error("Alignment has only %d sequences",mult_aln.nseqs);
                return;
        }

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	savetreew=FixedWindow(-50, -33, -10, -10, "CREATE TREE",RemoveWin);
	shift(savetreew, 0, 20);
	make_prompt(savetreew, "SAVE TREE AS :");
	Advance(savetreew);
	shift(savetreew, 0, -10);
	stdLineHeight=18;
	SelectFont(programFont);
	savetreetext=DialogText(savetreew, "", 35, NULL);
	SelectFont(systemFont);
	stdLineHeight=15;
	Break(savetreew);
	savegr=HiddenGroup(savetreew, 2, 0, NULL);
	shift(savegr, 140, 20);
	save_ok=PushButton(savegr, "  OK  ", CreateAlignTree);
	shift(savegr, 20, 0);
	save_can=PushButton(savegr, "CANCEL", CancelWin);

	get_path(mult_aln.filename,path);
	strcat(path,"dnd");
  
	SetTitle(savetreetext, path);
	Show(savetreew);
}

static void DrawTreeWin (IteM item)
{
	GrouP drawgr;
	GrouP output_list;
	ButtoN draw_ok, draw_can;
	char path[FILENAMELEN];
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);

	if (mult_aln.nseqs<=0)
	{
		error("No file loaded");
		return;
	}
        if (mult_aln.nseqs < 2)
	{
                error("Alignment has only %d sequences",mult_aln.nseqs);
                return;
        }


	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	drawtreew=FixedWindow(-50, -33, -10, -10, "DRAW TREE",RemoveWin);
	output_list=HiddenGroup(drawtreew, 2, 0, NULL);
	if ((*opt.tree_opt).output_tree_clustal)
	{
		make_prompt(output_list, "SAVE CLUSTAL TREE AS :");
		drawnjtreetext=DialogText(output_list, "", 35, NULL);
		get_path(mult_aln.filename,path);
        	strcat(path,"nj");
		SetTitle(drawnjtreetext, path);
		Break(output_list);
	}
	if ((*opt.tree_opt).output_tree_phylip)
	{
		make_prompt(output_list, "SAVE PHYLIP TREE AS :");
		drawphtreetext=DialogText(output_list, "", 35, NULL);
		get_path(mult_aln.filename,path);
        	strcat(path,"ph");
		SetTitle(drawphtreetext, path);
		Break(output_list);
	}
	if ((*opt.tree_opt).output_tree_distances)
	{
		make_prompt(output_list, "SAVE DISTANCE MATRIX AS :");
		drawdsttreetext=DialogText(output_list, "", 35, NULL);
		get_path(mult_aln.filename,path);
        	strcat(path,"dst");
		SetTitle(drawdsttreetext, path);
		Break(output_list);
	}
	if ((*opt.tree_opt).output_tree_bionj)
	{
		make_prompt(output_list, "SAVE BIONJ TREE AS :");
		drawbionjtreetext=DialogText(output_list, "", 35, NULL);
		get_path(mult_aln.filename,path);
        	strcat(path,"bionj");
		SetTitle(drawbionjtreetext, path);
		Break(output_list);
	}
        if ((*opt.tree_opt).output_tree_nexus)
        {
                make_prompt(output_list, "SAVE NEXUS TREE AS :");
                drawnxstreetext=DialogText(output_list, "", 35, NULL);
		get_path(mult_aln.filename,path);
                strcat(path,"tre");
                SetTitle(drawnxstreetext, path);
		Break(output_list);
        }

	SelectFont(systemFont);
	stdLineHeight=15;
	Break(drawtreew);
	drawgr=HiddenGroup(drawtreew, 2, 0, NULL);
	shift(drawgr, 140, 20);
	draw_ok=PushButton(drawgr, "  OK  ", DrawTree);
	shift(drawgr, 20, 0);
	draw_can=PushButton(drawgr, "CANCEL", CancelWin);


	Show(drawtreew);
}

static void BootstrapTreeWin (IteM item)
{
	GrouP bootgr;
	GrouP output_list;
	ButtoN boot_ok, boot_can;
	TexT seed,ntrials;
	char path[FILENAMELEN];
	char str[FILENAMELEN];
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);

	if (mult_aln.nseqs<=0)
	{
		error("No file loaded");
		return;
	}
        if (mult_aln.nseqs < 2)
	{
                error("Alignment has only %d sequences",mult_aln.nseqs);
                return;
        }


	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	boottreew=FixedWindow(-50, -33, -10, -10, "BOOTSTRAP TREE",RemoveWin);
        make_prompt(boottreew, "Random number generator seed [1-1000] :");
	Advance(boottreew);
	sprintf(str,"%4d",(*opt.bstree_opt).boot_ran_seed);
        seed=DialogText(boottreew, str, 4,set_ran_seed);
	Break(boottreew);
        make_prompt(boottreew, "Number of bootstrap trials [1-10000] :");
	Advance(boottreew);
	sprintf(str,"%5d",(*opt.bstree_opt).boot_ntrials);
        ntrials=DialogText(boottreew, str, 5,set_ntrials);
	Break(boottreew);

	output_list=HiddenGroup(boottreew, 2, 0, NULL);
	if ((*opt.tree_opt).output_tree_clustal)
	{
		make_prompt(output_list, "SAVE CLUSTAL TREE AS :");
		bootnjtreetext=DialogText(output_list, "", 35, NULL);
		get_path(mult_aln.filename,path);
        	strcat(path,"njb");
		SetTitle(bootnjtreetext, path);
		Break(output_list);
	}
	if ((*opt.tree_opt).output_tree_phylip)
	{
		make_prompt(output_list, "SAVE PHYLIP TREE AS :");
		bootphtreetext=DialogText(output_list, "", 35, NULL);
		get_path(mult_aln.filename,path);
        	strcat(path,"phb");
		SetTitle(bootphtreetext, path);
		Break(output_list);
	}
        if ((*opt.tree_opt).output_tree_nexus)
        {
                make_prompt(output_list, "SAVE NEXUS TREE AS :");
                bootnxstreetext=DialogText(output_list, "", 35, NULL);
		get_path(mult_aln.filename,path);
                strcat(path,"treb");
                SetTitle(bootnxstreetext, path);
		Break(output_list);
        }

	SelectFont(systemFont);
	stdLineHeight=15;
	Break(boottreew);
	bootgr=HiddenGroup(boottreew, 2, 0, NULL);
	shift(bootgr, 140, 20);
	boot_ok=PushButton(bootgr, "  OK  ", BootstrapTree);
	shift(bootgr, 20, 0);
	boot_can=PushButton(bootgr, "CANCEL", CancelWin);


	Show(boottreew);
}

static void MakeBallastMotifs(ButtoN but)
{
	int i,length=0;
	FILE *outfile;
	char path[FILENAMELEN];
	char motif_name[FILENAMELEN];
	char command[FILENAMELEN+30];
	panel_data data;

	GetTitle(makeballastmotiftext, filename, FILENAMELEN);
	stripspace(filename);
	strcpy(motif_name,filename);

	info("Making Ballast motifs...");
	GetPanelExtra(seq_panel.seqs,&data);

	WatchCursor();
	if (Visible(makeballastmotifw))
	{
		Remove(makeballastmotifw);
		makeballastmotifw=NULL;
	}

/* write current sequence data to file 'seqtmp.tfa' in fasta format */

	outfile=open_explicit_file("seqtmp.tfa"); 

	for (i=0;i<mult_aln.nseqs;i++)
	   if (length < mult_aln.seqs[i].len) length = mult_aln.seqs[i].len;

	fasta_out(outfile, 0, length, 0, mult_aln.nseqs,mult_aln);
	fclose(outfile);

/* system command to create ballast motifs */
	strcpy(command,"/home/julie/bin/ballast_anchors seqtmp.tfa ");
	strcat(command,motif_name);
	strcat(command," > /dev/null\n");
	system(command);
	/*system("rm seqtmp.tfa\n");*/

	ArrowCursor();
	info("Ballast motif file %s created",motif_name);
}

static void CreateAlignTree(ButtoN but)
{
	char path[FILENAMELEN];
	char phylip_name[FILENAMELEN];
	panel_data data;

	GetTitle(savetreetext, filename, FILENAMELEN);
	strcpy(phylip_name,filename);
	stripspace(filename);

	info("Doing pairwise alignments...");
	GetPanelExtra(seq_panel.seqs,&data);
	if(save_log)
	{
        	get_path(mult_aln.filename,path);
        	strcpy(save_log_filename,path);
        	strcat(save_log_filename,"log");
		if ((save_log_fd=fopen(save_log_filename,"a"))==NULL)
			error("Cannot open log file %s",save_log_filename);
	}

	WatchCursor();
	if (Visible(savetreew))
	{
		Remove(savetreew);
		savetreew=NULL;
	}
	make_tree(&mult_aln,opt,FALSE);
        if (save_parameters) create_parameter_output(opt,mult_aln);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}
	ArrowCursor();
	info("Tree %s created",filename);
}

static void DrawTree(ButtoN but)
{
	char path[FILENAMELEN];
	char phylip_name[FILENAMELEN];
	char clustal_name[FILENAMELEN];
	char dist_name[FILENAMELEN];
	char bionj_name[FILENAMELEN];
	char nexus_name[FILENAMELEN];
	panel_data data;

	if((*opt.tree_opt).output_tree_clustal)
	{
		GetTitle(drawnjtreetext, filename, FILENAMELEN);
		stripspace(filename);
		strcpy(clustal_name,filename);
	}
	if((*opt.tree_opt).output_tree_phylip)
	{
		GetTitle(drawphtreetext, filename, FILENAMELEN);
		stripspace(filename);
		strcpy(phylip_name,filename);
	}
	if((*opt.tree_opt).output_tree_distances)
	{
		GetTitle(drawdsttreetext, filename, FILENAMELEN);
		stripspace(filename);
		strcpy(dist_name,filename);
	}
	if((*opt.tree_opt).output_tree_bionj)
	{
		GetTitle(drawbionjtreetext, filename, FILENAMELEN);
		stripspace(filename);
		strcpy(bionj_name,filename);
	}
        if((*opt.tree_opt).output_tree_nexus)
        {
                GetTitle(drawnxstreetext, filename, FILENAMELEN);
                stripspace(filename);
                strcpy(nexus_name,filename);
        }


	info("Calculating tree...");
	WatchCursor();
	GetPanelExtra(seq_panel.seqs,&data);
	if(save_log)
	{
        	get_path(mult_aln.filename,path);
        	strcpy(save_log_filename,path);
        	strcat(save_log_filename,"log");
		if ((save_log_fd=fopen(save_log_filename,"a"))==NULL)
			error("Cannot open log file %s",save_log_filename);
	}
	if (Visible(drawtreew))
	{
		Remove(drawtreew);
		drawtreew=NULL;
	}
	phylogenetic_tree(mult_aln,phylip_name,clustal_name,dist_name,bionj_name,nexus_name,FALSE,*opt.tree_opt);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}
	ArrowCursor();
	info("Tree %s created",filename);
}

static void BootstrapTree(ButtoN but)
{
	char phylip_name[FILENAMELEN];
	char clustal_name[FILENAMELEN];
	char nexus_name[FILENAMELEN];
	char path[FILENAMELEN];
	panel_data data;

	if((*opt.tree_opt).output_tree_clustal)
	{
		GetTitle(bootnjtreetext, filename, FILENAMELEN);
		stripspace(filename);
		strcpy(clustal_name,filename);
	}
	if((*opt.tree_opt).output_tree_phylip)
	{
		GetTitle(bootphtreetext, filename, FILENAMELEN);
		stripspace(filename);
		strcpy(phylip_name,filename);
	}
        if((*opt.tree_opt).output_tree_nexus)
        {
                GetTitle(bootnxstreetext, filename, FILENAMELEN);
                stripspace(filename);
                strcpy(nexus_name,filename);
        }


	info("Bootstrapping tree...");

	WatchCursor();
	GetPanelExtra(seq_panel.seqs,&data);
	if(save_log)
	{
        	get_path(mult_aln.filename,path);
        	strcpy(save_log_filename,path);
        	strcat(save_log_filename,"log");
		if ((save_log_fd=fopen(save_log_filename,"a"))==NULL)
			warning("Cannot open log file %s",save_log_filename);
	}
	if (Visible(boottreew))
	{
		Remove(boottreew);
		boottreew=NULL;
	}
	bootstrap_tree(mult_aln,phylip_name,clustal_name,nexus_name,FALSE,*opt.tree_opt,*opt.bstree_opt);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}
	info("Bootstrap tree %s created",filename);
	ArrowCursor();
}

static void OpenSeqFile (IteM item)
{
	int n;
	panel_data data;

	GetPanelExtra(seq_panel.names,&data);
	if (data.nseqs>0)
	{
		if (Message(MSG_YN,"Replace existing sequences ?")==ANS_NO)
			return;
	}

	if (!GetInputFileName (filename,FILENAMELEN,"","")) return;
 
	GetPanelExtra(seq_panel.names,&data);
	data.nseqs=0;
	data.vseqs=0;
	SetPanelExtra(seq_panel.names,&data);
	GetPanelExtra(seq_panel.seqs,&data);
	data.nseqs=0;
	data.vseqs=0;
	SetPanelExtra(seq_panel.seqs,&data);
	
	n=seq_input(filename,opt.explicit_type,FALSE,&mult_aln);
	if (n<=0)
	{
		info("File %s not loaded.",mult_aln.filename);
		return;
	}

	load_aln(seq_panel,0,0,mult_aln.nseqs,TRUE);

	ncutseqs=0;

	info("File %s loaded.",mult_aln.filename);
}

static void AppendSeqFile (IteM item)
{
	int n;
	panel_data data;

	if (!GetInputFileName (filename,FILENAMELEN,"","")) return;
 
	GetPanelExtra(seq_panel.names,&data);
	data.nseqs=0;
	SetPanelExtra(seq_panel.names,&data);
	GetPanelExtra(seq_panel.seqs,&data);
	n=seq_input(filename,opt.explicit_type,TRUE,&mult_aln);
	if (n<=0)
	{
		info("File %s not loaded.",mult_aln.filename);
		return;
	}

	load_aln(seq_panel,0,0,mult_aln.nseqs,FALSE);

	info("File %s appended.",mult_aln.filename);
}

static void OpenPrf1File (IteM item)
{
	int i,j,n,tmpn=0,tmpfs;
	sint *tmplen_array;
	sint *tmpindex;
	char **tmp_array;
	char **tmpnames;
	char **tmptitles;
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);
	if (mult_aln.prf1.nseqs>0)
	{
		if (Message(MSG_YN,"Replace existing sequences ?")==ANS_NO)
			return;
	}

	if (!GetInputFileName (filename,FILENAMELEN,"","")) return;
 
	if(mult_aln.nseqs>mult_aln.prf1.nseqs)
	{
		tmpn=mult_aln.nseqs-mult_aln.prf1.nseqs;
		tmpfs=mult_aln.prf1.nseqs;
		tmpnames=(char **)ckalloc((tmpn+1)*sizeof(char *));
		tmptitles=(char **)ckalloc((tmpn+1)*sizeof(char *));
		tmplen_array=(sint *)ckalloc((tmpn+1)*sizeof(sint));
		tmpindex=(sint *)ckalloc((tmpn+1)*sizeof(sint));
		tmp_array=(char **)ckalloc((tmpn+1)*sizeof(char *));
		for(i=mult_aln.prf1.nseqs;i<mult_aln.nseqs;i++)
		{
			tmpnames[i-mult_aln.prf1.nseqs]=(char *)ckalloc((MAXNAMES+2)*sizeof(char));
			tmptitles[i-mult_aln.prf1.nseqs]=(char *)ckalloc((MAXTITLES+2)*sizeof(char));
			strcpy(tmpnames[i-mult_aln.prf1.nseqs],mult_aln.seqs[i].name);

			strcpy(tmptitles[i-mult_aln.prf1.nseqs],mult_aln.seqs[i].title);
			tmplen_array[i-mult_aln.prf1.nseqs]=mult_aln.seqs[i].len;
			tmpindex[i-mult_aln.prf1.nseqs]=mult_aln.seqs[i].output_index;
			tmp_array[i-mult_aln.prf1.nseqs]=(char *)ckalloc((mult_aln.seqs[i].len+2)*sizeof(char));
			for(j=0;j<mult_aln.seqs[i].len;j++)
				tmp_array[i-mult_aln.prf1.nseqs][j]=mult_aln.seqs[i].data[j];
		}
	}

	GetPanelExtra(prf_panel[0].names,&data);
	data.nseqs=0;
	data.vseqs=0;
	SetPanelExtra(prf_panel[0].names,&data);
	GetPanelExtra(prf_panel[0].seqs,&data);
	data.nseqs=0;
	data.vseqs=0;
	SetPanelExtra(prf_panel[0].seqs,&data);
        data.prf_no=1;
        n=profile_input(filename,1,opt.explicit_type,&mult_aln);
	if (n<=0)
	{
		info("File %s not loaded.",mult_aln.filename);
		return;
	}
	load_aln(prf_panel[0],1,0,mult_aln.prf1.nseqs,TRUE);

	if(tmpn!=0)
	{
		mult_aln.nseqs=tmpn+mult_aln.prf1.nseqs;
		realloc_aln(mult_aln.prf1.nseqs+1,mult_aln.nseqs,&mult_aln);
		for(i=mult_aln.prf1.nseqs;i<mult_aln.nseqs;i++)
		{
			mult_aln.seqs[i].name=(char *)ckalloc((MAXNAMES+2)*sizeof(char));
			mult_aln.seqs[i].title=(char *)ckalloc((MAXTITLES+2)*sizeof(char));

			strcpy(mult_aln.seqs[i].name,tmpnames[i-mult_aln.prf1.nseqs]);
			ckfree(tmpnames[i-mult_aln.prf1.nseqs]);
			strcpy(mult_aln.seqs[i].title,tmptitles[i-mult_aln.prf1.nseqs]);
			ckfree(tmptitles[i-mult_aln.prf1.nseqs]);
			mult_aln.seqs[i].len=tmplen_array[i-mult_aln.prf1.nseqs];
			mult_aln.seqs[i].output_index=tmpindex[i-mult_aln.prf1.nseqs]-tmpfs+mult_aln.prf1.nseqs;
			mult_aln.seqs[i].data=(char *)ckalloc((mult_aln.seqs[i].len+2)*sizeof(char));
			for(j=1;j<=mult_aln.seqs[i].len;j++)
				mult_aln.seqs[i].data[j]=tmp_array[i-mult_aln.prf1.nseqs-1][j];
			ckfree(tmp_array[i-mult_aln.prf1.nseqs-1]);
		}
		ckfree(tmpnames);
		ckfree(tmptitles);
		ckfree(tmplen_array);
		ckfree(tmpindex);
		ckfree(tmp_array);
	}
	load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,TRUE);

	ncutseqs=0;

	info("File %s loaded.",mult_aln.filename);
}

static void OpenPrf2File (IteM item)
{
	int n;
	panel_data data;

	GetPanelExtra(prf_panel[1].seqs,&data);
	if (mult_aln.nseqs>mult_aln.prf1.nseqs)
	{
		if (Message(MSG_YN,"Replace existing sequences ?")==ANS_NO)
			return;
	}

	if (!GetInputFileName (filename,FILENAMELEN,"","")) return;
 
	SetPanelExtra(prf_panel[1].seqs,&data);
	GetPanelExtra(prf_panel[1].names,&data);
	data.nseqs=0;
	data.vseqs=0;
	data.prf_no=2;
	SetPanelExtra(prf_panel[1].names,&data);
	GetPanelExtra(prf_panel[1].seqs,&data);
	data.nseqs=0;
	data.vseqs=0;
	data.prf_no=2;
	SetPanelExtra(prf_panel[1].seqs,&data);
        n=profile_input(filename,2,opt.explicit_type,&mult_aln);
	if (n<=0)
	{
		info("File %s not loaded.",mult_aln.filename);
		return;
	}
	ncutseqs=0;
	load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,TRUE);

	info("File %s loaded.",mult_aln.prf2.filename);
}


static void BlackandWhite(IteM item)
{

	set_bw();
	
	if (aln_mode == MULTIPLEM)
		color_seqs();
	else
	{
		color_prf1();
		color_prf2();
	}
	usebw=TRUE;
	usedefcolors=FALSE;
	useusercolors=FALSE;
        SetStatus(bw_item,usebw);
        SetStatus(defcol_item,usedefcolors);
        SetStatus(usercol_item,useusercolors);
	info("Done.");
}


static void DefColorPar(IteM item)
{
	panel_data data;

	GetPanelExtra(active_panel.seqs,&data);

	if (explicit_par_file != NULL)
		ckfree(explicit_par_file);
	explicit_par_file=NULL;
	if(mult_aln.dnaflag)
		par_file=find_file(def_dnapar_file);
	else
		par_file=find_file(def_protpar_file);
	init_color_parameters(par_file,inverted,mult_aln.dnaflag);
	if (aln_mode == MULTIPLEM)
		color_seqs();
	else
	{
		color_prf1();
		color_prf2();
	}
	usebw=FALSE;
	usedefcolors=TRUE;
	useusercolors=FALSE;
        SetStatus(bw_item,usebw);
        SetStatus(defcol_item,usedefcolors);
        SetStatus(usercol_item,useusercolors);
	info("Done.");
}

void set_reset_new_gaps(IteM i)
{
        (*opt.mult_opt).reset_alignments_new=GetStatus(i);
	if((*opt.mult_opt).reset_alignments_new==TRUE)
	{
		(*opt.mult_opt).reset_alignments_all=FALSE;
        	SetStatus(all_gaps_item,(*opt.mult_opt).reset_alignments_all);
	}
}
void set_reset_all_gaps(IteM i)
{
        (*opt.mult_opt).reset_alignments_all=GetStatus(i);
	if((*opt.mult_opt).reset_alignments_all==TRUE)
	{
		(*opt.mult_opt).reset_alignments_new=FALSE;
        	SetStatus(new_gaps_item,(*opt.mult_opt).reset_alignments_new);
	}
}
 

static void OpenColorParWin(IteM item)
{

	read_file_window("Input Color File","COLOR PARAMETER FILE NAME:",explicit_par_file,OpenColorPar);
}

static void OpenColorPar(ButtoN but)
{
	panel_data data;

	GetPanelExtra(active_panel.seqs,&data);
	GetTitle(readfiletext, filename, FILENAMELEN);
	stripspace(filename);
 
	if (explicit_par_file != NULL)
		ckfree(explicit_par_file);
	explicit_par_file=(char *)ckalloc(FILENAMELEN*sizeof(char));
	if (par_file != NULL)
		ckfree(par_file);
	par_file=(char *)ckalloc(FILENAMELEN*sizeof(char));

	strcpy(explicit_par_file,filename);
	strcpy(par_file,filename);
        info("Loading color file: %s\n",par_file);
	init_color_parameters(par_file,inverted,mult_aln.dnaflag);
	if (Visible(readfilew))
	{
		Remove(readfilew);
		readfilew=NULL;
	}
	if (aln_mode == MULTIPLEM)
		color_seqs();
	else
	{
		color_prf1();
		color_prf2();
	}
	usebw=FALSE;
	usedefcolors=FALSE;
	useusercolors=TRUE;
        SetStatus(bw_item,usebw);
        SetStatus(defcol_item,usedefcolors);
        SetStatus(usercol_item,useusercolors);
	info("Done.");
}

static void RemoveGapPos(IteM item)
{
	panel_data data;



	if(aln_mode==MULTIPLEM)
	{
		GetPanelExtra(seq_panel.seqs,&data);
		if (mult_aln.nseqs==0)
		{
			Message(MSG_OK,"No file loaded.");
			return;
		}

		if (Message(MSG_YN,"Remove positions that contain gaps in all sequences ?")==ANS_NO)
			return;
		remove_gap_pos(data.prf_no,mult_aln);
		load_aln(seq_panel,0,0,mult_aln.nseqs,FALSE);
	}
	else
	{
		GetPanelExtra(prf_panel[0].seqs,&data);
		if (mult_aln.nseqs==0)
		{
			Message(MSG_OK,"No file loaded.");
			return;
		}

		if (Message(MSG_YN,"Remove positions that contain gaps in all sequences ?")==ANS_NO)
			return;
		remove_gap_pos(data.prf_no,mult_aln);
		remove_gap_pos(data.prf_no,mult_aln);
		load_aln(prf_panel[0],1,0,mult_aln.prf1.nseqs,FALSE);
		GetPanelExtra(prf_panel[1].seqs,&data);
		remove_gap_pos(data.prf_no,mult_aln);
		load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
	}
	info("Gap positions removed.");
}


static void remove_gap_pos(sint prf_no,ALN mult_aln)
{
	int i,j,k,ngaps;
	int fseq,lseq;

	if(prf_no==0) {
		fseq=0;
		lseq=mult_aln.nseqs;
	}
	else if(prf_no==1) {
		fseq=0;
		lseq=mult_aln.prf1.nseqs;
	}
	else {
		fseq=mult_aln.prf1.nseqs;
		lseq=mult_aln.nseqs;
	}
	if(fseq>=lseq) return;

	for (i=0;i<mult_aln.seqs[fseq].len;)
	{
		ngaps=0;
		for (j=fseq;j<lseq;j++)
			if(!isalpha(mult_aln.seqs[j].data[i])) ngaps++;
		if (ngaps==lseq-fseq)
		{
			for (j=fseq;j<lseq;j++)
			{
				for(k=i+1;k<=mult_aln.seqs[j].len;k++)
					mult_aln.seqs[j].data[k-1]=mult_aln.seqs[j].data[k];
				mult_aln.seqs[j].len--;
			}
			if(prf_no==1 && mult_aln.prf1.ss.sec_struct_mask != NULL)
				for(k=i+1;k<=mult_aln.seqs[fseq].len;k++)
					mult_aln.prf1.ss.sec_struct_mask[k-1]=mult_aln.prf1.ss.sec_struct_mask[k];
			if(prf_no==1 && mult_aln.prf1.ss.gap_penalty_mask != NULL)
				for(k=i+1;k<=mult_aln.seqs[fseq].len;k++)
					mult_aln.prf1.ss.gap_penalty_mask[k-1]=mult_aln.prf1.ss.gap_penalty_mask[k];
			if(prf_no==2 && mult_aln.prf2.ss.sec_struct_mask != NULL)
				for(k=i+1;k<=mult_aln.seqs[fseq].len;k++)
					mult_aln.prf2.ss.sec_struct_mask[k-1]=mult_aln.prf2.ss.sec_struct_mask[k];
			if(prf_no==2 && mult_aln.prf2.ss.gap_penalty_mask != NULL)
				for(k=i+1;k<=mult_aln.seqs[fseq].len;k++)
					mult_aln.prf2.ss.gap_penalty_mask[k-1]=mult_aln.prf2.ss.gap_penalty_mask[k];
			if(mult_aln.seqs[fseq].len<=0) break;
		}
		else i++;
	}
}


static void RemoveGaps(IteM item)
{
	int i,j,sl;
	panel_data data,sdata;
	Boolean sel=FALSE;

	GetPanelExtra(active_panel.names,&data);
	if (data.nseqs==0)
	{
		Message(MSG_OK,"No file loaded.");
		return;
	}
	GetPanelExtra(active_panel.seqs,&sdata);
	for (i=0;i<data.nseqs;i++)
		if(data.selected[i]==TRUE)
		{
			sel=TRUE;
			break;
		}
	if(sel==FALSE)
	{
		Message(MSG_OK,"Select sequences by clicking on the names.");
		return;
	}

	if (Message(MSG_YN,"Remove gaps from selected sequences ?")==ANS_NO)
		return;

	for (i=data.firstseq;i<data.firstseq+data.nseqs;i++)
		if(data.selected[i-data.firstseq]==TRUE)
		{
                	sl=0;
                	for(j=0;j<mult_aln.seqs[i].len;++j) {
                        	if(mult_aln.seqs[i].data[j]=='-') continue;
                        	if(!isalpha(mult_aln.seqs[i].data[j])) break;
                        	mult_aln.seqs[i].data[sl++]=mult_aln.seqs[i].data[j];
                	}
                        mult_aln.seqs[i].data[sl]=EOS;
                	mult_aln.seqs[i].len=sl;
		}
	load_aln(active_panel,sdata.prf_no,data.firstseq,data.nseqs,FALSE);
	active_panel.modified=TRUE;
	info("Gaps in selected sequences removed.");
}

static void CutSequences(IteM item)
{
	int i,pos;
	Boolean sel=FALSE;
	panel_data data;

	GetPanelExtra(active_panel.names,&data);
	if (data.nseqs==0)
	{
		Message(MSG_OK,"No file loaded.");
		return;
	}

	for (i=0;i<data.nseqs;i++)
		if(data.selected[i]==TRUE)
		{
			sel=TRUE;
			pos=i;
			break;
		}
	if(sel==FALSE)
	{
		Message(MSG_OK,"Select sequences to be cut by clicking on the names.");
		return;
	}

	if(ncutseqs>0)
	{
		if (Message(MSG_YN,"The previously cut sequences will be lost.\nDo you want to continue?")==ANS_NO) return;
	}

	if (saveseqlen_array!=NULL) ckfree(saveseqlen_array);
	if (saveseq_array!=NULL)
	{
		for(i=0;i<ncutseqs;i++)
		{
			if (saveseq_array[i]!=NULL) ckfree(saveseq_array[i]);
		}
		ckfree(saveseq_array);
	}
	if (savetitles!=NULL)
	{
		for(i=0;i<ncutseqs;i++)
		{
			if (savetitles[i]!=NULL) ckfree(savetitles[i]);
		}
		ckfree(savetitles);
	}
	if (savenames!=NULL)
	{
		for(i=0;i<ncutseqs;i++)
		{
			if (savenames[i]!=NULL) ckfree(savenames[i]);
		}
		ckfree(savenames);
	}
	ncutseqs=0;

	savenames=(char **)ckalloc((data.nseqs+1) * sizeof(char *));
	savetitles=(char **)ckalloc((data.nseqs+1) * sizeof(char *));
	saveseq_array=(char **)ckalloc((data.nseqs+1) * sizeof(char *));
	saveseqlen_array=(sint *)ckalloc((data.nseqs+1) * sizeof(sint));
	for(i=0;i<data.nseqs;i++)
	{
		savenames[i]=NULL;
		savetitles[i]=NULL;
		saveseq_array[i]=NULL;
	}
	GetPanelExtra(active_panel.seqs,&data);
	if (data.prf_no == 0)
		cut_multiplem();
	else if (data.prf_no == 1)
		cut_profile1();
	else if (data.prf_no == 2)
		cut_profile2();

	GetPanelExtra(active_panel.names,&data);
	if(pos>=data.nseqs) pos=data.nseqs-1;
	if(data.nseqs>0)
		data.selected[pos]=TRUE;
	SetPanelExtra(active_panel.names,&data);
	DrawPanel(active_panel.names);

	active_panel.modified=TRUE;
	info("Cut %d sequences.",ncutseqs);
}

static void cut_multiplem(void)
{
	int i,j;
	panel_data sdata;
	panel_data ndata;

        GetPanelExtra(active_panel.names,&ndata);
        GetPanelExtra(active_panel.seqs,&sdata);
	for (i=ndata.nseqs-1;i>=0;i--)
	{
		if(ndata.selected[i]==TRUE)
		{
			ssave(mult_aln.seqs[i]);
			for(j=i;j<ndata.nseqs-1;j++)
				sscpy(&mult_aln.seqs[j],mult_aln.seqs[j+1]);
		}
	}
        mult_aln.nseqs-=ncutseqs;
	if (ncutseqs>0)
		if(mult_aln.nseqs<=sdata.vseqs)
			load_aln(active_panel,sdata.prf_no,0,mult_aln.nseqs,TRUE);
		else
			load_aln(active_panel,sdata.prf_no,0,mult_aln.nseqs,FALSE);
}

static void cut_profile1(void)
{
	int i,j;
	panel_data ndata,sdata;

        GetPanelExtra(active_panel.names,&ndata);
        GetPanelExtra(active_panel.seqs,&sdata);
	for (i=ndata.nseqs-1;i>=0;i--)
	{
		if(ndata.selected[i]==TRUE)
		{
			ssave(mult_aln.seqs[i]);
			for(j=i;j<mult_aln.nseqs-1;j++)
				sscpy(&mult_aln.seqs[j],mult_aln.seqs[j+1]);
		}
	}
        mult_aln.prf1.nseqs-=ncutseqs;
	mult_aln.nseqs-=ncutseqs;
	if (ncutseqs>0)
	{
		if(mult_aln.prf1.nseqs<=sdata.vseqs)
			load_aln(active_panel,sdata.prf_no,0,mult_aln.prf1.nseqs,TRUE);
		else
			load_aln(active_panel,sdata.prf_no,0,mult_aln.prf1.nseqs,FALSE);
		if (mult_aln.nseqs>mult_aln.prf1.nseqs)
			load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
	}
}

static void cut_profile2(void)
{
	int i,j;
	panel_data ndata,sdata;

        GetPanelExtra(active_panel.names,&ndata);
        GetPanelExtra(active_panel.seqs,&sdata);
	for (i=ndata.nseqs-1;i>=0;i--)
	{
		if(ndata.selected[i]==TRUE)
		{
			ssave(mult_aln.seqs[i+mult_aln.prf1.nseqs]);
			for(j=i+mult_aln.prf1.nseqs;j<mult_aln.nseqs-1;j++)
				sscpy(&mult_aln.seqs[j],mult_aln.seqs[j+1]);
		}
	}
        mult_aln.prf2.nseqs-=ncutseqs;
        mult_aln.nseqs-=ncutseqs;
	if (ncutseqs>0)
		if(mult_aln.nseqs-mult_aln.prf1.nseqs<=sdata.vseqs)
			load_aln(active_panel,sdata.prf_no,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
		else
			load_aln(active_panel,sdata.prf_no,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,TRUE);
}

static void PasteSequences(IteM item)
{
	int insert;
	int i,n;
	panel_data data;

	if (ncutseqs<=0)
	{
		Message(MSG_OK,"No sequences available for pasting.\n"
                  " Cut selected sequences first.");
		 return;
	}

	GetPanelExtra(active_panel.names,&data);
	n=ncutseqs;
	insert=(-1);
        if (data.nseqs>0)
        {
                for(i=data.nseqs-1;i>=0;i--)
                        if(data.selected[i]==TRUE)
                        {
                                insert=i;
                                break;
                        }
                if (insert==-1)
                {
                        Message(MSG_OK,"Select a sequence by clicking on the name.\n"
                        " Cut sequences will be pasted after this one.");
			return;
                }
        }
	GetPanelExtra(active_panel.seqs,&data);

	if (data.prf_no == 2)
	{
		insert += mult_aln.prf1.nseqs;
		for(i=mult_aln.prf1.nseqs+data.nseqs-1;i>insert;i--)
			sscpy(&mult_aln.seqs[i+ncutseqs],mult_aln.seqs[i]);
	}
       	else
	{
		for(i=mult_aln.nseqs-1;i>insert;i--)
			sscpy(&mult_aln.seqs[i+ncutseqs],mult_aln.seqs[i]);
	}
	for(i=0;ncutseqs>0;i++)
		sload(&mult_aln.seqs[insert+i+1]);

        if(data.prf_no==0)
        {
                mult_aln.nseqs=data.nseqs+n;
		load_aln(seq_panel,0,0,mult_aln.nseqs,FALSE);
        }
        else if(data.prf_no==1)
        {
                mult_aln.prf1.nseqs=data.nseqs+n;
                mult_aln.nseqs=mult_aln.prf1.nseqs+mult_aln.prf2.nseqs;
		load_aln(active_panel,1,0,mult_aln.prf1.nseqs,FALSE);
		if (mult_aln.nseqs>mult_aln.prf1.nseqs)
			load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
        }
        else if(data.prf_no==2)
        {
		mult_aln.prf2.nseqs=data.nseqs+n;
                mult_aln.nseqs=mult_aln.prf1.nseqs+mult_aln.prf2.nseqs;
		load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
        }

	active_panel.modified=TRUE;
	info("Pasted %d sequences.",n);
}

/* copies a sequence from clustal arrays position j to temp arrays */
static void ssave(SEQ seq)
{
	int k;

	if (saveseq_array[ncutseqs] != NULL) ckfree(saveseq_array[ncutseqs]);
	if (savenames[ncutseqs] != NULL) ckfree(savenames[ncutseqs]);
	if (savetitles[ncutseqs] != NULL) ckfree(savetitles[ncutseqs]);
	savenames[ncutseqs]=(char *)ckalloc((MAXNAMES+2)*sizeof(char));
	savetitles[ncutseqs]=(char *)ckalloc((MAXTITLES+2)*sizeof(char));

	strcpy(savenames[ncutseqs],seq.name);
	strcpy(savetitles[ncutseqs],seq.title);
	saveseqlen_array[ncutseqs]=seq.len;
	saveseq_array[ncutseqs]=(char *)ckalloc((seq.len+2)*sizeof(char));
	for(k=0;k<seq.len;k++)
		saveseq_array[ncutseqs][k]=seq.data[k];
	saveseq_array[ncutseqs][k]= EOS;
	ncutseqs++;
}

/* copies a sequence from clustal arrays position i to position j */
static void sscpy(SEQ *seq1, SEQ seq2)
{
	int k;


	strcpy(seq1->name,seq2.name);
	strcpy(seq1->title,seq2.title);
	seq1->len=seq2.len;
	realloc_seq(seq1,seq1->len);

	for(k=0;k<seq2.len;k++)
		seq1->data[k]=seq2.data[k];
	seq1->data[k]= EOS;
}

/* copies last sequence in temp arrays to clustal arrays after entry i */
static void sload(SEQ *seq)
{
	int k;

	if (ncutseqs<1) return;

	ncutseqs--;
	strcpy(seq->name,savenames[ncutseqs]);
	strcpy(seq->title,savetitles[ncutseqs]);
	seq->len=saveseqlen_array[ncutseqs];
	realloc_seq(seq,seq->len);
	for(k=0;k<seq->len;k++)
		seq->data[k]=saveseq_array[ncutseqs][k];
	seq->data[k]= EOS;
}

static void SelectSeqs(IteM item)
{
	select_seqs(seq_panel,TRUE);
}

static void SelectPrf1(IteM item)
{
	select_seqs(prf_panel[0],TRUE);
}

static void SelectPrf2(IteM item)
{
	select_seqs(prf_panel[1],TRUE);
}

static void MergeProfiles(IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);
        if (mult_aln.nseqs==mult_aln.prf1.nseqs)
        {
                error("Profile 2 not loaded");
                return;
        }
	data.prf_no=1;
	mult_aln.prf1.nseqs=mult_aln.nseqs;
	mult_aln.prf2.nseqs=0;
	load_aln(prf_panel[0],1,0,mult_aln.prf1.nseqs,FALSE);
	load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
	active_panel=prf_panel[0];

	info("Added Profile 2 to Profile 1.");
}


static void ClearSeqRange(IteM item)
{
	if(aln_mode==MULTIPLEM)
		clear_seqrange(seq_panel);
	else
	{
		clear_seqrange(prf_panel[1]);
		clear_seqrange(prf_panel[0]);
	}
}

static void ClearSeqs(IteM item)
{
	if(aln_mode==MULTIPLEM)
		select_seqs(seq_panel,FALSE);
	else
	{
		select_seqs(prf_panel[1],FALSE);
		select_seqs(prf_panel[0],FALSE);
	}
}

static void clear_seqrange(spanel p)
{
	int f,l;
	panel_data data;

	GetPanelExtra(p.seqs,&data);
	f=data.firstsel;
	l=data.lastsel;
	data.firstsel=data.lastsel=(-1);
	SetPanelExtra(p.seqs,&data);
	highlight_seqrange(p.seqs,f,l,NORMAL);
}

static void select_seqs(spanel p,Boolean flag)
{
	int i;
	panel_data data;

	GetPanelExtra(p.names,&data);
	if (data.nseqs == 0) return;

	for (i=0;i<data.nseqs;i++)
		data.selected[i]=flag;

	SetPanelExtra(p.names,&data);
	draw_names(p.names);
	if(flag==TRUE) active_panel=p;
}

static void CAlignWin (IteM item)
{
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);
        if (mult_aln.nseqs<=0)
        {
                error("No sequences loaded");
                return;
        }
        if (mult_aln.nseqs <= 1)
	{
                error("Alignment has only %d sequences",mult_aln.nseqs);
                return;
        }
	do_align_window(&calignw,&ctreetext,NEW,"Complete Alignment",CompleteAlign,mult_aln);
}

void CompleteAlign(ButtoN but)
{
	char phylip_name[FILENAMELEN];
	panel_data data;

	GetTitle(ctreetext, filename, FILENAMELEN);
	stripspace(filename);

        strcpy(phylip_name,filename);

	GetPanelExtra(seq_panel.seqs,&data);
	if (!open_aln_files(mult_aln)) return;
 
	WatchCursor();
	if (Visible(calignw))
	{
		Remove(calignw);
		calignw=NULL;
	}
        align(&mult_aln,opt,FALSE);

	create_alignment_output(mult_aln,0,mult_aln.nseqs,*opt.alnout_opt,*opt.ss_opt);

        if (save_parameters) create_parameter_output(opt,mult_aln);

	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}
/* reload the sequences from the output file (so that the sequence order is
correct - either INPUT or ALIGNED , don't output messages */
	reload_alignment(mult_aln.nseqs,mult_aln.seqs);

        load_aln(seq_panel,0,0,mult_aln.nseqs,FALSE);
	ArrowCursor();
}

static void RealignSeqsWin (IteM item)
{
	int i;
	Boolean sel=FALSE;
	panel_data data;

	GetPanelExtra(seq_panel.names,&data);

        if (data.nseqs<=0)
        {
                error("No sequences loaded");
                return;
        }
        if (data.nseqs <= 1)
	{
                error("Alignment has only %d sequences",data.nseqs);
                return;
        }

/* check some sequences have been selected */

	for (i=0;i<data.nseqs;i++)
		if(data.selected[i]==TRUE)
		{
			sel=TRUE;
			break;
		}
	if(sel==FALSE)
	{
		Message(MSG_OK,"Select sequences to be realigned\n"
                               "by clicking on the names.");
		return;
	}

	do_align_window(&ralignw,&rtreetext,NEW,"Realign Sequences",RealignSeqs,mult_aln);
}

static void RealignSeqs(ButtoN but)
{
	int insert;
	int i,j,n;
	panel_data ndata,sdata;
	char phylip_name[FILENAMELEN];

	GetTitle(rtreetext, filename, FILENAMELEN);
	stripspace(filename);

        strcpy(phylip_name,filename);


/* cut selected sequences */

	GetPanelExtra(seq_panel.names,&ndata);
	GetPanelExtra(seq_panel.seqs,&sdata);

	if (!open_aln_files(mult_aln)) return;

	if (saveseqlen_array!=NULL) ckfree(saveseqlen_array);
	if (saveseq_array!=NULL)
	{
		for(i=0;i<ncutseqs;i++)
		{
			if (saveseq_array[i]!=NULL) ckfree(saveseq_array[i]);
		}
		ckfree(saveseq_array);
	}
	if (savetitles!=NULL)
	{
		for(i=0;i<ncutseqs;i++)
		{
			if (savetitles[i]!=NULL) ckfree(savetitles[i]);
		}
		ckfree(savetitles);
	}
	if (savenames!=NULL)
	{
		for(i=0;i<ncutseqs;i++)
		{
			if (savenames[i]!=NULL) ckfree(savenames[i]);
		}
		ckfree(savenames);
	}
	ncutseqs=0;

	savenames=(char **)ckalloc((ndata.nseqs+1) * sizeof(char *));
	savetitles=(char **)ckalloc((ndata.nseqs+1) * sizeof(char *));
	saveseq_array=(char **)ckalloc((ndata.nseqs+1) * sizeof(char *));
	saveseqlen_array=(sint *)ckalloc((ndata.nseqs+1) * sizeof(sint));
	for(i=0;i<ndata.nseqs;i++)
	{
		savenames[i]=NULL;
		savetitles[i]=NULL;
		saveseq_array[i]=NULL;
	}
	for (i=ndata.nseqs-1;i>=0;i--)
	{
		if(ndata.selected[i]==TRUE)
		{
			ssave(mult_aln.seqs[i]);
			for(j=i;j<ndata.nseqs-1;j++)
				sscpy(&mult_aln.seqs[j],mult_aln.seqs[j+1]);
		}
	}
        mult_aln.nseqs=ndata.nseqs-ncutseqs;

/* paste selected sequences at the end */
	n=ncutseqs;
	mult_aln.prf1.nseqs=mult_aln.nseqs;
	insert=mult_aln.prf1.nseqs-1;

	for(i=mult_aln.nseqs-1;i>insert;i--)
		sscpy(&mult_aln.seqs[i+ncutseqs],mult_aln.seqs[i]);
	for(i=0;ncutseqs>0;i++)
		sload(&mult_aln.seqs[insert+i+1]);

        mult_aln.nseqs=mult_aln.prf1.nseqs+n;
	strcpy(mult_aln.treename, phylip_name);

/* align profile 2 sequences to profile 1 */
	WatchCursor();
	if (Visible(ralignw))
	{
		Remove(ralignw);
		ralignw=NULL;
	}
        new_sequence_align(&mult_aln,opt,FALSE,FALSE);
        if (save_parameters) create_parameter_output(opt,mult_aln);

	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}
/* reload the sequences from the output file (so that the sequence order is
correct - either INPUT or ALIGNED */
	reload_alignment(mult_aln.nseqs,mult_aln.seqs);
	load_aln(seq_panel,0,0,mult_aln.nseqs,FALSE);

	GetPanelExtra(seq_panel.names,&ndata);
	for (i=0;i<mult_aln.prf1.nseqs;i++)
		ndata.selected[i]=FALSE;
	for (i=mult_aln.prf1.nseqs;i<mult_aln.nseqs;i++)
		ndata.selected[i]=TRUE;
	SetPanelExtra(seq_panel.names,&ndata);
	draw_names(seq_panel.names);
	ArrowCursor();
	info("Selected sequences realigned.");
}

static void RealignSeqRangeWin (IteM item)
{
	panel_data data;
	GrouP aligngr;
	GrouP output_list;
	ButtoN align_ok, align_can;
	GrouP maing;
	PopuP end_gap_toggle;
	char name[FILENAMELEN+1];
	char path[FILENAMELEN+1];

	GetPanelExtra(seq_panel.seqs,&data);
        if (mult_aln.nseqs<=0)
        {
                error("No sequences loaded");
                return;
        }
        if (mult_aln.nseqs <= 1)
	{
                error("Alignment has only %d sequences",mult_aln.nseqs);
                return;
        }

/* check a range has been selected */

	if(data.firstsel==-1)
	{
		Message(MSG_OK,"Select residue range to be realigned\n"
                               "by clicking in the sequence display area.");
		return;
	}


	get_path(mult_aln.filename,path);
	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	rralignw=FixedWindow(-50, -33, -10, -10,"Realign Residue Range",RemoveWin);

	maing=HiddenGroup(rralignw,2,0,NULL);
	SetGroupSpacing(maing,0,10);

	make_prompt(rralignw, "Output Guide Tree File:");
        stdLineHeight=18;
        SelectFont(programFont);
	Break(rralignw);
	rrtreetext=DialogText(rralignw, "", 35, NULL);
	strcpy(name,path);
	strcat(name,"dnd");
	SetTitle(rrtreetext, name);
	Break(rralignw);

	make_prompt(rralignw, "Output Alignment Files:");
	output_list=HiddenGroup(rralignw, 2, 0, NULL);
	if((*opt.alnout_opt).output_clustal) {
		make_prompt(output_list,"Clustal: ");
		cl_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"aln");
		SetTitle(cl_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_nbrf) {
		make_prompt(output_list,"NBRF/PIR: ");
		pir_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"pir");
		SetTitle(pir_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_gcg) {
		make_prompt(output_list,"GCG/MSF: ");
		msf_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"msf");
		SetTitle(msf_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_phylip) {
		make_prompt(output_list,"Phylip: ");
		phylip_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"phy");
		SetTitle(phylip_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_gde) {
		make_prompt(output_list,"GDE: ");
		gde_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"gde");
		SetTitle(gde_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_nexus) {
		make_prompt(output_list,"NEXUS: ");
		nexus_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"nxs");
		SetTitle(nexus_outtext, name);
		Break(output_list);
	}

	Break(rralignw);
	end_gap_toggle=make_toggle(rralignw,"Realign Segment End Gap Penalties","ON","OFF",&realign_endgappenalties,set_realign_endgappenalties);
	Break(rralignw);

	aligngr=HiddenGroup(rralignw, 2, 0, NULL);
	shift(aligngr, 60, 20);
	align_ok=PushButton(aligngr, " ALIGN ", RealignSeqRange);
	shift(aligngr, 20,0);
	align_can=PushButton(aligngr, "CANCEL", CancelWin);

	Show(rralignw);
}

static void RealignSeqRange(ButtoN but)
{
	int i,j;
	int fs,save_order,length,length1,length2;
	panel_data data;
	sint *tmplen_array;
	char **tmp_array;
	sint *newlen_array;
	char **new_array;
	char phylip_name[FILENAMELEN];
	SEQ *seqs;

	GetTitle(rrtreetext, filename, FILENAMELEN);
	stripspace(filename);

        strcpy(phylip_name,filename);

	GetPanelExtra(seq_panel.seqs,&data);
	if (!open_aln_files(mult_aln)) return;

	WatchCursor();
	if (Visible(rralignw))
	{
		Remove(rralignw);
		rralignw=NULL;
	}
/* save the alignment into a temporary area */

	seqs=mult_aln.seqs;

	tmplen_array=(sint *)ckalloc((data.nseqs+2) * sizeof(sint));
	tmp_array=(char **)ckalloc((data.nseqs+2) * sizeof(char *));
	for (i=0;i<data.nseqs;i++)
	{
		tmplen_array[i]=seqs[i].len;
		tmp_array[i]=(char *)ckalloc((data.ncols+2) * sizeof(char));
		for(j=0;j<seqs[i].len;j++)
			tmp_array[i][j]=seqs[i].data[j];
		for(j=seqs[i].len;j<data.ncols;j++)
			tmp_array[i][j]=GAP2;
	}

/* copy the selected residue range to the clustal alignment arrays */

	fs=data.firstsel;
	length=data.lastsel-data.firstsel+1;
	for (i=0;i<data.nseqs;i++)
	{
		seqs[i].len=length;
		realloc_seq(&seqs[i],length);
		for(j=data.firstsel;j<=data.lastsel;j++)
			seqs[i].data[j-data.firstsel]=tmp_array[i][j];
		seqs[i].data[j-data.firstsel]=EOS;
	}
/* temporarily set the output order to be the same as the input */
	save_order=(*opt.alnout_opt).output_order;
	(*opt.alnout_opt).output_order=INPUT;
/* set the end gaps penalties */
	(*opt.mult_opt->gap_opt).endgappenalties=realign_endgappenalties;

/* align the residue range */
        align(&mult_aln,opt,FALSE);
        if (save_parameters) create_parameter_output(opt,mult_aln);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}

	(*opt.alnout_opt).output_order=save_order;
/* reset the end gaps penalties */
	(*opt.mult_opt->gap_opt).endgappenalties=align_endgappenalties;

/* remove positions that contain just gaps */
	remove_gap_pos(data.prf_no,mult_aln);


/* save the new alignment into another temporary area */
	newlen_array=(sint *)ckalloc((data.nseqs+2) * sizeof(sint));
	new_array=(char **)ckalloc((data.nseqs+2) * sizeof(char *));
	for (i=0;i<data.nseqs;i++)
	{
		newlen_array[i]=seqs[i].len;
		new_array[i]=(char *)ckalloc((seqs[i].len+2) * sizeof(char));
		for(j=0;j<seqs[i].len;j++)
			new_array[i][j]=seqs[i].data[j];
	}

/* paste the realigned range back into the alignment */
	length1=length2=0;
	for (i=0;i<data.nseqs;i++)
	{
		length1=tmplen_array[i]-length+newlen_array[i];
		length2=newlen_array[i];
		seqs[i].len=length1;
		realloc_seq(&seqs[i],length1);
		for(j=0;j<data.firstsel;j++)
			seqs[i].data[j]=tmp_array[i][j];
		for(j=data.firstsel;j<data.firstsel+length2;j++)
			seqs[i].data[j]=new_array[i][j-data.firstsel];
		for(j=data.firstsel+length2;j<length1;j++)
			seqs[i].data[j]=tmp_array[i][data.lastsel+j-data.firstsel-length2+1];
	}
	ckfree(tmplen_array);
	for(i=0;i<data.nseqs;i++)
		ckfree(tmp_array[i]);
	ckfree(tmp_array);
	ckfree(newlen_array);
	for(i=0;i<data.nseqs;i++)
		ckfree(new_array[i]);
	ckfree(new_array);

	if (open_aln_files(mult_aln))
        	create_alignment_output(mult_aln,0,data.nseqs-1,*opt.alnout_opt,*opt.ss_opt);

	load_aln(seq_panel,0,0,mult_aln.nseqs,FALSE);
	GetPanelExtra(seq_panel.seqs,&data);
	data.firstsel=fs;
	data.lastsel=data.firstsel+length2-1;
	SetPanelExtra(seq_panel.seqs,&data);
	highlight_seqrange(seq_panel.seqs,data.firstsel,data.lastsel,HIGHLIGHT);
	ArrowCursor();
	info("Selected sequence range realigned.");
}


void AlignFromTreeWin(IteM item)
{
	panel_data data;

	GetPanelExtra(seq_panel.seqs,&data);
        if (mult_aln.nseqs<=0)
        {
                error("No sequences loaded");
                return;
        }
        if (mult_aln.nseqs < 2)
	{
                error("Alignment has only %d sequences",mult_aln.nseqs);
                return;
        }
	do_align_window(&talignw,&ttreetext,OLD,"Alignment from Guide Tree",AlignFromTree,mult_aln);
}

static void do_align_window(WindoW *ralignw,TexT *rtreetext,Boolean treestatus,char *title,void align_proc(ButtoN but),ALN mult_aln)
{
	WindoW alignw;
	TexT treetext;
	GrouP aligngr;
	GrouP output_list;
	ButtoN align_ok, align_can;
	GrouP maing;
	char name[FILENAMELEN+1];
	char path[FILENAMELEN+1];

	get_path(mult_aln.filename,path);
	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	alignw=FixedWindow(-50, -33, -10, -10,title,RemoveWin);

	maing=HiddenGroup(alignw,2,0,NULL);
	SetGroupSpacing(maing,0,10);

	if(treestatus==NEW)
		make_prompt(alignw, "Output Guide Tree File:");
	else
		make_prompt(alignw, "Input Guide Tree File:");
        stdLineHeight=18;
        SelectFont(programFont);
	Break(alignw);
	treetext=DialogText(alignw, "", 35, NULL);
	strcpy(name,path);
	strcat(name,"dnd");
	SetTitle(treetext, name);
	Break(alignw);

	make_prompt(alignw, "Output Alignment Files:");
	output_list=HiddenGroup(alignw, 2, 0, NULL);
	if((*opt.alnout_opt).output_clustal) {
		make_prompt(output_list,"Clustal: ");
		cl_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"aln");
		SetTitle(cl_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_nbrf) {
		make_prompt(output_list,"NBRF/PIR: ");
		pir_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"pir");
		SetTitle(pir_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_gcg) {
		make_prompt(output_list,"GCG/MSF: ");
		msf_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"msf");
		SetTitle(msf_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_phylip) {
		make_prompt(output_list,"Phylip: ");
		phylip_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"phy");
		SetTitle(phylip_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_gde) {
		make_prompt(output_list,"GDE: ");
		gde_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"gde");
		SetTitle(gde_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_nexus) {
		make_prompt(output_list,"NEXUS: ");
		nexus_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"nxs");
		SetTitle(nexus_outtext, name);
		Break(output_list);
	}

	Break(alignw);
	aligngr=HiddenGroup(alignw, 2, 0, NULL);
	shift(aligngr, 60, 20);
	align_ok=PushButton(aligngr, " ALIGN ", align_proc);
	shift(aligngr, 20,0);
	align_can=PushButton(aligngr, "CANCEL", CancelWin);

	*ralignw=alignw;
	*rtreetext=treetext;
	Show(alignw);
}


static void do_palign_window(WindoW *ralignw,TexT *rtree1text,TexT *rtree2text,Boolean treestatus,char *title,void align_proc(ButtoN but),ALN mult_aln)
{
	WindoW alignw;
	TexT tree1text,tree2text;
	GrouP aligngr;
	GrouP output_list;
	ButtoN align_ok, align_can;
	GrouP maing;
	char name[FILENAMELEN+1];
	char path[FILENAMELEN+1];

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	alignw=FixedWindow(-50, -33, -10, -10,title,RemoveWin);

	maing=HiddenGroup(alignw,2,0,NULL);
	SetGroupSpacing(maing,0,10);

	if(treestatus==NEW)
		make_prompt(alignw, "Output Guide Tree Files:");
	else
		make_prompt(alignw, "Input Guide Tree Files:");
        stdLineHeight=18;
        SelectFont(programFont);
	Break(alignw);
	tree1text=DialogText(alignw, "", 35, NULL);
	get_path(mult_aln.prf1.filename,path);
	strcpy(name,path);
	strcat(name,"dnd");
	SetTitle(tree1text, name);
	Break(alignw);
	tree2text=DialogText(alignw, "", 35, NULL);
	get_path(mult_aln.prf2.filename,path);
	strcpy(name,path);
	strcat(name,"dnd");
	SetTitle(tree2text, name);
	Break(alignw);

	make_prompt(alignw, "Output Alignment Files:");
	output_list=HiddenGroup(alignw, 2, 0, NULL);
	if((*opt.alnout_opt).output_clustal) {
		make_prompt(output_list,"Clustal: ");
		cl_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"aln");
		SetTitle(cl_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_nbrf) {
		make_prompt(output_list,"NBRF/PIR: ");
		pir_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"pir");
		SetTitle(pir_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_gcg) {
		make_prompt(output_list,"GCG/MSF: ");
		msf_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"msf");
		SetTitle(msf_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_phylip) {
		make_prompt(output_list,"Phylip: ");
		phylip_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"phy");
		SetTitle(phylip_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_gde) {
		make_prompt(output_list,"GDE: ");
		gde_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"gde");
		SetTitle(gde_outtext, name);
		Break(output_list);
	}
	if((*opt.alnout_opt).output_nexus) {
		make_prompt(output_list,"NEXUS: ");
		nexus_outtext=DialogText(output_list, "", 35, NULL);
		strcpy(name,path);
		strcat(name,"nxs");
		SetTitle(nexus_outtext, name);
		Break(output_list);
	}

	Break(alignw);
	aligngr=HiddenGroup(alignw, 2, 0, NULL);
	shift(aligngr, 60, 20);
	align_ok=PushButton(aligngr, " ALIGN ", align_proc);
	shift(aligngr, 20,0);
	align_can=PushButton(aligngr, "CANCEL", CancelWin);

	*ralignw=alignw;
	*rtree1text=tree1text;
	*rtree2text=tree2text;
	Show(alignw);
}


void AlignFromTree(ButtoN but)
{
	FILE *tree;
	char phylip_name[FILENAMELEN];
	panel_data data;

	GetTitle(ttreetext, filename, FILENAMELEN);
	stripspace(filename);

        strcpy(phylip_name,filename);
#ifdef VMS
        if((tree=fopen(phylip_name,"r","rat=cr","rfm=var"))==NULL) {
#else
        if((tree=fopen(phylip_name,"r"))==NULL) {
#endif
                error("Cannot open tree file [%s]",phylip_name);
                return;
        }
	fclose(tree);

	GetPanelExtra(seq_panel.seqs,&data);
	if (!open_aln_files(mult_aln)) return;

	WatchCursor();
	info("Doing alignments from guide tree...");
	if (Visible(talignw))
	{
		Remove(talignw);
		talignw=NULL;
	}
	strcpy(mult_aln.treename,phylip_name);
	align_from_tree(&mult_aln,opt,FALSE);
        if (save_parameters) create_parameter_output(opt,mult_aln);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}
/* reload the sequences from the output file (so that the sequence order is
correct - either INPUT or ALIGNED */
	reload_alignment(mult_aln.nseqs,mult_aln.seqs);
        load_aln(seq_panel,0,0,mult_aln.nseqs,FALSE);
	ArrowCursor();
	info("Done.");
}

static void PrfPrfAlignWin (IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);
        if (mult_aln.prf1.nseqs<=0)
        {
                error("Profile 1 not loaded");
                return;
        }
        if (mult_aln.nseqs==mult_aln.prf1.nseqs)
        {
                error("Profile 2 not loaded");
                return;
        }
	do_palign_window(&palignw,&ptree1text,&ptree2text,NEW,"Profile to Profile Alignment",PrfPrfAlign,mult_aln);
}

static void PrfPrfTreeAlignWin (IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);
        if (mult_aln.prf1.nseqs<=0)
        {
                error("Profile 1 not loaded");
                return;
        }
        if (mult_aln.nseqs==mult_aln.prf1.nseqs)
        {
                error("Profile 2 not loaded");
                return;
        }
	do_palign_window(&palignw,&ptree1text,&ptree2text,OLD,"Profile Alignment from Tree",PrfPrfTreeAlign,mult_aln);
}

static void SeqPrfAlignWin (IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);
        if (mult_aln.prf1.nseqs<=0)
        {
                error("Profile 1 not loaded");
                return;
        }
        if (mult_aln.nseqs==mult_aln.prf1.nseqs)
        {
                error("Profile 2 not loaded");
                return;
        }
	do_align_window(&salignw,&streetext,NEW,"Sequence to Profile Alignment",SeqPrfAlign,mult_aln);
}

static void SeqPrfTreeAlignWin (IteM item)
{
	panel_data data;

	GetPanelExtra(prf_panel[0].seqs,&data);

        if (mult_aln.prf1.nseqs<=0)
        {
                error("Profile 1 not loaded");
                return;
        }
        if (mult_aln.nseqs==mult_aln.prf1.nseqs)
        {
                error("Profile 2 not loaded");
                return;
        }
	do_align_window(&salignw,&streetext,OLD,"Sequence to Profile Alignment from Tree",SeqPrfTreeAlign,mult_aln);
}

static void PrfPrfAlign(ButtoN but)
{
	panel_data data;

	GetTitle(ptree1text, filename, FILENAMELEN);
	stripspace(filename);
       	strcpy(mult_aln.prf1.treename,filename);

	GetTitle(ptree2text, filename, FILENAMELEN);
	stripspace(filename);
       	strcpy(mult_aln.prf2.treename,filename);

	GetPanelExtra(prf_panel[0].seqs,&data);
	if (!open_aln_files(mult_aln)) return;

	WatchCursor();
	if (Visible(palignw))
	{
		Remove(palignw);
		palignw=NULL;
	}
        profile_align(&mult_aln,opt,FALSE,FALSE,FALSE);
        if (save_parameters) create_parameter_output(opt,mult_aln);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}
	load_aln(prf_panel[0],1,0,mult_aln.prf1.nseqs,FALSE);
	load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
	ArrowCursor();
}

static void PrfPrfTreeAlign(ButtoN but)
{
	panel_data data;

	GetTitle(ptree1text, filename, FILENAMELEN);
	stripspace(filename);
       	strcpy(mult_aln.prf1.treename,filename);

	GetTitle(ptree2text, filename, FILENAMELEN);
	stripspace(filename);
       	strcpy(mult_aln.prf2.treename,filename);

	GetPanelExtra(prf_panel[0].seqs,&data);
	if (!open_aln_files(mult_aln)) return;

	WatchCursor();
	if (Visible(palignw))
	{
		Remove(palignw);
		palignw=NULL;
	}
        profile_align(&mult_aln,opt,TRUE,TRUE,FALSE);
        if (save_parameters) create_parameter_output(opt,mult_aln);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}
	load_aln(prf_panel[0],1,0,mult_aln.prf1.nseqs,FALSE);
	load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
	ArrowCursor();
}

static void SeqPrfAlign(ButtoN but)
{
	char phylip_name[FILENAMELEN];
	panel_data data;

	GetTitle(streetext, filename, FILENAMELEN);
	stripspace(filename);

        strcpy(mult_aln.treename,filename);

	GetPanelExtra(prf_panel[0].seqs,&data);
	if (!open_aln_files(mult_aln)) return;

	WatchCursor();
	if (Visible(salignw))
	{
		Remove(salignw);
		salignw=NULL;
	}
        new_sequence_align(&mult_aln,opt,FALSE,FALSE);
        if (save_parameters) create_parameter_output(opt,mult_aln);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}

/* reload the sequences from the output file (so that the sequence order is
correct - either INPUT or ALIGNED */
	reload_alignment(mult_aln.nseqs,mult_aln.seqs);

	load_aln(prf_panel[0],1,0,mult_aln.prf1.nseqs,FALSE);
	load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
	ArrowCursor();
}

static void SeqPrfTreeAlign(ButtoN but)
{
	panel_data data;

	GetTitle(streetext, filename, FILENAMELEN);
	stripspace(filename);

        strcpy(mult_aln.treename,filename);

	GetPanelExtra(prf_panel[0].seqs,&data);
	if (!open_aln_files(mult_aln)) return;

	WatchCursor();
	if (Visible(salignw))
	{
		Remove(salignw);
		salignw=NULL;
	}
        new_sequence_align(&mult_aln,opt,TRUE,FALSE);
        if (save_parameters) create_parameter_output(opt,mult_aln);
	if(save_log && save_log_fd!=NULL)
	{
		fclose(save_log_fd);
		save_log_fd=NULL;
	}

/* reload the sequences from the output file (so that the sequence order is
correct - either INPUT or ALIGNED */
	reload_alignment(mult_aln.nseqs,mult_aln.seqs);

	load_aln(prf_panel[0],1,0,mult_aln.prf1.nseqs,FALSE);
	load_aln(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,FALSE);
	ArrowCursor();
}
static void reload_alignment(sint nseqs,SEQ *seqs)
{
	sint i,k,ix;
	sint     *sseqlen_array;
	char     **sseq_array;
	char     **snames, **stitles;

	if (nseqs==0) return;
	if ((*opt.alnout_opt).output_order == INPUT) return;


	snames=(char **)ckalloc((nseqs+2) * sizeof(char *));
	stitles=(char **)ckalloc((nseqs+2) * sizeof(char *));
	sseq_array=(char **)ckalloc((nseqs+2) * sizeof(char *));
	sseqlen_array=(sint *)ckalloc((nseqs+2) * sizeof(sint));
	for (i=0;i<nseqs;i++)
	{
		ix=seqs[i].output_index;
		snames[ix]=(char *)ckalloc((MAXNAMES+2)*sizeof(char));
		stitles[ix]=(char *)ckalloc((MAXTITLES+2)*sizeof(char));
		sseq_array[ix]=(char *)ckalloc((seqs[i].len+2)*sizeof(char));
		strcpy(snames[ix],seqs[i].name);
		strcpy(stitles[ix],seqs[i].title);
		sseqlen_array[ix]=seqs[i].len;
		for(k=0;k<seqs[ix].len;k++)
			sseq_array[ix][k]=seqs[i].data[k];
	}
	for (i=0;i<nseqs;i++)
	{
		strcpy(seqs[i].name,snames[i]);
		strcpy(seqs[i].title,stitles[i]);
		seqs[i].len=sseqlen_array[i];
		realloc_seq(&seqs[i],seqs[i].len);
		for(k=0;k<seqs[i].len;k++)
			seqs[i].data[k]=sseq_array[i][k];
		seqs[i].output_index=i;
	}

	ckfree(sseqlen_array);
	for(i=0;i<nseqs;i++)
		ckfree(sseq_array[i]);
	ckfree(sseq_array);
	for(i=0;i<nseqs;i++)
		ckfree(stitles[i]);
	ckfree(stitles);
	for(i=0;i<nseqs;i++)
		ckfree(snames[i]);
	ckfree(snames);
	ncutseqs=0;
}

static void SegmentWin(IteM item)
{
	WindoW w;
	GrouP maing;
	ButtoN closeb;
	GrouP mat_list;
	ButtoN matrixb[5];
	panel_data data;

	GetPanelExtra(active_panel.seqs,&data);

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	w=FixedWindow(-50, -33, -10, -10, "Low-Scoring Segments",RemoveWin);
	maing=HiddenGroup(w,0,0,NULL);
	SetGroupSpacing(maing,20,10);
	closeb=PushButton(maing, "CLOSE", CancelWin);
	Break(maing);

	PushButton(maing, "Calculate Low-Scoring Segments", calc_segment_exceptions);
	Break(maing);

	length_cutofftext=make_scale(maing,"Minimum Length of Segments:",9,length_cutoff,19,set_lengthcutoff);
	Break(maing);

	segmentdnascaletext=make_scale(maing,"DNA Marking Scale:",9,segment_dnascale,9,set_segment_dnascale);
	if(!mult_aln.dnaflag) Disable(segmentdnascaletext);
	Break(maing);

	show_seg_toggle=make_toggle(maing,"Hide Low-Scoring Segments","OFF","ON",
					&segment_exceptions,set_show_segments);
	Break(maing);

	mat_list=NormalGroup(maing,4,0,"Protein Weight Matrix",systemFont,set_segment_matrix);
	matrixb[0]=RadioButton(mat_list,"Gonnet PAM 80");
	matrixb[1]=RadioButton(mat_list,"Gonnet PAM 120");
	matrixb[2]=RadioButton(mat_list,"Gonnet PAM 250");
	matrixb[3]=RadioButton(mat_list,"Gonnet PAM 350");
	matrixb[4]=RadioButton(mat_list,"User defined");
	SetValue(mat_list,segment_matnum);
	seg_matrix_list=mat_list;
	Break(maing);
	PushButton(maing, "Load protein matrix: ", set_segment_user_matrix);
	Advance(maing);
	segmentmattext=StaticPrompt(maing,"", MAXPROMPTLEN, dialogTextHeight, systemFont, 'l');
	SetTitle(segmentmattext,segment_mtrxname);
	Break(maing);

	mat_list=NormalGroup(maing,4,0,"DNA Weight Matrix",systemFont,set_segment_dnamatrix);
	matrixb[0]=RadioButton(mat_list,"IUB");
	matrixb[1]=RadioButton(mat_list,"CLUSTALW(1.6)");
	matrixb[2]=RadioButton(mat_list,"User defined");
	SetValue(mat_list,segment_dnamatnum);
	seg_dnamatrix_list=matrix_list;
	Break(maing);
	PushButton(maing, "Load DNA matrix: ", set_segment_user_dnamatrix);
	Advance(maing);
	segmentdnamattext=StaticPrompt(maing,"", MAXPROMPTLEN, dialogTextHeight, systemFont, 'l');
	SetTitle(segmentdnamattext,segment_dnamtrxname);
	Break(maing);


	Show(w);
}

static void ScoreWin(IteM item)
{
	WindoW w;
	GrouP maing;
	ButtoN closeb;
	GrouP mat_list;
	PopuP show_exceptions;
	ButtoN matrixb[5];

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	w=FixedWindow(-50, -33, -10, -10, "Score Parameters",RemoveWin);
	maing=HiddenGroup(w,0,0,NULL);
	SetGroupSpacing(maing,20,10);
	closeb=PushButton(maing, "CLOSE", CancelWin);
	Break(maing);


/* add a scale to set the scaling value for the alignment scoring function */
	scorescaletext=make_scale(maing,"Score Plot Scale:",9,score_scale,9,set_scorescale);
	Break(maing);

	residue_cutofftext=make_scale(maing,"Residue Exception Cutoff:",9,score_cutoff,9,set_scorecutoff);
	Break(maing);

	mat_list=NormalGroup(maing,4,0,"Protein Weight Matrix",systemFont,set_score_matrix);
	matrixb[0]=RadioButton(mat_list,"Identity");
	matrixb[1]=RadioButton(mat_list,"Gonnet PAM 80");
	matrixb[2]=RadioButton(mat_list,"Gonnet PAM 120");
	matrixb[3]=RadioButton(mat_list,"Gonnet PAM 250");
	matrixb[4]=RadioButton(mat_list,"Gonnet PAM 350");
	matrixb[5]=RadioButton(mat_list,"User defined");
	SetValue(mat_list,score_matnum);
	score_matrix_list=mat_list;
	Break(maing);
	PushButton(maing, "Load protein matrix: ", set_score_user_matrix);
	Advance(maing);
	scoremattext=StaticPrompt(maing,"", MAXPROMPTLEN, dialogTextHeight, systemFont, 'l');
	SetTitle(scoremattext,score_mtrxname);
	Break(maing);

	mat_list=NormalGroup(maing,4,0,"DNA Weight Matrix",systemFont,set_score_dnamatrix);
	matrixb[0]=RadioButton(mat_list,"IUB");
	matrixb[1]=RadioButton(mat_list,"CLUSTALW(1.6)");
	matrixb[2]=RadioButton(mat_list,"User defined");
	SetValue(mat_list,score_dnamatnum);
	score_dnamatrix_list=mat_list;
	Break(maing);
	PushButton(maing, "Load DNA matrix: ", set_score_user_dnamatrix);
	Advance(maing);
	scorednamattext=StaticPrompt(maing,"", MAXPROMPTLEN, dialogTextHeight, systemFont, 'l');
	SetTitle(scorednamattext,score_dnamtrxname);
	Break(maing);

	Show (w);
}


static void MotifParameters(IteM item)
{
	int i;
	WindoW w;
	GrouP maing;
	ButtoN closeb;
	PopuP motif_toggle;
	char str[FILENAMELEN];
	panel_data data;

	GetPanelExtra(active_panel.seqs,&data);

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	w=FixedWindow(-50, -33, -10, -10, "Motif Parameters",RemoveWin);
	maing=HiddenGroup(w,0,0,NULL);
	SetGroupSpacing(maing,0,10);
	closeb=PushButton(maing, "CLOSE", CancelWin);
	Break(maing);

	PushButton(maing, "Load Motif file: ", set_motif_file);
	Advance(maing);
	motiftext=StaticPrompt(maing,"", FILENAMELEN, dialogTextHeight, systemFont, 'l');
	SetTitle(motiftext,opt.mult_opt->motif_filename);

	Break(maing);
	motif_toggle=make_toggle(maing,"Use Motif File","ON","OFF",&(*opt.mult_opt).use_motifs,set_use_motifs);
	Break(maing);

	Show (w);
}


static void PWParameters(IteM item)
{
	int i;
	WindoW w;
	PoinT pt;
	GrouP maing;
	ButtoN closeb;
	TexT go_scale,ge_scale;
	TexT gp_scale,ktuple_scale,topdiags_scale,window_scale;
	PopuP fs_toggle,algo_toggle;
	GrouP mat_list;
	ButtoN matrixb[5];
	char str[FILENAMELEN];
	sint ktup,window,wind_gap,signif;
	float    pw_go_penalty;
	float    pw_ge_penalty;
	panel_data data;

	GetPanelExtra(active_panel.seqs,&data);

        if(mult_aln.dnaflag) {
                pw_go_penalty     = opt.pw_opt->dna_go_penalty;
                pw_ge_penalty     = opt.pw_opt->dna_ge_penalty;
                ktup       = opt.quickpw_opt->dna_ktup;
                window     = opt.quickpw_opt->dna_window;
                signif     = opt.quickpw_opt->dna_signif;
                wind_gap   = opt.quickpw_opt->dna_wind_gap;
 
        }
        else {
                pw_go_penalty     = opt.pw_opt->prot_go_penalty;
                pw_ge_penalty     = opt.pw_opt->prot_ge_penalty;
                ktup       = opt.quickpw_opt->prot_ktup;
                window     = opt.quickpw_opt->prot_window;
                signif     = opt.quickpw_opt->prot_signif;
                wind_gap   = opt.quickpw_opt->prot_wind_gap;
 
        }

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	w=FixedWindow(-50, -33, -10, -10, "Pairwise Parameters",RemoveWin);
	maing=HiddenGroup(w,0,0,NULL);
	SetGroupSpacing(maing,0,10);
	closeb=PushButton(maing, "CLOSE", CancelWin);
	Break(maing);
       	make_prompt(maing, "Guide tree algorithm :");
	Advance(maing);
	algo_toggle=PopupList(maing,TRUE,set_treealgo);
	PopupItem(algo_toggle,"NJ");
	PopupItem(algo_toggle,"BIONJ");
	if ((*opt.pw_opt).treealgo == NJ)
		SetValue(algo_toggle,1);
	else if ((*opt.pw_opt).treealgo == BIONJ)
		SetValue(algo_toggle,2);
	Break(maing);
	fs_toggle=make_toggle(maing,"Pairwise Alignments :","Fast-Approximate","Slow-Accurate",&opt.pw_opt->quick_pairalign,set_fs_toggle);
	Break(maing);

	GetNextPosition(maing,&pt);
	slow_para=NormalGroup(maing,0,0,"Pairwise Parameters",systemFont,NULL);
	SetGroupSpacing(slow_para,0,10);

	make_prompt(slow_para, "Gap Opening [0-100] :");
	Advance(slow_para);
	sprintf(str,"%.2f",pw_go_penalty);
	go_scale=DialogText(slow_para, str, 5, set_pw_go_penalty);
	Break(slow_para);

	make_prompt(slow_para, "Gap Extension [0-100] :");
	Advance(slow_para);
	sprintf(str,"%.2f",pw_ge_penalty);
	ge_scale=DialogText(slow_para, str, 5, set_pw_ge_penalty);
	Break(slow_para);
	mat_list=NormalGroup(slow_para,4,0,"Protein Weight Matrix",systemFont,set_pw_matrix);
	for(i=0;i<pw_matrix_menu.noptions;i++)
		matrixb[i]=RadioButton(mat_list,pw_matrix_menu.opt[i].title);
	SetValue(mat_list,opt.pw_opt->matnum);
	pw_matrix_list=mat_list;
	Break(slow_para);
	PushButton(slow_para, "Load protein matrix: ", set_pw_user_matrix);
	Advance(slow_para);
	pwmattext=StaticPrompt(slow_para,"", MAXPROMPTLEN, dialogTextHeight, systemFont, 'l');
	SetTitle(pwmattext,opt.pw_opt->usermtrxname);

	Break(slow_para);
	mat_list=NormalGroup(slow_para,4,0,"DNA Weight Matrix",systemFont,set_pw_dnamatrix);
	for(i=0;i<dnamatrix_menu.noptions;i++)
		matrixb[i]=RadioButton(mat_list,dnamatrix_menu.opt[i].title);
	SetValue(mat_list,opt.pw_opt->dnamatnum);
	pw_dnamatrix_list=mat_list;
	Break(slow_para);
	PushButton(slow_para, "Load DNA matrix: ", set_pw_user_dnamatrix);
	Advance(slow_para);
	pwdnamattext=StaticPrompt(slow_para,"", MAXPROMPTLEN, dialogTextHeight, systemFont, 'l');
	SetTitle(pwdnamattext,opt.pw_opt->dnausermtrxname);

	Break(slow_para);


/* fast parameters */
	SetNextPosition(maing,pt);
	fast_para=NormalGroup(maing,2,0,"Pairwise Parameters",systemFont,NULL);
	SetGroupSpacing(fast_para,0,10);
	make_prompt(fast_para, "Gap Penalty [1-500]:");
	sprintf(str,"%d",wind_gap);
	gp_scale=DialogText(fast_para, str, 3, set_gp);
	make_prompt(fast_para, "K-Tuple Size [1-2]:");
	sprintf(str,"%d",ktup);
	ktuple_scale=DialogText(fast_para, str, 1, set_ktuple);
	make_prompt(fast_para, "Top Diagonals [1-50]:");
	sprintf(str,"%d",signif);
	topdiags_scale=DialogText(fast_para, str, 2, set_topdiags);
	make_prompt(fast_para, "Window Size [1-50]:");
	sprintf(str,"%d",window);
	window_scale=DialogText(fast_para, str, 2, set_window);

	if (opt.pw_opt->quick_pairalign)
	{
		Hide(slow_para);
		Show(fast_para);
	}
	else
	{
		Hide(fast_para);
		Show(slow_para);
	}

	Break(maing);

	Show (w);
}


static void MultiParameters(IteM item)
{
	int i;
	WindoW w;
	GrouP maing;
	ButtoN closeb;
	TexT go_scale,ge_scale;
	GrouP mat_list;
	ButtoN matrixb[5];
	GrouP multi_para;
	TexT div_seq;
	TexT transitions;
	PopuP neg_mat_toggle;
	char str[FILENAMELEN];
	float	gap_open;
	float	gap_extend;
	panel_data data;

	GetPanelExtra(active_panel.seqs,&data);

        if(mult_aln.dnaflag) {
                gap_open   = (*opt.mult_opt).dna_gap_open;
                gap_extend = (*opt.mult_opt).dna_gap_extend;
 
        }
        else {
                gap_open   = (*opt.mult_opt).prot_gap_open;
                gap_extend = (*opt.mult_opt).prot_gap_extend;
        }

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	w=FixedWindow(-50, -33, -10, -10, "Alignment Parameters",RemoveWin);
	maing=HiddenGroup(w,0,0,NULL);
	SetGroupSpacing(maing,0,10);
	closeb=PushButton(maing, "CLOSE", CancelWin);
	Break(maing);
 
/* multiple alignment parameters */

	multi_para=NormalGroup(maing,0,0,"Multiple Parameters",systemFont,NULL);
	SetGroupSpacing(multi_para,0,10);
        make_prompt(multi_para, "Gap Opening [0-100] :");
	Advance(multi_para);
	sprintf(str,"%.2f",gap_open);
        go_scale=DialogText(multi_para, str, 5, set_go_penalty);
	Advance(multi_para);
        make_prompt(multi_para, "Gap Extention [0-100] :");
	Advance(multi_para);
	sprintf(str,"%.2f",gap_extend);
        ge_scale=DialogText(multi_para, str, 5, set_ge_penalty);
	Break(multi_para);
        make_prompt(multi_para, "Delay Divergent Sequences (%) :");
	Advance(multi_para);
	sprintf(str,"%d",(*opt.mult_opt).divergence_cutoff);
        div_seq=DialogText(multi_para, str, 3, set_div_seq);
	Break(multi_para);
 
        make_prompt(multi_para, "DNA Transition Weight [0-1] :");
        Advance(multi_para);
        sprintf(str,"%.2f",(*opt.mult_opt).transition_weight);
        transitions=DialogText(multi_para, str, 5, set_transitions);
        Break(multi_para);


	neg_mat_toggle=make_toggle(multi_para,"Use Negative Matrix","ON","OFF",&(*opt.mult_opt).neg_matrix,set_neg_matrix);

	Break(multi_para);
        mat_list=NormalGroup(multi_para,2,0,"Protein Weight Matrix",systemFont,set_matrix);
	for(i=0;i<matrix_menu.noptions;i++)
		matrixb[i]=RadioButton(mat_list,matrix_menu.opt[i].title);
        SetValue(mat_list,(*opt.mult_opt).matnum);
	matrix_list=mat_list;
	Break(multi_para);
	PushButton(multi_para, "Load protein matrix: ", set_user_matrix);
	Advance(multi_para);
	mattext=StaticPrompt(multi_para,"", MAXPROMPTLEN, dialogTextHeight, systemFont, 'l');
	SetTitle(mattext,(*opt.mult_opt).usermtrxname);

	Break(multi_para);
        mat_list=NormalGroup(multi_para,2,0,"DNA Weight Matrix",systemFont,set_dnamatrix);
	for(i=0;i<dnamatrix_menu.noptions;i++)
		matrixb[i]=RadioButton(mat_list,dnamatrix_menu.opt[i].title);
        SetValue(mat_list,(*opt.mult_opt).dnamatnum);
	dnamatrix_list=mat_list;
	Break(multi_para);
	PushButton(multi_para, "Load DNA: ", set_user_dnamatrix);
	Advance(multi_para);
	dnamattext=StaticPrompt(multi_para,"", MAXPROMPTLEN, dialogTextHeight, systemFont, 'l');
	SetTitle(dnamattext,(*opt.mult_opt).dnausermtrxname);
	Show (w);
}

static void GapParameters(IteM item)
{
	WindoW gapparaw;
	GrouP maing;
	ButtoN closeb;
	PopuP rp_toggle,vp_toggle,hp_toggle,end_gap_toggle;
	TexT gdist,hyd_text;
	char str[80];
 
	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	gapparaw=FixedWindow(-50, -33, -10, -10, "Protein Gap Parameters",RemoveWin);
	maing=HiddenGroup(gapparaw,0,0,NULL);
	SetGroupSpacing(maing,0,10);
	closeb=PushButton(maing, "CLOSE", CancelWin);
	Break(maing);
	rp_toggle=make_toggle(maing,"Residue-specific Penalties","OFF","ON",&(*opt.mult_opt->gap_opt).no_pref_penalties,set_pref_penalties);
	Break(maing);
	hp_toggle=make_toggle(maing,"Hydrophilic Penalties","OFF","ON",&(*opt.mult_opt->gap_opt).no_hyd_penalties,set_hyd_penalties);
	Break(maing);
        make_prompt(maing, "Hydrophilic Residues :");
	Advance(maing);
        hyd_text=DialogText(maing, (*opt.mult_opt->gap_opt).hyd_residues, 20, set_hyd_res);
	Break(maing);
	vp_toggle=make_toggle(maing,"Variable Zone Penalties","OFF","ON",&(*opt.mult_opt->gap_opt).no_var_penalties,set_var_penalties);
	Break(maing);
        make_prompt(maing, "Gap Separation Distance [0-100] :");
	Advance(maing);
	sprintf(str,"%d",(*opt.mult_opt->gap_opt).gap_dist);
        gdist=DialogText(maing, str, 3, set_gap_dist);
	Break(maing);
	end_gap_toggle=make_toggle(maing,"End Gap Separation","ON","OFF",&(*opt.mult_opt->gap_opt).use_endgaps,set_endgaps);

	Show (gapparaw);
}

static void SSParameters(IteM item)
{
	WindoW ssparaw;
	GrouP maing;
	ButtoN closeb;
	PopuP use_p1,use_p2;
	TexT helix_gp,strand_gp,loop_gp,terminal_gp,helix_minus,helix_plus;
	TexT strand_minus,strand_plus;
	GrouP output_list;
	ButtoN outputb[4]; 
	char str[80];
 
	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	ssparaw=FixedWindow(-50, -33, -10, -10, "Secondary Structure Options",RemoveWin);
	closeb=PushButton(ssparaw, "CLOSE", CancelWin);
	Break(ssparaw);

	use_p1=make_toggle(ssparaw,"Use profile 1 secondary structure / penalty mask","YES","NO",&(*opt.ss_opt).use_ss1,set_use_ss1);
	Break(ssparaw);
	use_p2=make_toggle(ssparaw,"Use profile 2 secondary structure / penalty mask","YES","NO",&(*opt.ss_opt).use_ss2,set_use_ss2);

	Break(ssparaw);
        output_list=NormalGroup(ssparaw,2,0,"Output ",systemFont,NULL)
;
        outputb[0]=CheckBox(output_list,"Secondary Structure",set_ss_output);
	if((*opt.ss_opt).output_struct_penalties==0 || (*opt.ss_opt).output_struct_penalties==2) 
		output_ss=TRUE;
	else
		output_ss=FALSE;
	SetStatus(outputb[0],output_ss);
        outputb[1]=CheckBox(output_list,"Gap Penalty Mask",set_gp_output);
	if((*opt.ss_opt).output_struct_penalties==1 || (*opt.ss_opt).output_struct_penalties==2) 
		output_gp=TRUE;
	else
		output_gp=FALSE;
	SetStatus(outputb[1],output_gp);

	Break(ssparaw);
	maing=HiddenGroup(ssparaw,2,0,NULL);
	SetGroupSpacing(maing,0,10);
        make_prompt(maing, "Helix Gap Penalty [0-9] :");
	sprintf(str,"%d",(*opt.ss_opt).helix_penalty);
        helix_gp=DialogText(maing, str, 1, set_helix_gp);
        make_prompt(maing, "Strand Gap Penalty [0-9] :");
	sprintf(str,"%d",(*opt.ss_opt).strand_penalty);
        strand_gp=DialogText(maing, str, 1, set_strand_gp);
        make_prompt(maing, "Loop Gap Penalty [0-9] :");
	sprintf(str,"%d",(*opt.ss_opt).loop_penalty);
        loop_gp=DialogText(maing, str, 1, set_loop_gp);
        make_prompt(maing, "Secondary Structure Terminal Penalty [0-9] :");
	sprintf(str,"%d",(*opt.ss_opt).helix_end_penalty);
        terminal_gp=DialogText(maing, str, 1, set_terminal_gp);
        make_prompt(maing, "Helix Terminal Positions [0-3]            within:");
	sprintf(str,"%d",(*opt.ss_opt).helix_end_minus);
        helix_minus=DialogText(maing, str, 1, set_helix_minus);
        make_prompt(maing, "outside:");
	sprintf(str,"%d",(*opt.ss_opt).helix_end_plus);
        helix_plus=DialogText(maing, str, 1, set_helix_plus);
        make_prompt(maing, "Strand Terminal Penalty [0-3]             within:");
	sprintf(str,"%d",(*opt.ss_opt).strand_end_minus);
        strand_minus=DialogText(maing, str, 1, set_strand_minus);
        make_prompt(maing, "outside:");
	sprintf(str,"%d",(*opt.ss_opt).strand_end_plus);
        strand_plus=DialogText(maing, str, 1, set_strand_plus);


	Show (ssparaw);
}

static void OutputParameters(IteM item)
{
	WindoW outputparaw;
	GrouP maing;
	ButtoN closeb;
	GrouP output_list;
	ButtoN outputb[6];
	PopuP order_toggle,para_toggle,dnd_toggle;
	PopuP case_toggle,snos_toggle;

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	outputparaw=FixedWindow(-50, -33, -10, -10, "Output Format Options",RemoveWin);
	closeb=PushButton(outputparaw, "CLOSE", CancelWin);
	Break(outputparaw);
        output_list=NormalGroup(outputparaw,2,0,"Output Files",systemFont,NULL);

	SelectFont(systemFont);
        outputb[0]=CheckBox(output_list,"CLUSTAL format",set_output_clustal);
	if((*opt.alnout_opt).output_clustal) SetStatus(outputb[0],TRUE);

        outputb[1]=CheckBox(output_list,"NBRF/PIR format",set_output_nbrf);
	if((*opt.alnout_opt).output_nbrf) SetStatus(outputb[1],TRUE);

        outputb[2]=CheckBox(output_list,"GCG/MSF format",set_output_gcg);
	if((*opt.alnout_opt).output_gcg) SetStatus(outputb[2],TRUE);

        outputb[3]=CheckBox(output_list,"PHYLIP format",set_output_phylip);
	if((*opt.alnout_opt).output_phylip) SetStatus(outputb[3],TRUE);

        outputb[4]=CheckBox(output_list,"GDE format",set_output_gde);
	if((*opt.alnout_opt).output_gde) SetStatus(outputb[4],TRUE);

        outputb[5]=CheckBox(output_list,"NEXUS format",set_output_nexus);
	if((*opt.alnout_opt).output_nexus) SetStatus(outputb[5],TRUE);

	maing=HiddenGroup(outputparaw,2,0,NULL);
	SetGroupSpacing(maing,0,10);


	case_toggle=make_toggle(maing,"GDE output case :","Lower","Upper",&(*opt.alnout_opt).lowercase,set_case);
	snos_toggle=make_toggle(maing,"CLUSTALW sequence numbers :","ON","OFF",&(*opt.alnout_opt).seq_numbers,set_snos);
       	make_prompt(maing, "Output order");
	order_toggle=PopupList(maing,TRUE,set_output_order);
	PopupItem(order_toggle,"INPUT");
	PopupItem(order_toggle,"ALIGNED");
	if ((*opt.alnout_opt).output_order == INPUT)
		SetValue(order_toggle,1);
	else if ((*opt.alnout_opt).output_order == ALIGNED)
		SetValue(order_toggle,2);
	para_toggle=make_toggle(maing,"Parameter output","ON","OFF",&save_parameters,set_save_paras);

	Show (outputparaw);
}


static void OutputTreeParameters(IteM item)
{
	WindoW outputtreeparaw;
	ButtoN closeb;
	GrouP output_list;
	ButtoN outputb[5];
	PopuP boot_format;

	SelectFont(systemFont);
	stdCharWidth=CharWidth('A');
	stdLineHeight=LineHeight();
	outputtreeparaw=FixedWindow(-50, -33, -10, -10, "Output Tree Format Options",RemoveWin);
	closeb=PushButton(outputtreeparaw, "CLOSE", CancelWin);
	Break(outputtreeparaw);
        output_list=NormalGroup(outputtreeparaw,2,0,"Output Files",systemFont,NULL);
        outputb[0]=CheckBox(output_list,"CLUSTAL format tree",set_output_tree_clustal);
	if((*opt.tree_opt).output_tree_clustal) SetStatus(outputb[0],TRUE);
        outputb[1]=CheckBox(output_list,"Phylip format tree",set_output_tree_phylip);
	if((*opt.tree_opt).output_tree_phylip) SetStatus(outputb[1],TRUE);
        outputb[2]=CheckBox(output_list,"Phylip distance matrix",set_output_tree_distances);
	if((*opt.tree_opt).output_tree_distances) SetStatus(outputb[2],TRUE);
        outputb[3]=CheckBox(output_list,"BioNJ format tree",set_output_tree_bionj);
	if((*opt.tree_opt).output_tree_bionj) SetStatus(outputb[3],TRUE);
        outputb[4]=CheckBox(output_list,"Nexus format tree",set_output_tree_nexus);
	if((*opt.tree_opt).output_tree_nexus) SetStatus(outputb[4],TRUE);

	Break(outputtreeparaw);
       	make_prompt(outputtreeparaw, "Bootstrap labels on:");
	Advance(outputtreeparaw);
	boot_format=PopupList(outputtreeparaw,TRUE,set_boot_format);
	PopupItem(boot_format ,"NODE");
	PopupItem(boot_format ,"BRANCH");
	if ((*opt.bstree_opt).bootstrap_format == BS_NODE_LABELS)
		SetValue(boot_format ,1);
	if ((*opt.bstree_opt).bootstrap_format == BS_BRANCH_LABELS)
		SetValue(boot_format ,2);
	Show (outputtreeparaw);
}

static PrompT make_prompt(GrouP g,CharPtr title)
{
	PrompT p=NULL;

	if (title != NULL)
        	p=StaticPrompt(g, title, 0, dialogTextHeight, systemFont, 'l');

	return p;
}

static PrompT make_scale(GrouP g,CharPtr title,int length, int value,int max,BarScrlProc SetProc)
{
	char str[FILENAMELEN];
	BaR scale;
	PrompT t;

	sprintf(str,"%s   %3d",title,value);
	t=make_prompt(g,str);
	Advance(g);
        scale=ScrollBar(g, length, -1, SetProc);
       	CorrectBarPage(scale,1,1);
        CorrectBarMax(scale,max);
        CorrectBarValue(scale,value);
	return t;
}

static PopuP make_toggle(GrouP g,CharPtr title,CharPtr true_text, CharPtr false_text, Boolean *value,PupActnProc SetProc)
{
	PopuP p;

	if (title != NULL)
        	make_prompt(g, title);
	Advance(g);
	p=PopupList(g,TRUE,SetProc);
	PopupItem(p,true_text);
	PopupItem(p,false_text);
	if (*value)
		SetValue(p,1);
	else
		SetValue(p,2);
	return p;
}


static void make_menu_headers(WindoW w)
{
        filem = PulldownMenu (w,"File");
        editm = PulldownMenu (w,"Edit");
        alignm = PulldownMenu (w,"Alignment");
        motifm = PulldownMenu (w,"Motifs");
	treem = PulldownMenu(w,"Trees");
	colorm = PulldownMenu(w,"Colors");
	scorem = PulldownMenu(w,"Quality");
	helpmenu = PulldownMenu(w,"Help");

}

static void make_file_menu(void)
{
	int n=0;

	file_item.mode[n] = MULTIPLEM;
       	file_item.i[n] = CommandItem (filem,"Load Sequences", OpenSeqFile); n++;

	file_item.mode[n] = MULTIPLEM;
       	file_item.i[n] = CommandItem (filem,"Append Sequences", AppendSeqFile); n++;
	file_item.mode[n] = MULTIPLEM;
       	file_item.i[n] = CommandItem (filem,"Save Sequences as...", SaveSeqFileWin); n++;
	file_item.mode[n] = PROFILEM;
       	file_item.i[n] = CommandItem (filem,"Load Profile 1", OpenPrf1File); n++;
	file_item.mode[n] = PROFILEM;
       	file_item.i[n] = CommandItem (filem,"Load Profile 2", OpenPrf2File); n++;
	file_item.mode[n] = PROFILEM;
       	file_item.i[n] = CommandItem (filem,"Save Profile 1 as...", SavePrf1FileWin); n++;
	file_item.mode[n] = PROFILEM;
       	file_item.i[n] = CommandItem (filem,"Save Profile 2 as...", SavePrf2FileWin); n++;
	file_item.mode[n] = MULTIPLEM;
       	file_item.i[n] = CommandItem (filem,"Write Alignment as PostScript", SavePSSeqWin); n++;
	file_item.mode[n] = PROFILEM;
       	file_item.i[n] = CommandItem (filem,"Write Profile 1 as PostScript", SavePSPrf1Win); n++;
	file_item.mode[n] = PROFILEM;
       	file_item.i[n] = CommandItem (filem,"Write Profile 2 as PostScript", SavePSPrf2Win); n++;
       	file_item.i[n] = CommandItem (filem,"Quit", QuitWinI); n++;
	file_item.num = n;
	if(aln_mode==MULTIPLEM)
		check_menus(file_item,PROFILEM);
	else
		check_menus(file_item,MULTIPLEM);
}

static void QuitWinI (IteM i)
{
	if(aln_mode == MULTIPLEM)
	{
		if(seq_panel.modified)
			if (Message(MSG_YN,"Alignment has not been saved.\n"
			"Quit program anyway?")==ANS_NO) return;
	}
	else if(aln_mode == PROFILEM)
	{
		if(prf_panel[0].modified)
			if (Message(MSG_YN,"Profile 1 has not been saved.\n"
			"Quit program anyway?")==ANS_NO) return;
		if(prf_panel[1].modified)
			if (Message(MSG_YN,"Profile 2 has not been saved.\n"
			"Quit program anyway?")==ANS_NO) return;
	}
	QuitProgram ();
}

static void make_score_menu(void)
{
	int n=0;

       	score_item.i[n] = CommandItem (scorem,"Low-Scoring Segments",SegmentWin); n++;
	score_item.i[n] = CommandItem (scorem,"Score Parameters",ScoreWin); n++;
	score_item.i[n]=StatusItem(scorem, "Show Exceptional Residues", set_residue_exceptions);
        SetStatus(score_item.i[n],residue_exceptions); n++;
	score_item.mode[n] = MULTIPLEM;
	score_item.i[n]=CommandItem(scorem, "Save Quality Scores to File", SaveScoresWin); n++;


	score_item.num = n;
}

static void make_help_menu(void)
{
	int n=0;

	help_item.ptr[n] = 'G';
       	help_item.i[n] = CommandItem (helpmenu,"General",HelpProc); n++;
	help_item.ptr[n] = 'F';
       	help_item.i[n] = CommandItem (helpmenu,"Input & Output Files",HelpProc); n++;
	help_item.ptr[n] = 'E';
       	help_item.i[n] = CommandItem (helpmenu,"Editing Alignments",HelpProc); n++;
	help_item.ptr[n] = 'M';
       	help_item.i[n] = CommandItem (helpmenu,"Multiple Alignments",HelpProc); n++;
	help_item.ptr[n] = 'P';
       	help_item.i[n] = CommandItem (helpmenu,"Profile Alignments",HelpProc); n++;
	help_item.ptr[n] = 'B';
       	help_item.i[n] = CommandItem (helpmenu,"Secondary Structures",HelpProc); n++;
	help_item.ptr[n] = 'T';
       	help_item.i[n] = CommandItem (helpmenu,"Trees",HelpProc); n++;
	help_item.ptr[n] = 'C';
       	help_item.i[n] = CommandItem (helpmenu,"Colors",HelpProc); n++;
	help_item.ptr[n] = 'Q';
       	help_item.i[n] = CommandItem (helpmenu,"Alignment Quality",HelpProc); n++;
	help_item.ptr[n] = '9';
       	help_item.i[n] = CommandItem (helpmenu,"Command Line Parameters",HelpProc); n++;
	help_item.ptr[n] = 'R';
       	help_item.i[n] = CommandItem (helpmenu,"References",HelpProc); n++;

	help_item.num = n;
}

static void HelpProc(IteM item)
{
	int n,hindex=(-1);
	FILE *fd;
	int  i;
	Boolean found_help;
	char temp[MAXLINE+1];
	char token;
	char *digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	char *help_marker    = ">>HELP";

	TexT htext;
	char *help_file = NULL;
	char *help_name;

	helptext[0]='\0';

	for(n=0;n<help_item.num;n++)
		if (item==help_item.i[n])
		{
			hindex=n;
			break;
		}

	if (hindex==-1)
	{
		error("Problem with HELP routines\n");
		return;
	}

	help_name=get_help_file_name();
	help_file=find_file(help_name);
	ckfree(help_name);

	if(help_file==NULL) {
            error("Cannot find help file");
            return;
        }
	
        if((fd=fopen(help_file,"r"))==NULL) {
            error("Cannot open help file [%s]",help_file);
            return;
        }
	found_help = FALSE;

	while(TRUE) {
		if(fgets(temp,MAXLINE+1,fd) == NULL) {
			if(!found_help)
				error("No help found in help file");
			fclose(fd);
			return;
		}
		if(strstr(temp,help_marker)) {
                        token = ' ';
			for(i=strlen(help_marker); i<8; i++)
				if(strchr(digits, temp[i])) {
					token = temp[i];
					break;
				}
			if(token == help_item.ptr[hindex]) {
				found_help = TRUE;
				while(fgets(temp,MAXLINE+1,fd)) {
					if(strstr(temp, help_marker)) break;
					if(strlen(helptext)+strlen(temp) >MAXHELPLENGTH)
						break;
						for(i=strlen(temp)-1;i>=0;i--)
							if(iscntrl(temp[i])||isspace(temp[i]))
								temp[i]='\0';
							else break;
/* ignore lines starting with < - these are for html output processing */
					if(temp[0]!='<') {
			       			strcat(helptext,temp);
#ifdef WIN_MAC
			       			strcat(helptext,"\r");
#else
#ifdef WIN32
			       			strcat(helptext,"\r\n");
#else
			       			strcat(helptext,"\n");
#endif
#endif
					}
				}
				fclose(fd);
#ifdef WIN_MAC
				if(numhelp>=1)
#else
				if(numhelp>=MAXHELPW)
#endif
				{
					error("Too many help windows");
					return;
				}
				numhelp++;
 
				helpw[numhelp]=FixedWindow(-50, -33, -10, -10, "", QuitHelpW);
				SelectFont(helpfont);
#ifdef WIN_MAC
				htext=ScrollText(helpw[numhelp], 50, 20, helpfont, TRUE, NULL);
#else
				htext=ScrollText(helpw[numhelp], 60, 30, helpfont, TRUE, NULL);
#endif
				Break(helpw[numhelp]);
				PushButton(helpw[numhelp], "OK", QuitHelpB);
				SetTitle(htext, helptext);
				Show(helpw[numhelp]);
				return;
			}
		}
	}
}

void QuitHelpB(ButtoN b)
{
 
	Remove(ParentWindow(b));
	numhelp--;
}
 
void QuitHelpW(WindoW w)
{
 
	Remove(w);
	numhelp--;
}
static void make_motif_menu(void)
{
	int n=0;

	motif_item.mode[n] = MULTIPLEM;
       	motif_item.i[n] = CommandItem (motifm,"Make Ballast Motif File", MakeBallastMotifWin); n++;
	motif_item.mode[n] = MULTIPLEM;
       	motif_item.i[n] = CommandItem (motifm,"Motif Parameters",MotifParameters); n++;
}

static void make_edit_menu(void)
{
	int n=0;

       	edit_item.i[n] = CommandItem (editm,"Cut Sequences", CutSequences); n++;
       	edit_item.i[n] = CommandItem (editm,"Paste Sequences", PasteSequences); n++;
	edit_item.mode[n] = MULTIPLEM;
       	edit_item.i[n] = CommandItem (editm,"Select All Sequences", SelectSeqs); n++;
	edit_item.mode[n] = PROFILEM;
       	edit_item.i[n] = CommandItem (editm,"Select Profile 1", SelectPrf1); n++;
	edit_item.mode[n] = PROFILEM;
       	edit_item.i[n] = CommandItem (editm,"Select Profile 2", SelectPrf2); n++;
	edit_item.mode[n] = PROFILEM;
       	edit_item.i[n] = CommandItem (editm,"Add Profile 2 to Profile 1", MergeProfiles); n++;
       	edit_item.i[n] = CommandItem (editm,"Clear Sequence Selection",ClearSeqs); n++;
       	edit_item.i[n] = CommandItem (editm,"Clear Range Selection",ClearSeqRange); n++;
	SeparatorItem(editm);
       	edit_item.i[n] = CommandItem (editm,"Search", SearchStrWin); n++;
	SeparatorItem(editm);
       	edit_item.i[n] = CommandItem (editm,"Remove All Gaps", RemoveGaps); n++;
       	edit_item.i[n] = CommandItem (editm,"Remove Gap-Only Columns", RemoveGapPos); n++;
	edit_item.num = n;
	if(aln_mode==MULTIPLEM)
		check_menus(edit_item,PROFILEM);
	else
		check_menus(edit_item,MULTIPLEM);
}

static void make_align_menu(void)
{
	MenU parasm;
	int n=0;

	align_item.mode[n] = MULTIPLEM;
       	align_item.i[n] = CommandItem (alignm,"Do Complete Aligment",CAlignWin); n++;
	align_item.mode[n] = MULTIPLEM;
       	align_item.i[n] = CommandItem (alignm,"Produce Guide Tree Only",SaveTreeWin); n++;
	align_item.mode[n] = MULTIPLEM;
       	align_item.i[n] = CommandItem (alignm,"Do Alignment from Guide Tree",AlignFromTreeWin); n++;
	SeparatorItem(alignm);
	align_item.mode[n] = MULTIPLEM;
       	align_item.i[n] = CommandItem (alignm,"Realign Selected Sequences",RealignSeqsWin); n++;
	align_item.mode[n] = MULTIPLEM;
       	align_item.i[n] = CommandItem (alignm,"Realign Selected Residue Range",RealignSeqRangeWin); n++;
	align_item.mode[n] = PROFILEM;
       	align_item.i[n] = CommandItem (alignm,"Align Profile 2 to Profile 1",PrfPrfAlignWin); n++;
	align_item.mode[n] = PROFILEM;
       	align_item.i[n] = CommandItem (alignm,"Align Profiles from Guide Trees",PrfPrfTreeAlignWin); n++;
	align_item.mode[n] = PROFILEM;
       	align_item.i[n] = CommandItem (alignm,"Align Sequences to Profile 1",SeqPrfAlignWin); n++;
	align_item.mode[n] = PROFILEM;
       	align_item.i[n] = CommandItem (alignm,"Align Sequences to Profile 1 from Tree",SeqPrfTreeAlignWin); n++;

       	SeparatorItem(alignm);
	parasm=SubMenu(alignm,"Alignment Parameters");
	new_gaps_item=align_item.i[n]=StatusItem(parasm, "Reset New Gaps before Alignment", set_reset_new_gaps);
	SetStatus(align_item.i[n],(*opt.mult_opt).reset_alignments_new); n++;
	all_gaps_item=align_item.i[n]=StatusItem(parasm, "Reset All Gaps before Alignment", set_reset_all_gaps);
	SetStatus(align_item.i[n],(*opt.mult_opt).reset_alignments_all); n++;
       	align_item.i[n] = CommandItem (parasm,"Pairwise Alignment Parameters",PWParameters); n++;
       	align_item.i[n] = CommandItem (parasm,"Multiple Alignment Parameters",MultiParameters); n++;
       	align_item.i[n] = CommandItem (parasm,"Protein Gap Parameters",GapParameters); n++;
	align_item.mode[n] = PROFILEM;
       	align_item.i[n] = CommandItem (parasm,"Secondary Structure Parameters",SSParameters); n++;
	align_item.i[n]=StatusItem(alignm, "Save Log File", set_save_log);
	save_item1=align_item.i[n];
	SetStatus(save_item1,save_log); n++;
       	align_item.i[n] = CommandItem (alignm,"Output Format Options",OutputParameters); n++;
	align_item.num = n;
	if(aln_mode==MULTIPLEM)
		check_menus(align_item,PROFILEM);
	else
		check_menus(align_item,MULTIPLEM);

}

void set_save_log(IteM i)
{
        save_log=GetStatus(i);
	SetStatus(save_item1,save_log);
	SetStatus(save_item2,save_log);
}

static void make_tree_menu(void)
{
	int n=0;

	tree_item.mode[n] = MULTIPLEM;
        tree_item.i[n] = CommandItem (treem,"Draw N-J Tree",DrawTreeWin); n++;
	tree_item.mode[n] = MULTIPLEM;
        tree_item.i[n] = CommandItem (treem,"Bootstrap N-J Tree",BootstrapTreeWin); n++;
        SeparatorItem(treem);
	tree_item.mode[n] = MULTIPLEM;
	tree_item.i[n]=StatusItem(treem, "Exclude Positions with Gaps", set_tossgaps);
	SetStatus(tree_item.i[n],(*opt.tree_opt).tossgaps); n++;
	tree_item.mode[n] = MULTIPLEM;
	tree_item.i[n]=StatusItem(treem, "Correct for Multiple Substitutions", set_kimura);
	SetStatus(tree_item.i[n],(*opt.tree_opt).kimura); n++;
        SeparatorItem(treem);
	tree_item.mode[n] = MULTIPLEM;
	tree_item.i[n]=StatusItem(treem, "Save Log File", set_save_log);
	save_item2=tree_item.i[n];
	SetStatus(save_item2,save_log); n++;
	tree_item.mode[n] = MULTIPLEM;
        tree_item.i[n] = CommandItem (treem,"Output Format Options",OutputTreeParameters); n++;
	tree_item.mode[n] = MULTIPLEM;
	tree_item.num = n;
	if(aln_mode==MULTIPLEM)
		check_menus(tree_item,PROFILEM);
	else
		check_menus(tree_item,MULTIPLEM);

}

static void make_color_menu(void)
{
	int n=0;

        color_item.i[n]=StatusItem(colorm, "Background Coloring", set_inverted);
        SetStatus(color_item.i[n],inverted); n++;
       	SeparatorItem(colorm);
        bw_item=color_item.i[n] = StatusItem (colorm,"Black and White",BlackandWhite);
        SetStatus(color_item.i[n],usebw); n++;
        defcol_item=color_item.i[n] = StatusItem (colorm,"Default Colors",DefColorPar);
        SetStatus(color_item.i[n],usedefcolors); n++;
        usercol_item=color_item.i[n] = StatusItem (colorm,"Load Color Parameter File",OpenColorParWin);
        SetStatus(color_item.i[n],useusercolors); n++;
	color_item.num = n;

}

void check_menus(menu_item m,int mode)
{
	int i;

	for (i=0;i<m.num;i++)
		if (m.mode[i] == mode)
			Disable(m.i[i]);
		else
			Enable(m.i[i]);
}

void switch_mode(void)
{
	char path[FILENAMELEN];

	if(aln_mode==MULTIPLEM)
	{
		Hide(prf1_display);
		Hide(prf2_display);
		Hide(pscrolltext);
		SetValue(modetext,1);
		resize_multi_window();
		check_menus(file_item,PROFILEM);
		check_menus(align_item,PROFILEM);
		check_menus(edit_item,PROFILEM);
		check_menus(tree_item,PROFILEM);
		check_menus(color_item,PROFILEM);
		active_panel=seq_panel;
		fix_gaps(&mult_aln);
		get_path(mult_aln.prf1.filename,path);
		strcpy(mult_aln.filename,path);
		load_aln_data(seq_panel,0,0,mult_aln.nseqs,TRUE,*opt.ss_opt);
		Show(seq_display);
	}
	else if(aln_mode==PROFILEM)
	{
		Hide(seq_display);
		resize_prf_window(mult_aln.nseqs,0);
		SetValue(modetext,2);

		check_menus(file_item,MULTIPLEM);
		check_menus(align_item,MULTIPLEM);
		check_menus(edit_item,MULTIPLEM);
		check_menus(tree_item,MULTIPLEM);
		check_menus(color_item,MULTIPLEM);
		active_panel=prf_panel[0];
		mult_aln.prf1.nseqs=mult_aln.nseqs;
		get_path(mult_aln.filename,path);
		strcpy(mult_aln.prf1.filename,path);
		strcat(mult_aln.prf1.filename,"1.");
		strcpy(mult_aln.prf2.filename,path);
		strcat(mult_aln.prf2.filename,"2.");
		fix_gaps(&mult_aln);
		load_aln_data(prf_panel[0],1,0,mult_aln.prf1.nseqs,TRUE,*opt.ss_opt);
		load_aln_data(prf_panel[1],2,mult_aln.prf1.nseqs,mult_aln.prf2.nseqs,TRUE,*opt.ss_opt);
		Show(prf1_display);
		Show(prf2_display);
		Show(pscrolltext);
	}
		
}

void load_aln(spanel p,int prf_no,int fs,int nseqs,Boolean reset)
{
        load_aln_data(p,prf_no,fs,nseqs,reset,*opt.ss_opt);

        DrawPanel(p.names);
        DrawPanel(p.seqs);
        correct_name_bars(reset);
        correct_seq_bars(reset);

}


